/*****************************************************************************
				PROCEDIMENTS D'ALTA
*****************************************************************************/

create or replace PROCEDURE altaAmbit (
   n_idvotacio     IN       NUMBER,
   n_idcodicens    IN       NUMBER,
   n_idtipuscens   IN       NUMBER,
   rst             OUT      VARCHAR2
)
IS   
/******************************************************************************
   altaAmbit:  Dona d'alta un cens en una votacio.
                
   PARAMETRES:  n_idvotacio, n_idcodicens, n_idtipuscens, rst
   INPUT:       n_idvotacio, n_idcodicens, n_idtipuscens
   OUTPUT:      rst
   RETORNA:     'OK' si s'ha executat correctament, 'ERROR: Tipus d'Error' en
                cas contrari
   EXEMPLE D'US:     altaAmbit(1,1,1, rst)
******************************************************************************/
   n_idvotacio_ins            NUMBER;
   n_idcodicens_ins           NUMBER;
   n_idtipuscens_ins          NUMBER;
   n_actiu_ins                NUMBER;
   err_msg                    VARCHAR2 (100);
   existe                     VARCHAR2(1);
   campsNuls                  EXCEPTION;
   CensInexistent             EXCEPTION;
   censBaixa                  EXCEPTION;   
   tipusCensInexistent        EXCEPTION;
   votacioInexistent          EXCEPTION;
   ambitDuplicat              EXCEPTION;
  
BEGIN
   n_idvotacio_ins    := n_idvotacio;
   n_idcodicens_ins   := n_idcodicens;
   n_idtipuscens_ins  := n_idtipusCens;
      

  --verifiquem que el codi de la votacio, el codi del cens i tipus de cens no siguin nuls
  
  IF (n_idvotacio_ins IS NULL) OR (n_idcodicens_ins IS NULL) OR (n_idtipuscens_ins IS NULL) THEN 
    RAISE campsNuls;
  END IF; 
  
  
  --verifiquem que la votacio no s'hagi donat d'alta previament en aquest cens.
  BEGIN
       SELECT 'S'
         INTO EXISTE
         FROM AMBIT
         WHERE idvotacio        = n_idvotacio
         AND   codicens         = n_idcodicens
         AND   idtipuscens      = n_idtipuscens
         AND ROWNUM               = 1;

        RAISE ambitDuplicat;
     EXCEPTION
        WHEN NO_DATA_FOUND THEN
           NULL;
     END;
  -- Verifiquem que la votacio existeixi
  
  BEGIN
       SELECT 'S'
         INTO EXISTE
         FROM VOTACIO
         WHERE idvotacio          = n_idvotacio
         AND ROWNUM               = 1;
                        
     EXCEPTION
       WHEN NO_DATA_FOUND THEN
          RAISE votacioInexistent;
     END;
  -- Verifiquem que el tipus de cens existeix i que no s'hagi donat de baixa
  BEGIN
  SELECT 'S'
         INTO EXISTE
         FROM TIPUSCENS
         WHERE idtipuscens        = n_idtipuscens
         AND ROWNUM               = 1;
         
     EXCEPTION
       WHEN NO_DATA_FOUND THEN
          RAISE tipusCensInexistent;
     END;
      
  -- Verifiquem que existeix el cens
  IF (n_idtipuscens=1) THEN
  IF (n_idcodicens!=1)THEN
  RAISE censinexistent;
  END IF;
  END IF;
  
  IF (n_idtipuscens=2) THEN   
   BEGIN
       SELECT 'S'
         INTO EXISTE
         FROM CENSESTATAL
         WHERE idpais             = n_idcodicens
         AND ROWNUM               = 1;   
         
      SELECT actiu
         INTO n_actiu_ins
         FROM CENSESTATAL
         WHERE idpais             = n_idcodicens;
      
      IF(n_actiu_ins=0) THEN RAISE censBaixa;
      END IF;                           
     EXCEPTION
       WHEN NO_DATA_FOUND THEN
          RAISE censInexistent;
     END;
     END IF;
     
    IF (n_idtipuscens=3) THEN   
   BEGIN
       SELECT 'S'
         INTO EXISTE
         FROM CENSREGIONAL
         WHERE idregio            = n_idcodicens
         AND ROWNUM               = 1;         
       
        SELECT actiu
         INTO n_actiu_ins
         FROM CENSREGIONAL
         WHERE idregio             = n_idcodicens;
      
      IF(n_actiu_ins=0) THEN RAISE censBaixa;
      END IF;
     EXCEPTION
       WHEN NO_DATA_FOUND THEN
          RAISE censInexistent;
     END;
     END IF;
    
     IF (n_idtipuscens=4) THEN   
   BEGIN
       SELECT 'S'
         INTO EXISTE
         FROM CENSLOCAL
         WHERE idlocal            = n_idcodicens
         AND ROWNUM               = 1;   
       
        SELECT actiu
         INTO n_actiu_ins
         FROM CENSLOCAL
         WHERE idlocal             = n_idcodicens;
      
      IF(n_actiu_ins=0) THEN RAISE censBaixa;
      END IF;
     EXCEPTION
       WHEN NO_DATA_FOUND THEN
          RAISE censInexistent;
     END;
     END IF;

    
   -- afegim el nou mbit
   INSERT INTO AMBIT(idvotacio, codicens, idtipuscens) VALUES ( n_idvotacio_ins, n_idcodicens_ins, n_idtipuscens);
   
   --Actualitzem les dades estadstiques de la estadstica 3
   
   ActualitzaE3(n_idvotacio_ins, n_idtipuscens_ins, n_idcodicens_ins, rst);
  
   -- si ha finalitzat correctament retorna OK, en cas contrari gestionem els errors
   rst := 'OK';

    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'AltaAmbit',
          'idVotacio: ' || n_idvotacio||', idcodicens:  ' || n_idcodicens || ' ,idtipuscens: ' || n_idtipuscens, 
          'RSP: '||rst);   
   COMMIT;
EXCEPTION
   WHEN ambitDuplicat
   THEN
   ROLLBACK;
      rst := 'ERROR: Aquest cens ja ha estat assignat a aquesta votacio';
      
    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'AltaAmbit',
          'idVotacio: ' || n_idvotacio||', idcodicens:  ' || n_idcodicens || ' ,idtipuscens: ' || n_idtipuscens, 
          'RSP: '||rst);   
      COMMIT;       
      
   WHEN censInexistent
   THEN
   ROLLBACK;
      rst := 'ERROR: El cens no existeix';
   
    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'AltaAmbit',
          'idVotacio: ' || n_idvotacio||', idcodicens:  ' || n_idcodicens || ' ,idtipuscens: ' || n_idtipuscens, 
          'RSP: '||rst);   
      COMMIT;       
  
  WHEN censBaixa
   THEN
   ROLLBACK;
      rst := 'ERROR: El cens est donat de baixa';
   
    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'AltaAmbit',
          'idVotacio: ' || n_idvotacio||', idcodicens:  ' || n_idcodicens || ' ,idtipuscens: ' || n_idtipuscens, 
          'RSP: '||rst);   
      COMMIT;       
   
   WHEN tipusCensInexistent
   THEN
   ROLLBACK;
      rst := 'ERROR: El tipus de cens no existeix';
   
    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'AltaAmbit',
          'idVotacio: ' || n_idvotacio||', idcodicens:  ' || n_idcodicens || ' ,idtipuscens: ' || n_idtipuscens, 
          'RSP: '||rst);   
      COMMIT;        
   
   WHEN votacioInexistent
   THEN
   ROLLBACK;
      rst := 'ERROR: La votaci no existeix';
   
    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'AltaAmbit',
          'idVotacio: ' || n_idvotacio||', idcodicens:  ' || n_idcodicens || ' ,idtipuscens: ' || n_idtipuscens, 
          'RSP: '||rst);    
      COMMIT;       
      
   WHEN campsNuls
   THEN
   ROLLBACK;
      rst := 'ERROR: el codi de la votaci, el codi del cens i el tipus de cens no poden ser nuls';
    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'AltaAmbit',
          'idVotacio: ' || n_idvotacio||', idcodicens:  ' || n_idcodicens || ' ,idtipuscens: ' || n_idtipuscens, 
          'RSP: '||rst);   
   
      COMMIT;
   WHEN OTHERS
   THEN
   ROLLBACK;
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:'
             || err_msg;
   
     INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'AltaAmbit',
          'idVotacio: ' || n_idvotacio||', idcodicens:  ' || n_idcodicens || ' ,idtipuscens: ' || n_idtipuscens, 
          'RSP: '||rst);   
      COMMIT;
END altaAmbit;
/
-- -----------------------------------------------------------------------------------------------------------------

create or replace PROCEDURE altaCensCiutada (
   n_idciutada     IN       NUMBER,
   n_idcodicens    IN       NUMBER,
   n_idtipuscens   IN       NUMBER,
   rst             OUT      VARCHAR2
)
IS   
/******************************************************************************
   altaCensCiutada:  Dona d'alta un ciutad en un cens.
                
   PARAMETRES:  n_idciutada, n_idcodicens, n_idtipuscens, rst
   INPUT:       n_idciutada, n_idcodicens, n_idtipuscens
   OUTPUT:      rst
   RETORNA:     'OK' si s'ha executat correctament, 'ERROR: Tipus d'Error' en
                cas contrari
   EXEMPLE D'US:     altaCensCiutada(1,1,1, rst)
******************************************************************************/
   n_idciutada_ins            NUMBER;
   n_idcodicens_ins           NUMBER;
   n_idtipuscens_ins          NUMBER;
   n_actiu_ins                NUMBER;
   err_msg                    VARCHAR2 (100);
   existe                     VARCHAR2(1);
   campsNuls                  EXCEPTION;
   CensInexistent             EXCEPTION;
   censBaixa                  EXCEPTION;
   CiutadaInexistent          EXCEPTION;
   ciutadaBaixa               EXCEPTION;
   censCiutadaDuplicat        EXCEPTION;
   tipusCensInexistent        EXCEPTION;
  
BEGIN
   n_idciutada_ins    := n_idciutada;
   n_idcodicens_ins   := n_idcodicens;
   n_idtipuscens_ins  := n_idtipusCens;
      

  --verifiquem que el codi del ciutada, el codi del cens i tipus de cens no siguin nuls
  
  IF (n_idciutada_ins IS NULL) OR (n_idcodicens_ins IS NULL) OR (n_idtipuscens_ins IS NULL) THEN 
    RAISE campsNuls;
  END IF; 
  
  
 
  -- Verifiquem que el ciutad existeixi i que estigui actiu
  
  BEGIN
       SELECT 'S'
         INTO EXISTE
         FROM CIUTADA
         WHERE idciutada          = n_idciutada
         AND ROWNUM               = 1;
      
      SELECT actiu
         INTO n_actiu_ins
         FROM CIUTADA
         WHERE idciutada          = n_idciutada;
      
      IF (n_actiu_ins=0) THEN RAISE ciutadaBaixa;
      END IF;
                        
     EXCEPTION
       WHEN NO_DATA_FOUND THEN
          RAISE ciutadaInexistent;
     END;
     
  -- Verifiquem que el tipus de cens existeix
  BEGIN
  SELECT 'S'
         INTO EXISTE
         FROM TIPUSCENS
         WHERE idtipuscens        = n_idtipuscens
         AND ROWNUM               = 1;
                     
     EXCEPTION
       WHEN NO_DATA_FOUND THEN
          RAISE tipusCensInexistent;
     END;
     
  -- Verifiquem que existeix el cens
  
  IF (n_idtipuscens=2) THEN   
   BEGIN
       SELECT 'S'
         INTO EXISTE
         FROM CENSESTATAL
         WHERE idpais             = n_idcodicens
         AND ROWNUM               = 1;
         
       SELECT actiu
         INTO n_actiu_ins
         FROM CENSESTATAL
         WHERE idpais             = n_idcodicens;
      
      IF(n_actiu_ins=0) THEN RAISE censBaixa;
      END IF;                           
                
     EXCEPTION
       WHEN NO_DATA_FOUND THEN
          RAISE censInexistent;
     END;
     END IF;
     
    IF (n_idtipuscens=3) THEN   
   BEGIN
       SELECT 'S'
         INTO EXISTE
         FROM CENSREGIONAL
         WHERE idregio            = n_idcodicens
         AND ROWNUM               = 1;
         
       SELECT actiu
         INTO n_actiu_ins
         FROM CENSREGIONAL
         WHERE idregio             = n_idcodicens;
      
      IF(n_actiu_ins=0) THEN RAISE censBaixa;
      END IF;
       
     EXCEPTION
       WHEN NO_DATA_FOUND THEN
          RAISE censInexistent;
     END;
     END IF;
    
     IF (n_idtipuscens=4) THEN   
   BEGIN
       SELECT 'S'
         INTO EXISTE
         FROM CENSLOCAL
         WHERE idlocal            = n_idcodicens
         AND ROWNUM               = 1; 
      
       SELECT actiu
         INTO n_actiu_ins
         FROM CENSLOCAL
         WHERE idlocal             = n_idcodicens;
      
      IF(n_actiu_ins=0) THEN RAISE censBaixa;
      END IF;
           
     EXCEPTION
       WHEN NO_DATA_FOUND THEN
          RAISE censInexistent;
     END;
     END IF;
     
      --verifiquem que el ciutada no s'hagi donat d'alta previament en aquest cens.
  BEGIN
       SELECT 'S'
         INTO EXISTE
         FROM CENSCIUTADANS
         WHERE idciutada        = n_idciutada
         AND   idcodicens       = n_idcodicens
         AND   idtipuscens      = n_idtipuscens
         AND ROWNUM               = 1;

        RAISE censCiutadaDuplicat;
     EXCEPTION
        WHEN NO_DATA_FOUND THEN
           NULL;
     END;

    
   -- afegim el nou Ciutada al cens
   INSERT INTO CENSCIUTADANS(idciutada, idcodicens, idtipuscens) VALUES ( n_idciutada_ins, n_idcodicens_ins, n_idtipuscens);
   
   -- si ha finalitzat correctament retorna OK, en cas contrari gestionem els errors
   rst := 'OK';

    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'AltaCensCiutada',
          'idCiutada: ' || n_idciutada||', idcodicens:  ' || n_idcodicens || ' ,idtipuscens: ' || n_idtipuscens, 
          'RSP: '||rst);   
   COMMIT;
EXCEPTION
   WHEN censCiutadaDuplicat
   THEN
   ROLLBACK;
      rst := 'ERROR: Aquest ciutada ja ha estat assignat a aquest cens';
      
    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'AltaCensCiutada',
          'idCiutada: ' || n_idciutada||', idcodicens:  ' || n_idcodicens || ' ,idtipuscens: ' || n_idtipuscens, 
          'RSP: '||rst);   
      COMMIT;       
      
   WHEN censInexistent
   THEN
   ROLLBACK;
      rst := 'ERROR: El cens no existeix';
   
    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'AltaCensCiutada',
          'idCiutada: ' || n_idciutada||', idcodicens:  ' || n_idcodicens || ' ,idtipuscens: ' || n_idtipuscens, 
          'RSP: '||rst);   
      COMMIT;  
      
  WHEN censBaixa
   THEN
   ROLLBACK;
      rst := 'ERROR: El cens est inactiu';
   
    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'AltaCensCiutada',
          'idCiutada: ' || n_idciutada||', idcodicens:  ' || n_idcodicens || ' ,idtipuscens: ' || n_idtipuscens, 
          'RSP: '||rst);   
      COMMIT;  
      
   WHEN tipusCensInexistent
   THEN
   ROLLBACK;
      rst := 'ERROR: El tipus de cens no existeix';
   
    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'AltaCensCiutada',
          'idCiutada: ' || n_idciutada||', idcodicens:  ' || n_idcodicens || ' ,idtipuscens: ' || n_idtipuscens, 
          'RSP: '||rst);   
      COMMIT;       
   
   WHEN ciutadaInexistent
   THEN
   ROLLBACK;
      rst := 'ERROR: El ciutada no existeix';
   
    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'AltaCensCiutada',
          'idCiutada: ' || n_idciutada||', idcodicens:  ' || n_idcodicens || ' ,idtipuscens: ' || n_idtipuscens, 
          'RSP: '||rst);   
      COMMIT;  
      
   WHEN ciutadaBaixa
   THEN
   ROLLBACK;
      rst := 'ERROR: El ciutada Est inactiu';
   
    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'AltaCensCiutada',
          'idCiutada: ' || n_idciutada||', idcodicens:  ' || n_idcodicens || ' ,idtipuscens: ' || n_idtipuscens, 
          'RSP: '||rst);   
      COMMIT;  
      
   WHEN campsNuls
   THEN
   ROLLBACK;
      rst := 'ERROR: el codi del ciutada, el codi del cens i el tipus de cens no poden ser nuls';
      
  INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
  VALUES (SYSDATE,'AltaCensCiutada',
          'idCiutada: ' || n_idciutada||', idcodicens:  ' || n_idcodicens || ' ,idtipuscens: ' || n_idtipuscens, 
          'RSP: '||rst); 
   
      COMMIT;
   WHEN OTHERS
   THEN
   ROLLBACK;
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:'
             || err_msg;
   
     INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
     VALUES (SYSDATE,'AltaCensCiutada',
          'idCiutada: ' || n_idciutada||', idcodicens:  ' || n_idcodicens || ' ,idtipuscens: ' || n_idtipuscens, 
          'RSP: '||rst);   
      COMMIT;
END altaCensCiutada;
/
-- ----------------------------------------------------------------------------------------------------------------

create or replace PROCEDURE altaCensEstatal (
   n_codipais       IN       VARCHAR2,
   n_nompais        IN       VARCHAR2,
   rst              OUT      VARCHAR2
)
IS   
/******************************************************************************
   altaCensEstatal:   Gestiona l'alta d'un cens Estatal al sistema
                
   PARAMETRES:  n_codipais, n_nompais, rst
   INPUT:       n_codipais, n_nompais
   OUTPUT:      rst
   RETORNA:     'OK' si s'ha executat correctament, 'ERROR: Tipus d'Error' en
                cas contrari
   EXEMPLE D'US:     altaCensEstatal('ES','ESPANYA' rst)
******************************************************************************/
   n_codipais_ins             VARCHAR2 (2);
   n_nompais_ins              VARCHAR2 (50);
   err_msg                    VARCHAR2 (100);
   existe                     VARCHAR2(1);
   paramE                     VARCHAR2(500);
   campsNuls                  EXCEPTION;
   paisDuplicat               EXCEPTION;            
BEGIN
   n_codipais_ins :=n_codipais;
   n_nompais_ins :=n_nompais;
   

  --verifiquem que el codi del pas i el seu nom no siguin nuls
  IF (n_codipais IS NULL) OR (n_nompais_ins IS NULL)  THEN 
    RAISE campsNuls;
  END IF; 
  
  --verifiquem que el pas no s'hagi donat d'alta previament.
  BEGIN
       SELECT 'S'
         INTO EXISTE
         FROM CensEstatal
         WHERE codipais           = n_codipais
         AND ROWNUM               = 1;

        RAISE paisDuplicat;
     EXCEPTION
        WHEN NO_DATA_FOUND THEN
           NULL;
     END;

   -- afegim el nou CensEstatal
   INSERT INTO CENSESTATAL(idtipuscens, codipais, nom) VALUES (2, n_codipais_ins, n_nompais_ins);
   
   -- si ha finalitzt correctament retorna OK, en cas contrari gestionem els errors
   rst := 'OK';

   INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'AltaCensEstatal',
          'codiPais: ' || n_codipais||', nomPais:  ' || n_nompais,
          'RSP: '||rst);
   
   COMMIT;
EXCEPTION
   WHEN paisDuplicat
   THEN
      rst := 'ERROR: Aquest pas ja ha estat donat d alta al cens';
   
    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'AltaCensEstatal',
          'codiPais: ' || n_codipais||', nomPais:  ' || n_nompais,
          'RSP: '||rst);
   
      COMMIT;
   WHEN campsNuls
   THEN
      rst := 'ERROR: el nom i el codi del Pas no poden ser nuls';
      
    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'AltaCensEstatal',
          'codiPais: ' || n_codipais||', nomPais:  ' || n_nompais,
          'RSP: '||rst);
   
      COMMIT;
   WHEN OTHERS
   THEN
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:'
             || err_msg;
   
    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'AltaCensEstatal',
           'codiPais: ' || n_codipais||', nomPais:  ' || n_nompais,
           'RSP: '||rst);
      COMMIT;
END altaCensEstatal;
/
-- ------------------------------------------------------------------------------------------------

create or replace PROCEDURE altaCensLocal (
   n_idpais          IN       VARCHAR2,
   n_idregio         IN       NUMBER,
   n_nomlocalitat    IN       VARCHAR2,
   rst               OUT      VARCHAR2
)
IS   
/******************************************************************************
   altaCensLocal:   Gestiona l'alta d'un cens Local al sistema
                
   PARAMETRES:  n_idpais, n_idregio, n_nomlocalitat rst
   INPUT:       n_idpais, n_idregio, n_nomlocalitat
   OUTPUT:      rst
   RETORNA:     'OK' si s'ha executat correctament, 'ERROR: Tipus d'Error' en
                cas contrari
   EXEMPLE D'US:     altaCensLocal('ES',1, 'Girona' rst)
******************************************************************************/
   n_idpais_ins                VARCHAR2 (2);
   n_idregio_ins               NUMBER;
   n_nomlocalitat_ins          VARCHAR2(50);
   err_msg                     VARCHAR2 (100);
   existe                      VARCHAR2(1);
   paramE                      VARCHAR2(500);
   campsNuls                   EXCEPTION;
   localitatDuplicada          EXCEPTION;  
   paisInexistent              EXCEPTION;
   regioInexistent             EXCEPTION;
BEGIN
   n_idpais_ins   :=n_idpais;
   n_idregio_ins  :=n_idregio;
   n_nomlocalitat_ins :=n_nomlocalitat;
   

  --verifiquem que el codi del pais el codi de la regio i el nom de la localitat no siguin nuls
  IF (n_idpais IS NULL) OR (n_idregio_ins IS NULL) OR (n_nomlocalitat_ins IS NULL)  THEN 
    RAISE campsNuls;
  END IF; 
  
  --verifiquem que la localitat no s'hagi donat d'alta previament.
  BEGIN
       SELECT 'S'
         INTO EXISTE
         FROM CENSLOCAL
         WHERE idpais             = n_idpais
         AND   idregio            = n_idregio
         AND   nom                = n_nomlocalitat
         AND ROWNUM               = 1;

        RAISE localitatDuplicada;
     EXCEPTION
        WHEN NO_DATA_FOUND THEN
           NULL;
     END;
     
   -- verifiquem que el pais s'hagi donat d'alta al sistema
   
   BEGIN
       SELECT 'S'
         INTO EXISTE
         FROM CENSESTATAL
         WHERE codipais           = n_idpais
         AND ROWNUM               = 1;
        
     EXCEPTION
       WHEN NO_DATA_FOUND THEN
          RAISE paisInexistent;
     END;
     
   -- verifiquem que la regio s'hagi donat d'alta al sistema
   
   BEGIN
       SELECT 'S'
         INTO EXISTE
         FROM CENSREGIONAL
         WHERE idregio            = n_idregio
         AND ROWNUM               = 1;
        
     EXCEPTION
       WHEN NO_DATA_FOUND THEN
          RAISE regioInexistent;
     END;
     
   -- afegim el nou CensLocal
   INSERT INTO CENSLOCAL(idtipuscens, idpais,idregio, nom) VALUES (4, n_idpais_ins, n_idregio_ins, n_nomlocalitat_ins);
   
   -- si ha finalitzt correctament retorna OK, en cas contrari gestionem els errors
   rst := 'OK';

    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'AltaCensLocal',
          'codiPais: ' || n_idpais||', nomRegio:  ' || n_idregio, -- ' , nomLocalitat: ' || n_nomlocalitat ,
          'RSP: '||rst);
   
   COMMIT;
EXCEPTION
   WHEN localitatDuplicada
   THEN
      rst := 'ERROR: Aquesta localitat ja ha estat donada d alta al cens';
   
    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'AltaCensLocal',
          'codiPais: ' || n_idpais||', nomRegio:  ' || n_idregio ||' , nomLocalitat: ' || n_nomlocalitat,
          'RSP: '||rst);
   
      COMMIT;
            
   WHEN paisInexistent
   THEN
      rst := 'ERROR: El Pas no ha estat donat d alta al sistema';
   
   INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'AltaCensLocal',
          'codiPais: ' || n_idpais||', nomRegio:  ' || n_idregio || ' , nomLocalitat: ' || n_nomlocalitat,
          'RSP: '||rst);
   
      COMMIT;
      
   WHEN regioInexistent
   THEN
      rst := 'ERROR: La Regi no ha estat donat d alta al sistema';
   
    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'AltaCensLocal',
          'codiPais: ' || n_idpais||', nomRegio:  ' || n_idregio || ' , nomLocalitat: ' || n_nomlocalitat,
          'RSP: '||rst);
   
      COMMIT;
  
   WHEN campsNuls
   THEN
      rst := 'ERROR: el nom de la localitat, el codi de la regio i el codi del Pas no poden ser nuls';
      
    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'AltaCensLocal',
          'codiPais: ' || n_idpais||', nomRegio:  ' || n_idregio || ' , nomLocalitat: ' || n_nomlocalitat,
          'RSP: '||rst);
   
      COMMIT;
   WHEN OTHERS
   THEN
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:'
             || err_msg;
   
    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'AltaCensLocal',
          'codiPais: ' || n_idpais||', nomRegio:  ' || n_idregio || ' , nomLocalitat: ' || n_nomlocalitat,
          'RSP: '||rst);
      COMMIT;
END altaCensLocal;
/
-- ---------------------------------------------------------------------------------------------------------

create or replace PROCEDURE altaCensRegional (
   n_idpais          IN       VARCHAR2,
   n_nomregio        IN       VARCHAR2,
   rst               OUT      VARCHAR2
)
IS   
/******************************************************************************
   altaCensRegional:   Gestiona l'alta d'un cens Regional al sistema
                
   PARAMETRES:  n_idpais, n_nomregio, rst
   INPUT:       n_codipais, n_nompais
   OUTPUT:      rst
   RETORNA:     'OK' si s'ha executat correctament, 'ERROR: Tipus d'Error' en
                cas contrari
   EXEMPLE D'US:     altaCensregional('ES','Catalunya' rst)
******************************************************************************/
   n_idpais_ins                VARCHAR2 (2);
   n_nomregio_ins              VARCHAR2 (50);
   err_msg                     VARCHAR2 (100);
   existe                      VARCHAR2(1);
   paramE                      VARCHAR2(500);
   campsNuls                   EXCEPTION;
   regioDuplicada              EXCEPTION;  
   paisInexistent              EXCEPTION;
BEGIN
   n_idpais_ins   :=n_idpais;
   n_nomregio_ins :=n_nomregio;
   

  --verifiquem que el codi del pais i el nom de la regio no siguin nuls
  IF (n_idpais IS NULL) OR (n_nomregio_ins IS NULL)  THEN 
    RAISE campsNuls;
  END IF; 
  
  --verifiquem que la regio no s'hagi donat d'alta previament.
  BEGIN
       SELECT 'S'
         INTO EXISTE
         FROM CENSREGIONAL
         WHERE idpais             = n_idpais
         AND nom                  = n_nomregio
         AND ROWNUM               = 1;

        RAISE regioDuplicada;
     EXCEPTION
        WHEN NO_DATA_FOUND THEN
           NULL;
     END;
     
   -- verifiquem que el pais s'hagi donat d'alta al sistema
   
   BEGIN
       SELECT 'S'
         INTO EXISTE
         FROM CENSESTATAL
         WHERE codipais             = n_idpais
         AND ROWNUM               = 1;
        
     EXCEPTION
       WHEN NO_DATA_FOUND THEN
          RAISE paisInexistent;
     END;

   -- afegim el nou CensRegional
   INSERT INTO CENSREGIONAL(idtipuscens, idpais, nom) VALUES (3, n_idpais_ins, n_nomregio_ins);
   
   -- si ha finalitzt correctament retorna OK, en cas contrari gestionem els errors
   rst := 'OK';

   INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'AltaCensRegional',
          'codiPais: ' || n_idpais||', nomRegio:  ' || n_nomregio,
          'RSP: '||rst);
   
   COMMIT;
EXCEPTION
   WHEN regioDuplicada
   THEN
      rst := 'ERROR: Aquesta Regi ja ha estat donada d alta al cens';
   
   INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'AltaCensRegional',
          'codiPais: ' || n_idpais||', nomRegio:  ' || n_nomregio,
          'RSP: '||rst);
   
      COMMIT;
   WHEN paisInexistent
   THEN
      rst := 'ERROR: El Pas no ha estat donat d alta al sistema';
   
   INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'AltaCensRegional',
          'codiPais: ' || n_idpais||', nomRegio:  ' || n_nomregio,
          'RSP: '||rst);
   
      COMMIT;
   WHEN campsNuls
   THEN
      rst := 'ERROR: el nom de la regio i el codi del Pas no poden ser nuls';
      
    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'AltaCensRegional',
          'codiPais: ' || n_idpais||', nomRegio:  ' || n_nomregio,
          'RSP: '||rst);
   
      COMMIT;
   WHEN OTHERS
   THEN
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:'
             || err_msg;
   
    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'AltaCensRegional',
          'codiPais: ' || n_idpais||', nomRegio:  ' || n_nomregio,
          'RSP: '||rst);
      COMMIT;
END altaCensRegional;
/
-- ----------------------------------------------------------------------------------------------------

create or replace PROCEDURE altaCiutada (
   n_codiciutada     IN       VARCHAR2,
   n_datanaixement   IN       DATE,
   rst               OUT      VARCHAR2
)
IS   
/******************************************************************************
   altaCiutada:  Gestiona l'alta d'un ciutada al sistema
                
   PARAMETRES:  n_codiciutada, n_datanaixement,rst
   INPUT:       n_codiciutada, n_datanaixement
   OUTPUT:      rst
   RETORNA:     'OK' si s'ha executat correctament, 'ERROR: Tipus d'Error' en
                cas contrari
   EXEMPLE D'US:     altaCiutada('AMH658564BS','27/01/1998' rst)
******************************************************************************/
   n_codiciutada_ins           VARCHAR2 (11);
   n_datanaixement_ins         DATE;
   err_msg                     VARCHAR2 (100);
   existe                      VARCHAR2(1);
   campsNuls                   EXCEPTION;
   ciutadaDuplicat             EXCEPTION;  
   ciutadaMenorEdat            EXCEPTION;
  
BEGIN
   n_codiciutada_ins   :=n_codiciutada;
   n_datanaixement_ins  :=n_datanaixement;
      

  --verifiquem que el codi del ciutada  i la data de naixement no siguin nuls
  IF (n_codiciutada_ins IS NULL) OR (n_datanaixement_ins IS NULL)  THEN 
    RAISE campsNuls;
  END IF; 
  
  -- verifiquem que el ciutada s major d'edat i per tant pot votar
  
  IF ((MONTHS_BETWEEN(sysdate, n_datanaixement_ins)/12)<18) THEN
  RAISE ciutadaMenorEdat;
  END IF;
  
  --verifiquem que el ciutada no s'hagi donat d'alta previament.
  BEGIN
       SELECT 'S'
         INTO EXISTE
         FROM CIUTADA
         WHERE codiciutada        = n_codiciutada
         AND ROWNUM               = 1;

        RAISE ciutadaDuplicat;
     EXCEPTION
        WHEN NO_DATA_FOUND THEN
           NULL;
     END;
    
   -- afegim el nou Ciutada
   
   INSERT INTO CIUTADA(datanaixement, codiciutada) VALUES ( n_datanaixement_ins, n_codiciutada_ins);
   
   -- si ha finalitzat correctament retorna OK, en cas contrari gestionem els errors
   rst := 'OK';

    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'AltaCiutada',
          'codiCiutada: ' || n_codiciutada||', dataNaixement:  ' ||to_char( n_datanaixement),
          'RSP: '||rst);   
   COMMIT;
EXCEPTION
   WHEN ciutadaDuplicat
   THEN
      rst := 'ERROR: Aquest ciutada ja ha estat donat d alta al sistema';
   
    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'AltaCiutada',
          'codiCiutada: ' || n_codiciutada||', dataNaixement:  ' ||to_char( n_datanaixement),
          'RSP: '||rst);   
      COMMIT;       
      
     WHEN ciutadaMenorEdat
   THEN
      rst := 'ERROR: Aquest Ciutada s menor d edat i no es pot donar d alta al sistema';
   
    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'AltaCiutada',
          'codiCiutada: ' || n_codiciutada||', dataNaixement:  ' ||to_char( n_datanaixement),
          'RSP: '||rst);   
      COMMIT;       
      
   WHEN campsNuls
   THEN
      rst := 'ERROR: el codi del ciutada i la data de naixement no poden ser nuls';
      
   INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'AltaCiutada',
          'codiCiutada: ' || n_codiciutada||', dataNaixement:  ' ||to_char( n_datanaixement),
          'RSP: '||rst);     
      COMMIT;
   WHEN OTHERS
   THEN
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:'
             || err_msg;
   
    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'AltaCiutada',
          'codiCiutada: ' || n_codiciutada||', dataNaixement:  ' ||to_char( n_datanaixement),
          'RSP: '||rst);   
      COMMIT;
END altaCiutada;
/
-- ----------------------------------------------------------------------------------------------------

create or replace PROCEDURE altaOpcioVot (
   
   n_idvotacio       IN       NUMBER,
   n_numopcio        IN       NUMBER,
   n_descripcio      IN       VARCHAR2,
   rst               OUT      VARCHAR2
)
IS   
/******************************************************************************
   altaOpcioVot:  Gestiona l'alta d'una opcio de vot en una votaci.
                
   PARAMETRES:  n_numvotacio, n_idvotacio, n_descripcio,rst
   INPUT:       n_numvotacio, n_idvotacio, n_descripcio
   OUTPUT:      rst
   RETORNA:     'OK' si s'ha executat correctament, 'ERROR: Tipus d'Error' en
                cas contrari
   EXEMPLE D'US:     altaOpcioVot(1,1,'descripcio opcio' rst)
******************************************************************************/
   n_idvotacio_ins             NUMBER;
   n_numopcio_ins              NUMBER;
   n_descripcio_ins            VARCHAR2 (100);
   err_msg                     VARCHAR2 (100);
   existe                      VARCHAR2(1);
   campsNuls                   EXCEPTION;
   votacioInexistent           EXCEPTION;
   opcioDuplicada              EXCEPTION;
  
BEGIN
   n_numopcio_ins    :=n_numopcio;
   n_idvotacio_ins   :=n_idvotacio;
   n_descripcio_ins  :=n_descripcio;
      

  --verifiquem que el codi de la votacio, el numero de la opci  i la descripcio no siguin nuls
  IF (n_numopcio IS NULL) OR (n_idvotacio_ins IS NULL) OR (n_descripcio_ins IS NULL)  THEN 
    RAISE campsNuls;
  END IF; 
  
  --verifiquem que la votacio s'hagi donat d alta al sistema.
  BEGIN
       SELECT 'S'
         INTO EXISTE
         FROM VOTACIO
         WHERE idvotacio          = n_idvotacio
         AND ROWNUM               = 1;

        
     EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RAISE votacioInexistent;
     END;
     
  --verifiquem que la opci no s'hagi donat d alta al sistema prviament.
  BEGIN
       SELECT 'S'
         INTO EXISTE
         FROM OPCIOVOT
         WHERE idvotacio          = n_idvotacio
         AND   numopcio           = n_numopcio
         AND ROWNUM               = 1;
          
        RAISE opcioDuplicada; 
         EXCEPTION
        WHEN NO_DATA_FOUND THEN
          NULL;
     END;
    
   -- afegim la nova opcio
   INSERT INTO OPCIOVOT( idvotacio,numopcio, descripcio) VALUES (  n_idvotacio_ins, n_numopcio_ins, n_descripcio_ins);
   
   -- si ha finalitzat correctament retorna OK, en cas contrari gestionem els errors
   rst := 'OK';

    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'AltaOpcio',
          'numopcio: ' || n_numopcio ||' idVotacio: ' || n_idvotacio||', descripcio:  ' ||n_descripcio,
          'RSP: '||rst);   
   COMMIT;
EXCEPTION
   WHEN votacioInexistent
   THEN
      rst := 'ERROR: la votaci no existeix';
   
    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'AltaOpcio',
          'numopcio: ' || n_numopcio ||' idVotacio: ' || n_idvotacio||', descripcio:  ' ||n_descripcio,
          'RSP: '||rst);   
   COMMIT;
   
    WHEN opcioDuplicada
   THEN
      rst := 'ERROR: la opci ja ha estat donada d alta al sistema prviament';
   
    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'AltaOpcio',
          'numopcio: ' || n_numopcio ||' idVotacio: ' || n_idvotacio||', descripcio:  ' ||n_descripcio,
          'RSP: '||rst);   
   COMMIT;
      
   WHEN campsNuls
   THEN
      rst := 'ERROR: el codi de la votacio i la descripci no poden ser nuls';
      
   INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'AltaOpcio',
          'numopcio: ' || n_numopcio ||' idVotacio: ' || n_idvotacio||', descripcio:  ' ||n_descripcio,
          'RSP: '||rst);   
   COMMIT;
   WHEN OTHERS
   THEN
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:'
             || err_msg;
   
    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'AltaOpcio',
          'numopcio: ' || n_numopcio ||' idVotacio: ' || n_idvotacio||', descripcio:  ' ||n_descripcio,
          'RSP: '||rst);   
   COMMIT;
END altaOpcioVot;
/
-- --------------------------------------------------------------------------------------------------------

create or replace PROCEDURE altaVotacio (
   n_titol          IN       VARCHAR2,
   n_descripcio     IN       VARCHAR2,
   n_datainici      IN       DATE,
   n_datafi         IN       DATE,
   n_datapublicacio IN       DATE,
   n_president      IN       NUMBER,
   n_web            IN       VARCHAR2,
   rst              OUT      VARCHAR2
)
IS   
/******************************************************************************
   altaVotacio:  Dona d'alta una votacio al sistema.
                
   PARAMETRES:  n_titol, n_descripcio, n_datainici, n_datafi, n_datapublicacio,
                n_president, n_web, rst
   INPUT:       n_titol, n_descripcio, n_datainici, n_datafi, n_datapublicacio,
                n_president, n_web
   OUTPUT:      rst
   RETORNA:     'OK' si s'ha executat correctament, 'ERROR: Tipus d'Error' en
                cas contrari
   EXEMPLE D'US:     altaVotacio('votacio 2491','descripcio de la votacio',
                                  '21/01/2012','29/01/2012', '10/01/2012',
                                  125, 'http://www.planaweb.org', rst)
******************************************************************************/
   n_titol_ins                    VARCHAR2(50);
   n_descripcio_ins               VARCHAR2(100);
   n_datainici_ins                DATE;
   n_datafi_ins                   DATE;
   n_datapublicacio_ins           DATE;
   n_president_ins                NUMBER;
   n_web_ins                      VARCHAR2(50);
   err_msg                        VARCHAR2 (100);
   existe                         VARCHAR2(1);
   campsNuls                      EXCEPTION;
   PresidentInexistent            EXCEPTION;  
   datapublicacioerronea          EXCEPTION;
   datafierronea                  EXCEPTION;
  
BEGIN
   n_titol_ins            := n_titol;
   n_descripcio_ins       := n_descripcio;
   n_datainici_ins        := n_datainici;
   n_datafi_ins           := n_datafi;
   n_datapublicacio_ins   := n_datapublicacio;
   n_president_ins        := n_president;
   n_web_ins              := n_web;
   
  --verifiquem que el ttol, la descripcio, la data d'inici, la de fi i la de 
  --publicaci i el president no siguin nuls.
  
  IF (n_titol_ins IS NULL) OR (n_descripcio_ins IS NULL) OR (n_datainici_ins IS NULL) 
      OR (n_datafi_ins IS NULL) OR (n_datapublicacio_ins IS NULL)OR (n_president_ins IS NULL) THEN 
    RAISE campsNuls;
  END IF; 
          
  -- Verifiquem que existeix el ciutada que far el paper de president.
    
   BEGIN
       SELECT 'S'
         INTO EXISTE
         FROM CIUTADA
         WHERE idciutada          = n_president
         AND ROWNUM               = 1;                          
                        
     EXCEPTION
       WHEN NO_DATA_FOUND THEN
          RAISE presidentInexistent;
     END;
     
  -- Verifiquem que la data de puclicaci es com a mnim una setmana abans que la d'inici de la votaci
  
  IF (n_datainici<=n_datapublicacio+7) THEN
   RAISE datapublicacioerronea;
     END IF;
     
  -- Verifiquem que la data de inici sigui anterior a la data de fi de la votacio
  
  IF (n_datainici>=n_datafi) THEN
   RAISE datafierronea;
     END IF;
  
  -- Si ha passat les verificacions afegim la votacio al sistema
  
  INSERT INTO VOTACIO (titol, descripcio, datainici, datafi, datapublicacio, president, web)
         VALUES ( n_titol_ins, n_descripcio_ins, n_datainici_ins, n_datafi_ins, n_datapublicacio_ins, 
                  n_president_ins, n_web_ins);
      
  -- si ha finalitzat correctament retorna OK, en cas contrari gestionem els errors
   rst := 'OK';

    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'AltaVotacio',
          'ttol: ' || n_titol||', descripcio:  ' || n_descripcio || ' ,datainici: ' || to_char(n_datainici)||
          ' datafi : ' ||to_char(n_datafi)|| ' datapublicacio: ' ||to_char(n_datapublicacio)|| ' president: '
          ||n_president|| ' web: ' ||n_web,
          'RSP: '||rst);   
   COMMIT;
EXCEPTION        
     WHEN presidentInexistent
   THEN
   ROLLBACK;
      rst := 'ERROR: El ciutad que es vol introduir com a president no existeix';
   
    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'AltaVotacio',
          'ttol: ' || n_titol||', descripcio:  ' || n_descripcio || ' ,datainici: ' || to_char(n_datainici)||
          ' datafi : ' ||to_char(n_datafi)|| ' datapublicacio: ' ||to_char(n_datapublicacio)|| ' president: '
          ||n_president|| ' web: ' ||n_web,
          'RSP: '||rst);   
      COMMIT;       
    WHEN datapublicacioerronea
   THEN
   ROLLBACK;
      rst := 'ERROR: la data de publicaci ha de ser com a mnim una setmana abans que la d inici de la votacio';
   
    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'AltaVotacio',
          'ttol: ' || n_titol||', descripcio:  ' || n_descripcio || ' ,datainici: ' || to_char(n_datainici)||
          ' datafi : ' ||to_char(n_datafi)|| ' datapublicacio: ' ||to_char(n_datapublicacio)|| ' president: '
          ||n_president|| ' web: ' ||n_web,
          'RSP: '||rst);   
      COMMIT;   
    
    WHEN datafierronea
   THEN
   ROLLBACK;
      rst := 'ERROR: la data de finalitzaci no pot ser anterior ni igual que la d inici de la votacio';
   
    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'AltaVotacio',
          'ttol: ' || n_titol||', descripcio:  ' || n_descripcio || ' ,datainici: ' || to_char(n_datainici)||
          ' datafi : ' ||to_char(n_datafi)|| ' datapublicacio: ' ||to_char(n_datapublicacio)|| ' president: '
          ||n_president|| ' web: ' ||n_web,
          'RSP: '||rst);   
      COMMIT;   
      
   WHEN campsNuls
   THEN
   ROLLBACK;
      rst := 'ERROR: hi ha camps nuls que no poden ser nuls';
    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'AltaVotacio',
          'ttol: ' || n_titol||', descripcio:  ' || n_descripcio || ' ,datainici: ' || to_char(n_datainici)||
          ' datafi : ' ||to_char(n_datafi)|| ' datapublicacio: ' ||to_char(n_datapublicacio)|| ' president: '
          ||n_president|| ' web: ' ||n_web,
          'RSP: '||rst);   
   
      COMMIT;
   WHEN OTHERS
   THEN
   ROLLBACK;
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:'
             || err_msg;
   
    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'AltaVotacio',
          'ttol: ' || n_titol||', descripcio:  ' || n_descripcio || ' ,datainici: ' || to_char(n_datainici)||
          ' datafi : ' ||to_char(n_datafi)|| ' datapublicacio: ' ||to_char(n_datapublicacio)|| ' president: '
          ||n_president|| ' web: ' ||n_web,
          'RSP: '||rst);   
      COMMIT;
END altaVotacio;
/

-- ----------------------------------------------------------------------------------------------------------------

/********************************************************************************
			PROCEDUMENTS DE BAIXA
********************************************************************************/

create or replace PROCEDURE baixaAmbit (
   n_idvotacio     IN       NUMBER,
   n_idcodicens    IN       NUMBER,
   n_idtipuscens   IN       NUMBER,
   rst             OUT      VARCHAR2
)
IS   
/******************************************************************************
   baixaAmbit:  Dona de baixa un cens en una votacio.
                
   PARAMETRES:  n_idvotacio, n_idcodicens, n_idtipuscens, rst
   INPUT:       n_idvotacio, n_idcodicens, n_idtipuscens
   OUTPUT:      rst
   RETORNA:     'OK' si s'ha executat correctament, 'ERROR: Tipus d'Error' en
                cas contrari
   EXEMPLE D'US:     baixaAmbit(1,1,1, rst)
******************************************************************************/
   n_idvotacio_ins            NUMBER;
   n_idcodicens_ins           NUMBER;
   n_idtipuscens_ins          NUMBER;
   n_estat_ins                NUMBER;
   err_msg                    VARCHAR2 (100);
   existe                     VARCHAR2(1);
   campsNuls                  EXCEPTION;
   CensInexistent             EXCEPTION;
   tipusCensInexistent        EXCEPTION;
   votacioInexistent          EXCEPTION;
   votacioOberta              EXCEPTION;
   votacioTancada             EXCEPTION;
   ambitInexistent            EXCEPTION;
  
BEGIN
   n_idvotacio_ins    := n_idvotacio;
   n_idcodicens_ins   := n_idcodicens;
   n_idtipuscens_ins  := n_idtipusCens;
      

  --verifiquem que el codi de la votacio, el codi del cens i tipus de cens no siguin nuls
  
  IF (n_idvotacio_ins IS NULL) OR (n_idcodicens_ins IS NULL) OR (n_idtipuscens_ins IS NULL) THEN 
    RAISE campsNuls;
  END IF; 
  
  
  --verifiquem que la votacio s'hagi donat d'alta previament en aquest cens.
  BEGIN
       SELECT 'S'
         INTO EXISTE
         FROM AMBIT
         WHERE idvotacio        = n_idvotacio
         AND   codicens         = n_idcodicens
         AND   idtipuscens      = n_idtipuscens
         AND ROWNUM               = 1;

        
     EXCEPTION
        WHEN NO_DATA_FOUND THEN
          RAISE ambitInexistent;
     END;
 
   -- verifiquem que la votaci no estigui oberta
   SELECT estatvotacio
   INTO n_estat_ins
   FROM VOTACIO
   WHERE idvotacio=n_idvotacio_ins;
   
   IF (n_estat_ins=2) THEN  RAISE votacioOberta;
   END IF;
   
   -- verifiquem que la votaci no estigui tancada
   IF (n_estat_ins=3) THEN  RAISE votacioTancada; 
   END IF;

    
   -- Eliminem l Ambit del sistema
   DELETE FROM AMBIT
   WHERE idvotacio    = n_idvotacio_ins
   AND   codicens     = n_idcodicens_ins
   AND   idtipuscens  = n_idtipuscens_ins;
   
   -- si ha finalitzat correctament retorna OK, en cas contrari gestionem els errors
   rst := 'OK';

    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'BaixaAmbit',
          'idVotacio: ' || n_idvotacio||', idcodicens:  ' || n_idcodicens || ' ,idtipuscens: ' || n_idtipuscens, 
          'RSP: '||rst);   
   COMMIT;
EXCEPTION
   WHEN ambitInexistent
   THEN
   ROLLBACK;
      rst := 'ERROR: Aquest mbit no existeix';
      
    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'BaixaAmbit',
          'idVotacio: ' || n_idvotacio||', idcodicens:  ' || n_idcodicens || ' ,idtipuscens: ' || n_idtipuscens, 
          'RSP: '||rst);   
      COMMIT;       
       
   WHEN votacioOberta
   THEN
   ROLLBACK;
      rst := 'ERROR: La votaci ja est oberta';
   
    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'BaixaAmbit',
          'idVotacio: ' || n_idvotacio||', idcodicens:  ' || n_idcodicens || ' ,idtipuscens: ' || n_idtipuscens, 
          'RSP: '||rst);    
      COMMIT;  
      
   WHEN votacioTancada
   THEN
   ROLLBACK;
      rst := 'ERROR: La votaci ja est tancada';
   
    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'BaixaAmbit',
          'idVotacio: ' || n_idvotacio||', idcodicens:  ' || n_idcodicens || ' ,idtipuscens: ' || n_idtipuscens, 
          'RSP: '||rst);    
      COMMIT;       
      
   WHEN campsNuls
   THEN
   ROLLBACK;
      rst := 'ERROR: el codi de la votaci, el codi del cens i el tipus de cens no poden ser nuls';
    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'BaixaAmbit',
          'idVotacio: ' || n_idvotacio||', idcodicens:  ' || n_idcodicens || ' ,idtipuscens: ' || n_idtipuscens, 
          'RSP: '||rst);   
   
      COMMIT;
   WHEN OTHERS
   THEN
   ROLLBACK;
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:'
             || err_msg;
   
     INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'BaixaAmbit',
          'idVotacio: ' || n_idvotacio||', idcodicens:  ' || n_idcodicens || ' ,idtipuscens: ' || n_idtipuscens, 
          'RSP: '||rst);   
      COMMIT;
END baixaAmbit;
/
-- ------------------------------------------------------------------------------------------------------

create or replace PROCEDURE baixaCensCiutada (
   n_idciutada     IN       NUMBER,
   n_idcodicens    IN       NUMBER,
   n_idtipuscens   IN       NUMBER,
   rst             OUT      VARCHAR2
)
IS   
/******************************************************************************
   baixaCensCiutada:  Dona de baixa un ciutad en un cens.
                
   PARAMETRES:  n_idciutada, n_idcodicens, n_idtipuscens, rst
   INPUT:       n_idciutada, n_idcodicens, n_idtipuscens
   OUTPUT:      rst
   RETORNA:     'OK' si s'ha executat correctament, 'ERROR: Tipus d'Error' en
                cas contrari
   EXEMPLE D'US:     baixaCensCiutada(1,1,1, rst)
******************************************************************************/
   n_idciutada_ins            NUMBER;
   n_idcodicens_ins           NUMBER;
   n_idtipuscens_ins          NUMBER;
   n_actiu_ins                NUMBER;
   err_msg                    VARCHAR2 (100);
   existe                     VARCHAR2(1);
   campsNuls                  EXCEPTION;
   CensInexistent             EXCEPTION;
   CiutadaInexistent          EXCEPTION;
   censCiutadaBaixa           EXCEPTION;
   tipusCensInexistent        EXCEPTION;
  
BEGIN
   n_idciutada_ins    := n_idciutada;
   n_idcodicens_ins   := n_idcodicens;
   n_idtipuscens_ins  := n_idtipusCens;
      

  --verifiquem que el codi del ciutada, el codi del cens i tipus de cens no siguin nuls
  
  IF (n_idciutada_ins IS NULL) OR (n_idcodicens_ins IS NULL) OR (n_idtipuscens_ins IS NULL) THEN 
    RAISE campsNuls;
  END IF; 
  
  
  
  -- Verifiquem que el ciutad existeixi
  
  BEGIN
       SELECT 'S'
         INTO EXISTE
         FROM CIUTADA
         WHERE idciutada          = n_idciutada
         AND ROWNUM               = 1;
                        
     EXCEPTION
       WHEN NO_DATA_FOUND THEN
          RAISE ciutadaInexistent;
     END;
  -- Verifiquem que el tipus de cens existeix
  BEGIN
  SELECT 'S'
         INTO EXISTE
         FROM TIPUSCENS
         WHERE idtipuscens        = n_idtipuscens
         AND ROWNUM               = 1;
                     
     EXCEPTION
       WHEN NO_DATA_FOUND THEN
          RAISE tipusCensInexistent;
     END;
     
  -- Verifiquem que existeix el cens
  
  IF (n_idtipuscens=2) THEN   
   BEGIN
       SELECT 'S'
         INTO EXISTE
         FROM CENSESTATAL
         WHERE idpais             = n_idcodicens
         AND ROWNUM               = 1;
                
     EXCEPTION
       WHEN NO_DATA_FOUND THEN
          RAISE censInexistent;
     END;
     END IF;
     
    IF (n_idtipuscens=3) THEN   
   BEGIN
       SELECT 'S'
         INTO EXISTE
         FROM CENSREGIONAL
         WHERE idregio            = n_idcodicens
         AND ROWNUM               = 1;
       
     EXCEPTION
       WHEN NO_DATA_FOUND THEN
          RAISE censInexistent;
     END;
     END IF;
    
     IF (n_idtipuscens=4) THEN   
   BEGIN
       SELECT 'S'
         INTO EXISTE
         FROM CENSLOCAL
         WHERE idlocal            = n_idcodicens
         AND ROWNUM               = 1;   
           
     EXCEPTION
       WHEN NO_DATA_FOUND THEN
          RAISE censInexistent;
     END;
     END IF;
     
     --verifiquem que el ciutada no s'hagi donat de baixa previament en aquest cens.
  BEGIN
       SELECT actiu
         INTO n_actiu_ins
         FROM CENSCIUTADANS
         WHERE idciutada        = n_idciutada
         AND   idcodicens       = n_idcodicens
         AND   idtipuscens      = n_idtipuscens;
        
        IF (n_actiu_ins=0) THEN
        RAISE censCiutadaBaixa;
        END IF;
     END;

    
   -- donem de baixa el cens
   
   UPDATE CENSCIUTADANS SET actiu = 0, databaixa = SYSDATE
          WHERE idciutada        = n_idciutada
          AND   idcodicens       = n_idcodicens
          AND   idtipuscens      = n_idtipuscens ;
   
   -- si ha finalitzat correctament retorna OK, en cas contrari gestionem els errors
   rst := 'OK';

    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'BaixaCensCiutada',
          'idCiutada: ' || n_idciutada||', idcodicens:  ' || n_idcodicens || ' ,idtipuscens: ' || n_idtipuscens, 
          'RSP: '||rst);   
   COMMIT;
EXCEPTION
   WHEN censCiutadaBaixa
   THEN
   ROLLBACK;
      rst := 'ERROR: Aquest ciutada ja ha estat donat de baixa en aquest cens';
      
    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'BaixaCensCiutada',
          'idCiutada: ' || n_idciutada||', idcodicens:  ' || n_idcodicens || ' ,idtipuscens: ' || n_idtipuscens, 
          'RSP: '||rst);   
   COMMIT;      
      
   WHEN censInexistent
   THEN
   ROLLBACK;
      rst := 'ERROR: El cens no existeix';
   
    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'BaixaCensCiutada',
          'idCiutada: ' || n_idciutada||', idcodicens:  ' || n_idcodicens || ' ,idtipuscens: ' || n_idtipuscens, 
          'RSP: '||rst);   
   COMMIT;  
      
   WHEN tipusCensInexistent
   THEN
   ROLLBACK;
      rst := 'ERROR: El tipus de cens no existeix';
   
    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'BaixaCensCiutada',
          'idCiutada: ' || n_idciutada||', idcodicens:  ' || n_idcodicens || ' ,idtipuscens: ' || n_idtipuscens, 
          'RSP: '||rst);   
   COMMIT;      
   
   WHEN ciutadaInexistent
   THEN
   ROLLBACK;
      rst := 'ERROR: El ciutada no existeix';
   
    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'BaixaCensCiutada',
          'idCiutada: ' || n_idciutada||', idcodicens:  ' || n_idcodicens || ' ,idtipuscens: ' || n_idtipuscens, 
          'RSP: '||rst);   
   COMMIT;       
      
   WHEN campsNuls
   THEN
   ROLLBACK;
      rst := 'ERROR: el codi del ciutada, el codi del cens i el tipus de cens no poden ser nuls';
      
  INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'BaixaCensCiutada',
          'idCiutada: ' || n_idciutada||', idcodicens:  ' || n_idcodicens || ' ,idtipuscens: ' || n_idtipuscens, 
          'RSP: '||rst);   
   COMMIT;
   WHEN OTHERS
   THEN
   ROLLBACK;
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:'
             || err_msg;
   
    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'BaixaCensCiutada',
          'idCiutada: ' || n_idciutada||', idcodicens:  ' || n_idcodicens || ' ,idtipuscens: ' || n_idtipuscens, 
          'RSP: '||rst);   
   COMMIT;
END baixaCensCiutada;
/
-- ------------------------------------------------------------------------------------------------------------

create or replace PROCEDURE baixaCensEstatal (
   n_codipais        IN       VARCHAR2,
   rst               OUT      VARCHAR2
)
IS   
/******************************************************************************
   baixaCensEstatal:   Gestiona la baixa d'un cens Estatal al sistema
                
   PARAMETRES:  n_codipais, rst
   INPUT:       n_codipais
   OUTPUT:      rst
   RETORNA:     'OK' si s'ha executat correctament, 'ERROR: Tipus d'Error' en
                cas contrari
   EXEMPLE D'US:     BaixaCensEstatal('ES', rst)
******************************************************************************/
   n_codipais_ins              VARCHAR2(2);
   n_actiu_ins                 NUMBER;
   N                           NUMBER:=0;
   err_msg                     VARCHAR2 (100);
   existe                      VARCHAR2(1);
   paramE                      VARCHAR2(500);
   campsNuls                   EXCEPTION;
   paisBaixa                   EXCEPTION; 
   paisInexistent              EXCEPTION;
   
   
   -- cursor que llista les regions que pertanyen al pas
   
    CURSOR CUR_REGIONAL IS
        SELECT idregio
         FROM CENSREGIONAL
         WHERE idpais = n_codipais_ins;
BEGIN

n_codipais_ins :=n_codipais;
    

  --verifiquem que el codi del pas no sigui nul
  IF (n_codipais_ins IS NULL)  THEN 
    RAISE campsNuls;
  END IF; 
  
    -- verifiquem que el pas s'hagi donat d'alta al sistema   
   BEGIN
       SELECT 'S'
         INTO EXISTE
         FROM CENSESTATAL
         WHERE codipais           = n_codipais_ins
         AND ROWNUM               = 1;
        
     EXCEPTION
       WHEN NO_DATA_FOUND THEN
          RAISE paisInexistent;
     END;
     
   --verifiquem que pas no s'hagi donat de baixa previament.
  BEGIN
       SELECT actiu
         INTO n_actiu_ins
         FROM CENSESTATAL
         WHERE codipais             = n_codipais_ins;
         
    IF(n_actiu_ins=0) THEN
        RAISE paisBaixa;
     END IF;
     END;
    
     
   -- Donem de baixa el CensEstatal
   UPDATE CENSESTATAL SET actiu=0, databaixa=SYSDATE
          WHERE codipais=n_codipais_ins;
          
   -- Donem de baixa les regions associades al cens estatal
    FOR R IN CUR_REGIONAL LOOP
    N := N + 1;
    BEGIN    
    BAIXACENSREGIONAL(R.idregio, rst); 
    END;
    END LOOP;
   
   -- si ha finalitzat correctament retorna OK, en cas contrari gestionem els errors
   rst := 'OK';

    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'BaixaCensEstatal',
          'codiPais: ' || n_codipais ,
          'RSP: '||rst);   
   COMMIT;
   
EXCEPTION
   WHEN paisBaixa
   THEN
      rst := 'ERROR: Aquest pas ja ha estat donada de baixa al cens';
   
    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'BaixaCensEstatal',
          'codiPais: ' || n_codipais ,
          'RSP: '||rst);   
   COMMIT;          
   
   WHEN paisInexistent
   THEN
      rst := 'ERROR: El pas no ha estat donat d alta al sistema';
   
    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'BaixaCensEstatal',
          'codiPais: ' || n_codipais ,
          'RSP: '||rst);   
   COMMIT;
  
   WHEN campsNuls
   THEN
      rst := 'ERROR: el codi del pas no pot ser nul';
      
   INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'BaixaCensEstatal',
          'codiPais: ' || n_codipais ,
          'RSP: '||rst);   
   COMMIT;
   WHEN OTHERS
   THEN
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:'
             || err_msg;
   
   INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'BaixaCensEstatal',
          'codiPais: ' || n_codipais ,
          'RSP: '||rst);   
   COMMIT;
END BaixaCensEstatal;

/
-- ------------------------------------------------------------------------------------

create or replace PROCEDURE BaixaCensLocal (
   n_idlocalitat          IN       NUMBER,
   rst               OUT      VARCHAR2
)
IS   
/******************************************************************************
   baixaCensLocal:   Gestiona la baixa d'un cens Local al sistema
                
   PARAMETRES:  n_idlocalitat, rst
   INPUT:       n_idlocalitat
   OUTPUT:      rst
   RETORNA:     'OK' si s'ha executat correctament, 'ERROR: Tipus d'Error' en
                cas contrari
   EXEMPLE D'US:     BaixaCensLocal(1, rst)
******************************************************************************/
   n_idlocalitat_ins           NUMBER;
   n_actiu_ins                 NUMBER;
   N                           NUMBER:=0;
   err_msg                     VARCHAR2 (100);
   existe                      VARCHAR2(1);
   paramE                      VARCHAR2(500);
   campsNuls                   EXCEPTION;
   localitatBaixa              EXCEPTION; 
   localitatInexistent         EXCEPTION;
   
   
   -- cursor que llista els ciutadans assignats al cens local
   
    CURSOR CUR_CIUTADANS IS
        SELECT idciutada
         FROM CENSCIUTADANS
         WHERE idtipuscens=4
         AND idcodicens= n_idlocalitat_ins;
BEGIN

n_idlocalitat_ins := n_idlocalitat;
    

  --verifiquem que el codi de la localitat no sigui nul
  IF (n_idlocalitat_ins IS NULL)  THEN 
    RAISE campsNuls;
  END IF;  
    
   -- verifiquem que la localitat s'hagi donat d'alta al sistema   
   BEGIN
       SELECT 'S'
         INTO EXISTE
         FROM CENSLOCAL
         WHERE idlocal            = n_idlocalitat_ins
         AND ROWNUM               = 1;
        
     EXCEPTION
       WHEN NO_DATA_FOUND THEN
          RAISE localitatInexistent;
     END;
     
   --verifiquem que la localitat no s'hagi donat de baixa previament.
  BEGIN
       SELECT actiu
         INTO n_actiu_ins
         FROM CENSLOCAL
         WHERE idlocal             = n_idlocalitat_ins;
         
     IF(n_actiu_ins=0) THEN
        RAISE localitatBaixa;
     END IF;
   END;
     
   -- Donem de baixa el CensLocal
   UPDATE CENSLOCAL SET actiu=0, databaixa=SYSDATE
          WHERE idlocal=n_idlocalitat_ins;
          
   -- Donem de baixa els ciutadans Associats al cens local
    FOR R IN CUR_CIUTADANS LOOP
    N := N + 1;
    BEGIN    
    BAIXACIUTADA(IDCIUTADA_CODICIUTADA(R.idciutada), rst); 
    END;
    END LOOP;
    
  -- actualitzem la Estadstica 5
  
  ACTUALITZAE5(rst);
   
   -- si ha finalitzat correctament retorna OK, en cas contrari gestionem els errors
   rst := 'OK';

    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'BaixaCensLocal',
          'idLocalitat: ' || n_idlocalitat ,
          'RSP: '||rst);
   
   COMMIT;
EXCEPTION
   WHEN localitatBaixa
   THEN
      rst := 'ERROR: Aquesta localitat ja ha estat donada de baixa al cens';
   
    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'BaixaCensLocal',
          'idLocalitat: ' || n_idlocalitat ,
          'RSP: '||rst);
    COMMIT;            
   
   WHEN localitatInexistent
   THEN
      rst := 'ERROR: La Localitat no ha estat donada d alta al sistema';
   
    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'BaixaCensLocal',
          'idLocalitat: ' || n_idlocalitat ,
          'RSP: '||rst);
    COMMIT;
  
   WHEN campsNuls
   THEN
      rst := 'ERROR: el codi de la localitat no pot ser nul';
      
   INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'BaixaCensLocal',
          'idLocalitat: ' || n_idlocalitat ,
          'RSP: '||rst);
    COMMIT;
   WHEN OTHERS
   THEN
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:'
             || err_msg;
   
   INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'BaixaCensLocal',
          'idLocalitat: ' || n_idlocalitat ,
          'RSP: '||rst);
    COMMIT;
END BaixaCensLocal;
/
-- ----------------------------------------------------------------------------------------

create or replace PROCEDURE baixaCensRegional (
   n_idregio         IN       NUMBER,
   rst               OUT      VARCHAR2
)
IS   
/******************************************************************************
   baixaCensRegional:   Gestiona la baixa d'un cens Regional al sistema
                
   PARAMETRES:  n_idregio, rst
   INPUT:       n_idregio
   OUTPUT:      rst
   RETORNA:     'OK' si s'ha executat correctament, 'ERROR: Tipus d'Error' en
                cas contrari
   EXEMPLE D'US:     BaixaCensRegional(1, rst)
******************************************************************************/
   n_idregio_ins               NUMBER;
   n_actiu_ins                 NUMBER;
   N                           NUMBER:=0;
   err_msg                     VARCHAR2 (100);
   existe                      VARCHAR2(1);
   paramE                      VARCHAR2(500);
   campsNuls                   EXCEPTION;
   regioBaixa                  EXCEPTION; 
   regioInexistent             EXCEPTION;
   
   
   -- cursor que llista les localitats que pertanyen a la regi
   
    CURSOR CUR_LOCAL IS
        SELECT idlocal
         FROM CENSLOCAL
         WHERE  idregio= n_idregio_ins;
BEGIN

n_idregio_ins :=n_idregio;
    

  -- verifiquem que el codi de la regio no sigui nul
  IF (n_idregio_ins IS NULL)  THEN 
    RAISE campsNuls;
  END IF; 
  
   -- verifiquem que la regio s'hagi donat d'alta al sistema   
   BEGIN
       SELECT 'S'
         INTO EXISTE
         FROM CENSREGIONAL
         WHERE idregio            = n_idregio_ins
         AND ROWNUM               = 1;
        
     EXCEPTION
       WHEN NO_DATA_FOUND THEN
          RAISE regioInexistent;
     END;
     
  -- verifiquem que la regio no s'hagi donat de baixa previament.
  BEGIN
       SELECT actiu
         INTO n_actiu_ins
         FROM CENSREGIONAL
         WHERE idregio             = n_idregio_ins;
         
    IF(n_actiu_ins=0) THEN
        RAISE regioBaixa;
     END IF;
     END;
     
   -- Donem de baixa el CensRegional
   UPDATE CENSREGIONAL SET actiu=0, databaixa=SYSDATE
          WHERE idregio=n_idregio_ins;
          
   -- Donem de baixa les localitats associades al cens regional
    FOR R IN CUR_LOCAL LOOP
    N := N + 1;
    BEGIN    
    BAIXACENSLOCAL(R.idlocal, rst); 
    END;
    END LOOP;
   
   -- si ha finalitzat correctament retorna OK, en cas contrari gestionem els errors
   rst := 'OK';

    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'BaixaCensRegional',
          'idRegio: ' || n_idregio ,
          'RSP: '||rst);   
   COMMIT;
   
EXCEPTION
   WHEN regioBaixa
   THEN
      rst := 'ERROR: Aquesta regi ja ha estat donada de baixa al cens';
   
    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'BaixaCensRegional',
          'idRegio: ' || n_idregio ,
          'RSP: '||rst);   
    COMMIT;           
   
   WHEN regioInexistent
   THEN
      rst := 'ERROR: La regi no ha estat donat d alta al sistema';
   
    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'BaixaCensRegional',
          'idRegio: ' || n_idregio ,
          'RSP: '||rst);   
   COMMIT;
  
   WHEN campsNuls
   THEN
      rst := 'ERROR: el codi de la regi no pot ser nul';
      
   INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
   VALUES (SYSDATE,'BaixaCensRegional',
          'idRegio: ' || n_idregio ,
          'RSP: '||rst);   
   COMMIT;
   WHEN OTHERS
   THEN
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:'
             || err_msg;
   
   INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'BaixaCensRegional',
          'idRegio: ' || n_idregio ,
          'RSP: '||rst);   
   COMMIT;
END BaixaCensRegional;
/
-- ------------------------------------------------------------------------------------------

create or replace PROCEDURE baixaCiutada (
   n_codiciutada     IN       VARCHAR2,
   rst               OUT      VARCHAR2
)
IS   
/******************************************************************************
   baixaCiutada:  Gestiona la baixa d'un ciutada al sistema
                
   PARAMETRES:  n_codiciutada,rst
   INPUT:       n_codiciutada
   OUTPUT:      rst
   RETORNA:     'OK' si s'ha executat correctament, 'ERROR: Tipus d'Error' en
                cas contrari
   EXEMPLE D'US:     baixaCiutada('AMH658564BS', rst)
******************************************************************************/
   n_codiciutada_ins           VARCHAR2 (11);
   n_idciutada_ins             NUMBER;
   err_msg                     VARCHAR2 (100);
   existe                      VARCHAR2(1);
   n_actiu_ins                 NUMBER;
   campsNuls                   EXCEPTION;
   ciutadaInexistent           EXCEPTION;  
   ciutadaDonatBaixa           EXCEPTION;
  
BEGIN
   n_codiciutada_ins   :=n_codiciutada;
   
      

  --verifiquem que el codi del ciutada  no sigui nul
  IF (n_codiciutada_ins IS NULL) THEN 
    RAISE campsNuls;
  END IF; 
 
  --verifiquem que el ciutada existeixi.
  BEGIN
       SELECT 'S'
         INTO EXISTE
         FROM CIUTADA
         WHERE codiciutada        = n_codiciutada
         AND ROWNUM               = 1;
      
     EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RAISE ciutadaInexistent;
     END;
     
   -- Verifiquem que no s'hagi donat de baixa previament
 
   SELECT actiu, idciutada
   INTO n_actiu_ins, n_idciutada_ins
   FROM CIUTADA
   WHERE codiciutada=n_codiciutada_ins;
   
   IF (n_actiu_ins=0) THEN
   RAISE ciutadaDonatBaixa;
   END IF; 
  
   -- Donem de Baixa al ciutada
   
   UPDATE CIUTADA SET actiu = 0 ,databaixa = sysdate
          WHERE codiciutada = n_codiciutada_ins;
  
   -- Donem de baixa els censos als quals est registrat
   
   UPDATE CENSCIUTADANS SET actiu=0, databaixa = SYSDATE
          WHERE idciutada = n_idciutada_ins;
   
   -- si ha finalitzat correctament retorna OK, en cas contrari gestionem els errors
   rst := 'OK';

    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'BaixaCiutada',
          'codiCiutada: ' || n_codiciutada,
          'RSP: '||rst);   
      COMMIT;
EXCEPTION
   WHEN ciutadaInexistent
   THEN
      rst := 'ERROR: Aquest ciutada no ha estat donat d alta al sistema';
   
    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'BaixaCiutada',
          'codiCiutada: ' || n_codiciutada,
          'RSP: '||rst);   
      COMMIT;     
      
  WHEN ciutadaDonatBaixa
   THEN
      rst := 'ERROR: Aquest ciutada ja ha estat donat de baixa al sistema';
   
    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'BaixaCiutada',
          'codiCiutada: ' || n_codiciutada,
          'RSP: '||rst);   
      COMMIT;    
    
   WHEN campsNuls
   THEN
      rst := 'ERROR: el codi del ciutada no pot ser nul';
      
   INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'BaixaCiutada',
          'codiCiutada: ' || n_codiciutada,
          'RSP: '||rst);   
      COMMIT;
   WHEN OTHERS
   THEN
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:'
             || err_msg;
   
    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'BaixaCiutada',
          'codiCiutada: ' || n_codiciutada,
          'RSP: '||rst);   
      COMMIT;
END baixaCiutada;
/
-- --------------------------------------------------------------------------------

create or replace PROCEDURE baixaOpcioVot (
   
   n_idvotacio       IN       NUMBER,
   n_numopcio        IN       NUMBER,
   rst               OUT      VARCHAR2
)
IS   
/******************************************************************************
   baixaOpcioVot:  Gestiona la baixa d'una opcio de vot en una votaci.
                
   PARAMETRES:  n_idvotacio, n_numvotacio, rst
   INPUT:       n_idvotacio, n_numvotacio
   OUTPUT:      rst
   RETORNA:     'OK' si s'ha executat correctament, 'ERROR: Tipus d'Error' en
                cas contrari
   EXEMPLE D'US:     baixaOpcioVot(1,1, rst)
******************************************************************************/
   n_idvotacio_ins             NUMBER;
   n_numopcio_ins              NUMBER;
   n_estat_ins                 NUMBER;
   err_msg                     VARCHAR2 (100);
   existe                      VARCHAR2(1);
   campsNuls                   EXCEPTION;
   votacioInexistent           EXCEPTION;
   votacioTancada              EXCEPTION;
   votacioOberta               EXCEPTION;
  
BEGIN
   n_numopcio_ins    :=n_numopcio;
   n_idvotacio_ins   :=n_idvotacio;
       

  --verifiquem que el codi de la votacio i el numero de la opci no siguin nuls
  IF (n_numopcio IS NULL) OR (n_idvotacio_ins IS NULL) THEN 
    RAISE campsNuls;
  END IF; 
  
  --verifiquem que la votacio s'hagi donat d alta al sistema.
  BEGIN
       SELECT 'S'
         INTO EXISTE
         FROM VOTACIO
         WHERE idvotacio          = n_idvotacio
         AND ROWNUM               = 1;

        
     EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RAISE votacioInexistent;
     END;
     
   -- verifiquem que la votaci no estigui oberta
   SELECT estatvotacio
   INTO n_estat_ins
   FROM VOTACIO
   WHERE idvotacio=n_idvotacio_ins;
   
   IF (n_estat_ins=2) THEN  RAISE votacioOberta;
   END IF;
   
   -- verifiquem que la votaci no estigui tancada
   IF (n_estat_ins=3) THEN  RAISE votacioTancada; 
   END IF;
    
   -- Esborrem la opci
   DELETE FROM OPCIOVOT
   WHERE idvotacio = n_idvotacio_ins
   AND   numopcio  = n_numopcio_ins ;
   
   -- si ha finalitzat correctament retorna OK, en cas contrari gestionem els errors
   rst := 'OK';

    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'BaixaOpcio',
          'numopcio: ' || n_numopcio ||' idVotacio: ' || n_idvotacio,
          'RSP: '||rst);   
   COMMIT;
EXCEPTION
   WHEN votacioInexistent
   THEN
      rst := 'ERROR: la votaci no existeix';
   
   INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'BaixaOpcio',
          'numopcio: ' || n_numopcio ||' idVotacio: ' || n_idvotacio,
          'RSP: '||rst);   
   COMMIT;
   
   WHEN votacioOberta
   THEN
      rst := 'ERROR: la votaci ja est oberta';
   
   INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'BaixaOpcio',
          'numopcio: ' || n_numopcio ||' idVotacio: ' || n_idvotacio,
          'RSP: '||rst);   
   COMMIT;
   
   WHEN votacioTancada
   THEN
      rst := 'ERROR: la votaci ja est tancada';
   
   INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'BaixaOpcio',
          'numopcio: ' || n_numopcio ||' idVotacio: ' || n_idvotacio,
          'RSP: '||rst);   
   COMMIT;
      
   WHEN campsNuls
   THEN
      rst := 'ERROR: el codi de la votacio i la descripci no poden ser nuls';
      
  INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'BaixaOpcio',
          'numopcio: ' || n_numopcio ||' idVotacio: ' || n_idvotacio,
          'RSP: '||rst);   
   COMMIT;
   WHEN OTHERS
   THEN
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:'
             || err_msg;
   
    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'BaixaOpcio',
          'numopcio: ' || n_numopcio ||' idVotacio: ' || n_idvotacio,
          'RSP: '||rst);   
   COMMIT;
END baixaOpcioVot;
/
-- ---------------------------------------------------------------------------------

create or replace PROCEDURE baixaVotacio (
   n_idvotacio      IN       NUMBER,
   rst              OUT      VARCHAR2
)
IS   
/******************************************************************************
   baixaVotacio:  Dona de baixa una votacio al sistema.
                
   PARAMETRES:  n_idvotacio, rst
   INPUT:       n_idvotacio
   OUTPUT:      rst
   RETORNA:     'OK' si s'ha executat correctament, 'ERROR: Tipus d'Error' en
                cas contrari
   EXEMPLE D'US:     baixaVotacio(1, rst)
******************************************************************************/
   n_idvotacio_ins                NUMBER;
   n_estat_ins                    NUMBER;
   err_msg                        VARCHAR2 (100);
   existe                         VARCHAR2(1);
   campsNuls                      EXCEPTION;
   votacioInexistent              EXCEPTION;  
   votacioOberta                  EXCEPTION;
   votacioTancada                 EXCEPTION;
  
BEGIN
   n_idvotacio_ins        := n_idvotacio;
  
  --verifiquem que el codi de la votaci no sigui nul.
  
  IF (n_idvotacio_ins IS NULL) THEN 
    RAISE campsNuls;
  END IF; 
  
   --verifiquem que la votacio s'hagi donat d alta al sistema.
  BEGIN
       SELECT 'S'
         INTO EXISTE
         FROM VOTACIO
         WHERE idvotacio          = n_idvotacio_ins
         AND ROWNUM               = 1;

        
     EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RAISE votacioInexistent;
     END;
  
    -- verifiquem que la votaci no estigui oberta
   SELECT estatvotacio
   INTO n_estat_ins
   FROM VOTACIO
   WHERE idvotacio=n_idvotacio_ins;
   
   IF (n_estat_ins=2) THEN  RAISE votacioOberta;
   END IF;
   
   -- verifiquem que la votaci no estigui tancada
   IF (n_estat_ins=3) THEN  RAISE votacioTancada; 
   END IF;
  
  -- Si ha passat les verificacions esborrem la votacio del sistema
  
  DELETE FROM VOTACIO WHERE idvotacio=n_idvotacio_ins;
  
  -- Eliminem els censos associats a la votaci
  
  DELETE FROM AMBIT WHERE idvotacio=n_idvotacio_ins; 
      
  -- si ha finalitzat correctament retorna OK, en cas contrari gestionem els errors
   rst := 'OK';

    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'BaixaVotacio',
          'idvotacio: ' || n_idvotacio,
          'RSP: '||rst);   
   COMMIT;
EXCEPTION        
     WHEN votacioOberta
   THEN
   ROLLBACK;
      rst := 'ERROR: La votaci ja est oberta';
   
     INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'BaixaVotacio',
          'idvotacio: ' || n_idvotacio,
          'RSP: '||rst);   
   COMMIT;     
    WHEN votacioTancada
   THEN
   ROLLBACK;
      rst := 'ERROR: la votacio est tancada';
   
     INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'BaixaVotacio',
          'idvotacio: ' || n_idvotacio,
          'RSP: '||rst);   
   COMMIT;   
    
   WHEN votacioInexistent
   THEN
   ROLLBACK;
      rst := 'ERROR: la votaci no est donada d alta al sistema';
   
     INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'BaixaVotacio',
          'idvotacio: ' || n_idvotacio,
          'RSP: '||rst);   
   COMMIT; 
      
   WHEN campsNuls
   THEN
   ROLLBACK;
      rst := 'ERROR: el codi de la votaci no pot ser nul';
     INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'BaixaVotacio',
          'idvotacio: ' || n_idvotacio,
          'RSP: '||rst);   
   COMMIT;
   WHEN OTHERS
   THEN
   ROLLBACK;
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:'
             || err_msg;
    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'BaixaVotacio',
          'idvotacio: ' || n_idvotacio,
          'RSP: '||rst);   
   COMMIT;
END baixaVotacio;
/
-- ------------------------------------------------------------------------------
/*****************************************************************************
			PROCEDIMENTS DE MODIFICACI
*****************************************************************************/

create or replace PROCEDURE modAmbit (
   n_idambit       IN       NUMBER,
   n_idvotacio     IN       NUMBER,
   n_idcodicens    IN       NUMBER,
   n_idtipuscens   IN       NUMBER,
   rst             OUT      VARCHAR2
)
IS   
/******************************************************************************
   modAmbit:  modifica un cens en una votacio.
                
   PARAMETRES:  n_idambit, n_idvotacio, n_idcodicens, n_idtipuscens, rst
   INPUT:       n_idambit, n_idvotacio, n_idcodicens, n_idtipuscens
   OUTPUT:      rst
   RETORNA:     'OK' si s'ha executat correctament, 'ERROR: Tipus d'Error' en
                cas contrari
   EXEMPLE D'US:     modAmbit(1,1,1, rst)
******************************************************************************/
   n_idambit_ins              NUMBER;
   n_idvotacio_ins            NUMBER;
   n_idcodicens_ins           NUMBER;
   n_idtipuscens_ins          NUMBER;
   n_estat_ins                NUMBER;
   err_msg                    VARCHAR2 (100);
   existe                     VARCHAR2(1);
   campsNuls                  EXCEPTION;
   CensInexistent             EXCEPTION;
   tipusCensInexistent        EXCEPTION;
   votacioInexistent          EXCEPTION;
   votacioOberta              EXCEPTION;
   votacioTancada             EXCEPTION;
   ambitduplicat              EXCEPTION;
   ambitInexistent            EXCEPTION;
  
BEGIN
   n_idambit_ins      := n_idambit;
   n_idvotacio_ins    := n_idvotacio;
   n_idcodicens_ins   := n_idcodicens;
   n_idtipuscens_ins  := n_idtipusCens;
      

  --verifiquem que el codi de la votacio, el codi del cens i tipus de cens no siguin nuls
  
  IF (n_idvotacio_ins IS NULL) OR (n_idcodicens_ins IS NULL) OR (n_idtipuscens_ins IS NULL) THEN 
    RAISE campsNuls;
  END IF; 
  
   --verifiquem l'mbit existeixi.
  BEGIN
       SELECT 'S'
         INTO EXISTE
         FROM AMBIT
         WHERE idambit          = n_idambit_ins
         AND ROWNUM               = 1;
      
       EXCEPTION
       WHEN NO_DATA_FOUND THEN
       RAISE ambitInexistent;
     END;
  
  -- Verifiquem que la votacio existeixi
  
  BEGIN
       SELECT 'S'
         INTO EXISTE
         FROM VOTACIO
         WHERE idvotacio          = n_idvotacio_ins
         AND ROWNUM               = 1;
                        
     EXCEPTION
       WHEN NO_DATA_FOUND THEN
          RAISE votacioInexistent;
     END;
  -- Verifiquem que el tipus de cens existeix
  BEGIN
  SELECT 'S'
         INTO EXISTE
         FROM TIPUSCENS
         WHERE idtipuscens        = n_idtipuscens_ins
         AND ROWNUM               = 1;
                     
     EXCEPTION
       WHEN NO_DATA_FOUND THEN
          RAISE tipusCensInexistent;
     END;
      
  -- Verifiquem que existeix el cens
  
  IF (n_idtipuscens=2) THEN   
   BEGIN
       SELECT 'S'
         INTO EXISTE
         FROM CENSESTATAL
         WHERE idpais             = n_idcodicens_ins
         AND ROWNUM               = 1;                        
                        
     EXCEPTION
       WHEN NO_DATA_FOUND THEN
          RAISE censInexistent;
     END;
     END IF;
     
    IF (n_idtipuscens=3) THEN   
   BEGIN
       SELECT 'S'
         INTO EXISTE
         FROM CENSREGIONAL
         WHERE idregio            = n_idcodicens_ins
         AND ROWNUM               = 1;         
               
     EXCEPTION
       WHEN NO_DATA_FOUND THEN
          RAISE censInexistent;
     END;
     END IF;
    
     IF (n_idtipuscens=4) THEN   
   BEGIN
       SELECT 'S'
         INTO EXISTE
         FROM CENSLOCAL
         WHERE idlocal            = n_idcodicens_ins
         AND ROWNUM               = 1;   
         
     EXCEPTION
       WHEN NO_DATA_FOUND THEN
          RAISE censInexistent;
     END;
     END IF;
     
  --verifiquem que la votacio no s'hagi donat d'alta previament en aquest cens.
  BEGIN
       SELECT 'S'
         INTO EXISTE
         FROM AMBIT
         WHERE idvotacio        = n_idvotacio_ins
         AND   codicens         = n_idcodicens_ins
         AND   idtipuscens      = n_idtipuscens_ins
         AND ROWNUM               = 1;
       
       RAISE ambitDuplicat;
       
      EXCEPTION
       WHEN NO_DATA_FOUND THEN
          NULL;
     
     END;
  
   -- verifiquem que la votaci no estigui oberta
   SELECT estatvotacio
   INTO n_estat_ins
   FROM VOTACIO
   WHERE idvotacio=n_idvotacio_ins;
   
   IF (n_estat_ins=2) THEN  RAISE votacioOberta;
   END IF;
   
   -- verifiquem que la votaci no estigui tancada
   IF (n_estat_ins=3) THEN  RAISE votacioTancada; 
   END IF;

    
   -- modifiquem l'Ambit
   UPDATE  AMBIT
   SET idvotacio    = n_idvotacio_ins, codicens     = n_idcodicens_ins, idtipuscens  = n_idtipuscens_ins
   WHERE idambit=n_idambit_ins;
   
   -- si ha finalitzat correctament retorna OK, en cas contrari gestionem els errors
   rst := 'OK';

    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'modAmbit',
          'idambit: '||n_idambit||', idVotacio: ' || n_idvotacio||', idcodicens:  ' || 
           n_idcodicens || ' ,idtipuscens: ' || n_idtipuscens, 
          'RSP: '||rst);   
   COMMIT;
EXCEPTION
   WHEN ambitDuplicat
   THEN
   ROLLBACK;
      rst := 'ERROR: La modificaci dona com a resultat un mbit ja existent';
      
    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'modAmbit',
          'idambit: '||n_idambit||', idVotacio: ' || n_idvotacio||', idcodicens:  ' || 
           n_idcodicens || ' ,idtipuscens: ' || n_idtipuscens, 
          'RSP: '||rst);   
   COMMIT;
   WHEN ambitInexistent
   THEN
   ROLLBACK;
      rst := 'ERROR: l ambit no existeix al sistema';
      
    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'modAmbit',
          'idambit: '||n_idambit||', idVotacio: ' || n_idvotacio||', idcodicens:  ' || 
           n_idcodicens || ' ,idtipuscens: ' || n_idtipuscens, 
          'RSP: '||rst);   
   COMMIT;
   WHEN censInexistent
   THEN
   ROLLBACK;
      rst := 'ERROR: El cens no existeix';
   
    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'modAmbit',
          'idambit: '||n_idambit||', idVotacio: ' || n_idvotacio||', idcodicens:  ' || 
           n_idcodicens || ' ,idtipuscens: ' || n_idtipuscens, 
          'RSP: '||rst);   
   COMMIT;
   WHEN tipusCensInexistent
   THEN
   ROLLBACK;
      rst := 'ERROR: El tipus de cens no existeix';
   
    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'modAmbit',
          'idambit: '||n_idambit||', idVotacio: ' || n_idvotacio||', idcodicens:  ' || 
           n_idcodicens || ' ,idtipuscens: ' || n_idtipuscens, 
          'RSP: '||rst);   
   COMMIT;
   WHEN votacioInexistent
   THEN
   ROLLBACK;
      rst := 'ERROR: La votaci no existeix';
   
    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'modAmbit',
          'idambit: '||n_idambit||', idVotacio: ' || n_idvotacio||', idcodicens:  ' || 
           n_idcodicens || ' ,idtipuscens: ' || n_idtipuscens, 
          'RSP: '||rst);   
   COMMIT;
   WHEN votacioOberta
   THEN
   ROLLBACK;
      rst := 'ERROR: La votaci ja est oberta';
   
    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'modAmbit',
          'idambit: '||n_idambit||', idVotacio: ' || n_idvotacio||', idcodicens:  ' || 
           n_idcodicens || ' ,idtipuscens: ' || n_idtipuscens, 
          'RSP: '||rst);   
   COMMIT;
   WHEN votacioTancada
   THEN
   ROLLBACK;
      rst := 'ERROR: La votaci ja est tancada';
   
    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'modAmbit',
          'idambit: '||n_idambit||', idVotacio: ' || n_idvotacio||', idcodicens:  ' || 
           n_idcodicens || ' ,idtipuscens: ' || n_idtipuscens, 
          'RSP: '||rst);   
   COMMIT;
   WHEN campsNuls
   THEN
   ROLLBACK;
      rst := 'ERROR: el codi de la votaci, el codi del cens i el tipus de cens no poden ser nuls';
    
    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'modAmbit',
          'idambit: '||n_idambit||', idVotacio: ' || n_idvotacio||', idcodicens:  ' || 
           n_idcodicens || ' ,idtipuscens: ' || n_idtipuscens, 
          'RSP: '||rst);   
   COMMIT;
   WHEN OTHERS
   THEN
   ROLLBACK;
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:'
             || err_msg;
   
   INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'modAmbit',
          'idambit: '||n_idambit||', idVotacio: ' || n_idvotacio||', idcodicens:  ' || 
           n_idcodicens || ' ,idtipuscens: ' || n_idtipuscens, 
          'RSP: '||rst);   
   COMMIT;
END modAmbit;
/
-- -----------------------------------------------------------------------------------------------

create or replace PROCEDURE modCensCiutada (
   n_idcensciutada IN       NUMBER,
   n_idciutada     IN       NUMBER,
   n_idcodicens    IN       NUMBER,
   n_idtipuscens   IN       NUMBER,
   rst             OUT      VARCHAR2
)
IS   
/******************************************************************************
   modCensCiutada:  Modifica un censciutada.
                
   PARAMETRES:  n_idciutada, n_idcodicens, n_idtipuscens, rst
   INPUT:       n_idciutada, n_idcodicens, n_idtipuscens
   OUTPUT:      rst
   RETORNA:     'OK' si s'ha executat correctament, 'ERROR: Tipus d'Error' en
                cas contrari
   EXEMPLE D'US:     modCensCiutada(1,1,1,1, rst)
******************************************************************************/
   n_idcensciutada_ins        NUMBER;
   n_idciutada_ins            NUMBER;
   n_idcodicens_ins           NUMBER;
   n_idtipuscens_ins          NUMBER;
   n_actiu_ins                NUMBER;
   err_msg                    VARCHAR2 (100);
   existe                     VARCHAR2(1);
   campsNuls                  EXCEPTION;
   CensInexistent             EXCEPTION;
   CiutadaInexistent          EXCEPTION;
   censCiutadaBaixa           EXCEPTION;
   censciutadaInexistent      EXCEPTION;
   censciutadaDuplicat        EXCEPTION;
   tipusCensInexistent        EXCEPTION;
  
BEGIN
   n_idcensciutada_ins := n_idcensciutada;
   n_idciutada_ins     := n_idciutada;
   n_idcodicens_ins    := n_idcodicens;
   n_idtipuscens_ins   := n_idtipusCens;
      

  --verifiquem que el codi del ciutada, el codi del cens i tipus de cens no siguin nuls
  
  IF (n_idciutada_ins IS NULL) OR (n_idcodicens_ins IS NULL) OR (n_idtipuscens_ins IS NULL) THEN 
    RAISE campsNuls;
  END IF; 
  
   -- Verifiquem que el ciutad existeixi
  
  BEGIN
       SELECT 'S'
         INTO EXISTE
         FROM CIUTADA
         WHERE idciutada          = n_idciutada
         AND ROWNUM               = 1;
                        
     EXCEPTION
       WHEN NO_DATA_FOUND THEN
          RAISE ciutadaInexistent;
     END;
 
     
  --verifiquem que el ciutada no estigui duplicat.
  BEGIN
       SELECT 'S'
         INTO EXISTE
         FROM CENSCIUTADANS
         WHERE idciutada        = n_idciutada
         AND   idcodicens       = n_idcodicens
         AND   idtipuscens      = n_idtipuscens
         AND ROWNUM               = 1;

        RAISE censCiutadaDuplicat;
        EXCEPTION
       WHEN NO_DATA_FOUND THEN
          NULL;
          
     END;
      
   -- Verifiquem que el censciutada existeixi
  
  BEGIN
       SELECT 'S'
         INTO EXISTE
         FROM CENSCIUTADANS
         WHERE idcensciutadans          = n_idcensciutada
         AND ROWNUM               = 1;
                        
     EXCEPTION
       WHEN NO_DATA_FOUND THEN
          RAISE censciutadaInexistent;
     END;
     
   --verifiquem que el censciutada no s'hagi donat de baixa.
   
  BEGIN
       SELECT actiu
         INTO n_actiu_ins
         FROM CENSCIUTADANS
         WHERE idcensciutadans    = n_idcensciutada;
                 
        IF (n_actiu_ins=0) THEN
        RAISE censCiutadaBaixa;
        END IF;
     END;
  -- Verifiquem que el tipus de cens existeix
  BEGIN
  SELECT 'S'
         INTO EXISTE
         FROM TIPUSCENS
         WHERE idtipuscens        = n_idtipuscens
         AND ROWNUM               = 1;
                     
     EXCEPTION
       WHEN NO_DATA_FOUND THEN
          RAISE tipusCensInexistent;
     END;
     
  -- Verifiquem que existeix el cens
  
  IF (n_idtipuscens=2) THEN   
   BEGIN
       SELECT 'S'
         INTO EXISTE
         FROM CENSESTATAL
         WHERE idpais             = n_idcodicens
         AND ROWNUM               = 1;
                
     EXCEPTION
       WHEN NO_DATA_FOUND THEN
          RAISE censInexistent;
     END;
     END IF;
     
    IF (n_idtipuscens=3) THEN   
   BEGIN
       SELECT 'S'
         INTO EXISTE
         FROM CENSREGIONAL
         WHERE idregio            = n_idcodicens
         AND ROWNUM               = 1;
       
     EXCEPTION
       WHEN NO_DATA_FOUND THEN
          RAISE censInexistent;
     END;
     END IF;
    
     IF (n_idtipuscens=4) THEN   
   BEGIN
       SELECT 'S'
         INTO EXISTE
         FROM CENSLOCAL
         WHERE idlocal            = n_idcodicens
         AND ROWNUM               = 1;   
           
     EXCEPTION
       WHEN NO_DATA_FOUND THEN
          RAISE censInexistent;
     END;
     END IF;

    
   -- modifiquem el cens
   
   UPDATE CENSCIUTADANS SET idciutada = n_idciutada_ins, idcodicens = n_idcodicens_ins,
                            idtipuscens = n_idtipuscens_ins 
          WHERE idcensciutadans =n_idcensciutada_ins;
   
   -- si ha finalitzat correctament retorna OK, en cas contrari gestionem els errors
   rst := 'OK';

    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'modCensCiutada',
          'idcensciutada: '||n_idcensciutada|| ',idCiutada: ' || n_idciutada||', idcodicens:  ' 
           || n_idcodicens || ' ,idtipuscens: ' || n_idtipuscens, 
          'RSP: '||rst);   
   COMMIT;
EXCEPTION
   WHEN censCiutadaBaixa
   THEN
   ROLLBACK;
      rst := 'ERROR: Aquest censciutada ha estat donat de baixa';
      
   INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'modCensCiutada',
          'idcensciutada: '||n_idcensciutada|| ',idCiutada: ' || n_idciutada||', idcodicens:  ' 
           || n_idcodicens || ' ,idtipuscens: ' || n_idtipuscens, 
          'RSP: '||rst);   
   COMMIT; 
   
   WHEN censCiutadaInexistent
   THEN
   ROLLBACK;
      rst := 'ERROR: Aquest censciutada no existeix al sistema';
      
    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'modCensCiutada',
          'idcensciutada: '||n_idcensciutada|| ',idCiutada: ' || n_idciutada||', idcodicens:  ' 
           || n_idcodicens || ' ,idtipuscens: ' || n_idtipuscens, 
          'RSP: '||rst);   
   COMMIT;
   
   WHEN censCiutadaDuplicat
   THEN
   ROLLBACK;
      rst := 'ERROR: la modificaci que es vol fer dona com a resultat un censciutada duplicat';
      
    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'modCensCiutada',
          'idcensciutada: '||n_idcensciutada|| ',idCiutada: ' || n_idciutada||', idcodicens:  ' 
           || n_idcodicens || ' ,idtipuscens: ' || n_idtipuscens, 
          'RSP: '||rst);   
   COMMIT;
      
   WHEN censInexistent
   THEN
   ROLLBACK;
      rst := 'ERROR: El cens no existeix';
   
    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'modCensCiutada',
          'idcensciutada: '||n_idcensciutada|| ',idCiutada: ' || n_idciutada||', idcodicens:  ' 
           || n_idcodicens || ' ,idtipuscens: ' || n_idtipuscens, 
          'RSP: '||rst);   
   COMMIT;  
      
   WHEN tipusCensInexistent
   THEN
   ROLLBACK;
      rst := 'ERROR: El tipus de cens no existeix';
   
    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'modCensCiutada',
          'idcensciutada: '||n_idcensciutada|| ',idCiutada: ' || n_idciutada||', idcodicens:  ' 
           || n_idcodicens || ' ,idtipuscens: ' || n_idtipuscens, 
          'RSP: '||rst);   
   COMMIT;     
   
   WHEN ciutadaInexistent
   THEN
   ROLLBACK;
      rst := 'ERROR: El ciutada no existeix';
   
   INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'modCensCiutada',
          'idcensciutada: '||n_idcensciutada|| ',idCiutada: ' || n_idciutada||', idcodicens:  ' 
           || n_idcodicens || ' ,idtipuscens: ' || n_idtipuscens, 
          'RSP: '||rst);   
   COMMIT;      
      
   WHEN campsNuls
   THEN
   ROLLBACK;
      rst := 'ERROR: el codi del censciutada, el codi del ciutada, el codi del cens i el tipus de cens no poden ser nuls';
      
  INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'modCensCiutada',
          'idcensciutada: '||n_idcensciutada|| ',idCiutada: ' || n_idciutada||', idcodicens:  ' 
           || n_idcodicens || ' ,idtipuscens: ' || n_idtipuscens, 
          'RSP: '||rst);   
   COMMIT;
   WHEN OTHERS
   THEN
   ROLLBACK;
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:'
             || err_msg;
   
    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'modCensCiutada',
          'idcensciutada: '||n_idcensciutada|| ',idCiutada: ' || n_idciutada||', idcodicens:  ' 
           || n_idcodicens || ' ,idtipuscens: ' || n_idtipuscens, 
          'RSP: '||rst);   
   COMMIT;
END modCensCiutada;
/
-- -------------------------------------------------------------------------------------------

create or replace PROCEDURE modCensEstatal (
   n_codipais        IN       VARCHAR2,
   n_nom             IN       VARCHAR2,
   rst               OUT      VARCHAR2
)
IS   
/******************************************************************************
   modCensEstatal:   modifica el nom d'un pas en un censEstatal
                
   PARAMETRES:  n_codipais,n_nom, rst
   INPUT:       n_codipais, n_nom
   OUTPUT:      rst
   RETORNA:     'OK' si s'ha executat correctament, 'ERROR: Tipus d'Error' en
                cas contrari
   EXEMPLE D'US:     modCensEstatal('ES','ESPANYA' rst)
******************************************************************************/
   n_codipais_ins              VARCHAR2(2);
   n_nom_ins                   VARCHAR2(50);
   err_msg                     VARCHAR2 (100);
   existe                      VARCHAR2(1);
   paramE                      VARCHAR2(500);
   campsNuls                   EXCEPTION;
   paisInexistent              EXCEPTION;
   
   
   
BEGIN

n_codipais_ins := n_codipais;
n_nom_ins      := n_nom;
    

  --verifiquem que el codi del pas i el nom no siguin nuls
  IF (n_codipais_ins IS NULL)OR (n_nom_ins IS NULL)  THEN 
    RAISE campsNuls;
  END IF; 
   
   -- verifiquem que el pas s'hagi donat d'alta al sistema
   
   BEGIN
       SELECT 'S'
         INTO EXISTE
         FROM CENSESTATAL
         WHERE codipais           = n_codipais_ins
         AND ROWNUM               = 1;
        
     EXCEPTION
       WHEN NO_DATA_FOUND THEN
          RAISE paisInexistent;
     END;
     
   -- modifiquem el CensEstatal
   UPDATE CENSESTATAL SET nom=n_nom_ins
          WHERE codipais=n_codipais_ins;
  
   -- si ha finalitzat correctament retorna OK, en cas contrari gestionem els errors
   rst := 'OK';

    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'modCensEstatal',
          'codiPais: ' || n_codipais|| ' ,nom: ' ||n_nom ,
          'RSP: '||rst);   
   COMMIT;
   
EXCEPTION
  
   WHEN paisInexistent
   THEN
      rst := 'ERROR: El pas no ha estat donat d alta al sistema';
   
    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'modCensEstatal',
          'codiPais: ' || n_codipais|| ' ,nom: ' ||n_nom ,
          'RSP: '||rst);   
   COMMIT;
  
   WHEN campsNuls
   THEN
      rst := 'ERROR: el codi del pas i el nom no poden ser nuls';
      
  INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'modCensEstatal',
          'codiPais: ' || n_codipais|| ' ,nom: ' ||n_nom ,
          'RSP: '||rst);   
   COMMIT;
   WHEN OTHERS
   THEN
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:'
             || err_msg;
   
   INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'modCensEstatal',
          'codiPais: ' || n_codipais|| ' ,nom: ' ||n_nom ,
          'RSP: '||rst);   
   COMMIT;
END modCensEstatal;
/
-- ----------------------------------------------------------------------

create or replace PROCEDURE modCensLocal (
   n_idlocalitat          IN       NUMBER,
   n_nom                  IN       VARCHAR2,
   rst                    OUT      VARCHAR2
)
IS   
/******************************************************************************
   modCensLocal:   Modifica el nom d'un cens Local al sistema
                
   PARAMETRES:  n_idlocalitat, n_nom, rst
   INPUT:       n_idlocalitat, n_nom
   OUTPUT:      rst
   RETORNA:     'OK' si s'ha executat correctament, 'ERROR: Tipus d'Error' en
                cas contrari
   EXEMPLE D'US:     modCensLocal(1,'Gerona', rst)
******************************************************************************/
   n_idlocalitat_ins           NUMBER;
   n_nom_ins                   VARCHAR2(50);
   err_msg                     VARCHAR2 (100);
   existe                      VARCHAR2(1);
   paramE                      VARCHAR2(500);
   campsNuls                   EXCEPTION;
   localitatInexistent         EXCEPTION;
  
BEGIN

n_idlocalitat_ins := n_idlocalitat;
n_nom_ins         := n_nom;
    

  --verifiquem que el codi de la localitat no sigui nul
  IF (n_idlocalitat_ins IS NULL) OR (n_nom_ins IS NULL) THEN 
    RAISE campsNuls;
  END IF; 
  
 
   -- verifiquem que la localitat s'hagi donat d'alta al sistema
   
   BEGIN
       SELECT 'S'
         INTO EXISTE
         FROM CENSLOCAL
         WHERE idlocal            = n_idlocalitat_ins
         AND ROWNUM               = 1;
        
     EXCEPTION
       WHEN NO_DATA_FOUND THEN
          RAISE localitatInexistent;
     END;
     
   -- Modifiquem el CensLocal
   UPDATE CENSLOCAL SET nom=n_nom_ins
          WHERE idlocal=n_idlocalitat_ins;
    
   -- si ha finalitzat correctament retorna OK, en cas contrari gestionem els errors
   rst := 'OK';

    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'modCensLocal',
          'idLocalitat: ' || n_idlocalitat|| ',nom: '|| n_nom ,
          'RSP: '||rst);
    COMMIT;
EXCEPTION
   
   WHEN localitatInexistent
   THEN
      rst := 'ERROR: La Localitat no ha estat donat d alta al sistema';
   
    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'modCensLocal',
          'idLocalitat: ' || n_idlocalitat|| ',nom: '|| n_nom ,
          'RSP: '||rst);
    COMMIT;
  
   WHEN campsNuls
   THEN
      rst := 'ERROR: el codi de la localitat i el nom no poden ser nuls';
      
     INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'modCensLocal',
          'idLocalitat: ' || n_idlocalitat|| ',nom: '|| n_nom ,
          'RSP: '||rst);
    COMMIT;
   WHEN OTHERS
   THEN
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:'
             || err_msg;
   
     INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'modCensLocal',
          'idLocalitat: ' || n_idlocalitat|| ',nom: '|| n_nom ,
          'RSP: '||rst);
    COMMIT;
END modCensLocal;
/
-- -----------------------------------------------------------------------------

create or replace PROCEDURE modCensRegional (
   n_idregio         IN       NUMBER,
   n_nom             IN       VARCHAR2,
   rst               OUT      VARCHAR2
)
IS   
/******************************************************************************
   modCensRegional:   modifica el nom d'una regi en un cens regional
                
   PARAMETRES:  n_idregio,n_nom, rst
   INPUT:       n_idregio, n_nom
   OUTPUT:      rst
   RETORNA:     'OK' si s'ha executat correctament, 'ERROR: Tipus d'Error' en
                cas contrari
   EXEMPLE D'US:     modCensRegional(1,'CATALUA' rst)
******************************************************************************/
   n_idregio_ins               NUMBER;
   n_nom_ins                   VARCHAR2(50);
   err_msg                     VARCHAR2 (100);
   existe                      VARCHAR2(1);
   paramE                      VARCHAR2(500);
   campsNuls                   EXCEPTION;
   regioInexistent             EXCEPTION;
   
   
  
BEGIN

n_idregio_ins := n_idregio;
n_nom_ins     := n_nom;
    

  -- verifiquem que el codi de la regio no sigui nul
  IF (n_idregio_ins IS NULL) OR (n_nom_ins IS NULL)  THEN 
    RAISE campsNuls;
  END IF; 
   
   -- verifiquem que la regio s'hagi donat d'alta al sistema
   
   BEGIN
       SELECT 'S'
         INTO EXISTE
         FROM CENSREGIONAL
         WHERE idregio            = n_idregio_ins
         AND ROWNUM               = 1;
        
     EXCEPTION
       WHEN NO_DATA_FOUND THEN
          RAISE regioInexistent;
     END;
     
   -- Modifiquem el CensRegional
   UPDATE CENSREGIONAL SET nom=n_nom_ins
          WHERE idregio=n_idregio_ins;
   
   -- si ha finalitzat correctament retorna OK, en cas contrari gestionem els errors
   rst := 'OK';

    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'modCensRegional',
          'idRegio: ' || n_idregio || ' ,nom: '||n_nom ,
          'RSP: '||rst);   
   COMMIT;
EXCEPTION
   
   WHEN regioInexistent
   THEN
      rst := 'ERROR: La regi no ha estat donat d alta al sistema';
   
    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'modCensRegional',
          'idRegio: ' || n_idregio || ' ,nom: '||n_nom ,
          'RSP: '||rst);   
   COMMIT;
   WHEN campsNuls
   THEN
      rst := 'ERROR: el codi de la regi i el nom no poden ser nuls';
      
    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'modCensRegional',
          'idRegio: ' || n_idregio || ' ,nom: '||n_nom ,
          'RSP: '||rst);   
   COMMIT;
   WHEN OTHERS
   THEN
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:'
             || err_msg;
   
    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'modCensRegional',
          'idRegio: ' || n_idregio || ' ,nom: '||n_nom ,
          'RSP: '||rst);   
   COMMIT;
END modCensRegional;
/
-- -------------------------------------------------------------------------------

create or replace PROCEDURE modCiutada (
   n_codiciutada     IN       VARCHAR2,
   n_datanaixement   IN       DATE,
   rst               OUT      VARCHAR2
)
IS   
/******************************************************************************
   modCiutada:  Modifica un ciutada canviant la data de naixement.
                
   PARAMETRES:  n_codiciutada, n_datanaixement, rst
   INPUT:       n_codiciutada, n_datanaixement
   OUTPUT:      rst
   RETORNA:     'OK' si s'ha executat correctament, 'ERROR: Tipus d'Error' en
                cas contrari
   EXEMPLE D'US:     modCiutada('AMH658564BS', '28/02/1975' rst)
******************************************************************************/
   n_codiciutada_ins           VARCHAR2 (11);
   n_datanaixement_ins         DATE;
   err_msg                     VARCHAR2 (100);
   existe                      VARCHAR2(1);
   n_actiu_ins                 NUMBER;
   campsNuls                   EXCEPTION;
   ciutadaInexistent           EXCEPTION;
   ciutadaMenorEdat            EXCEPTION;
  
BEGIN
   n_codiciutada_ins   := n_codiciutada;
   n_datanaixement_ins := n_datanaixement;
   
      

  --verifiquem que el codi del ciutada  no sigui nul
  IF (n_codiciutada_ins IS NULL)OR (n_datanaixement_ins IS NULL) THEN 
    RAISE campsNuls;
  END IF; 
 
  --verifiquem que el ciutada existeixi.
  BEGIN
       SELECT 'S'
         INTO EXISTE
         FROM CIUTADA
         WHERE codiciutada        = n_codiciutada
         AND ROWNUM               = 1;
      
     EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RAISE ciutadaInexistent;
     END;
     
   -- verifiquem que el ciutada s major d'edat i per tant pot votar
  
  IF ((MONTHS_BETWEEN(sysdate, n_datanaixement_ins)/12)<18) THEN
  RAISE ciutadaMenorEdat;
  END IF;
  
   -- Modifiquem el ciutada
   
   UPDATE CIUTADA SET datanaixement=n_datanaixement_ins
          WHERE codiciutada = n_codiciutada_ins;
  
   -- si ha finalitzat correctament retorna OK, en cas contrari gestionem els errors
   rst := 'OK';

    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'modCiutada',
          'codiCiutada: ' || n_codiciutada||', datanaixement: '||n_datanaixement,
          'RSP: '||rst);   
      COMMIT;
EXCEPTION
   WHEN ciutadaInexistent
   THEN
      rst := 'ERROR: Aquest ciutada no ha estat donat d alta al sistema';
   
    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'modCiutada',
          'codiCiutada: ' || n_codiciutada||', datanaixement: '||n_datanaixement,
          'RSP: '||rst);   
      COMMIT;   
      
  WHEN ciutadaMenorEdat
   THEN
      rst := 'ERROR: la data de naixement implica que el ciutad es menor d edat';
   
    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'modCiutada',
          'codiCiutada: ' || n_codiciutada||', datanaixement: '||n_datanaixement,
          'RSP: '||rst);   
      COMMIT;  
    
   WHEN campsNuls
   THEN
      rst := 'ERROR: el codi del ciutada i la data de naixement no poden ser nuls';
      
   INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'modCiutada',
          'codiCiutada: ' || n_codiciutada||', datanaixement: '||n_datanaixement,
          'RSP: '||rst);   
      COMMIT;
   WHEN OTHERS
   THEN
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:'
             || err_msg;
   
   INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'modCiutada',
          'codiCiutada: ' || n_codiciutada||', datanaixement: '||n_datanaixement,
          'RSP: '||rst);   
      COMMIT;
END modCiutada;
/
-- --------------------------------------------------------------------------------

create or replace PROCEDURE modOpcioVot (
   
   n_idvotacio       IN       NUMBER,
   n_numopcio        IN       NUMBER,
   n_descripcio      IN       VARCHAR2,
   rst               OUT      VARCHAR2
)
IS   
/******************************************************************************
   modOpcioVot:  Modifica una opci de vot d'una votaci.
                
   PARAMETRES:  n_idvotacio, n_numvotacio, n_descripci rst
   INPUT:       n_idvotacio, n_numvotacio
   OUTPUT:      rst
   RETORNA:     'OK' si s'ha executat correctament, 'ERROR: Tipus d'Error' en
                cas contrari
   EXEMPLE D'US:     modOpcioVot(1,1,'descripci de la opci de vot' rst)
******************************************************************************/
   n_idvotacio_ins             NUMBER;
   n_numopcio_ins              NUMBER;
   n_descripcio_ins            VARCHAR2(100);
   n_estat_ins                 NUMBER;
   err_msg                     VARCHAR2 (100);
   existe                      VARCHAR2(1);
   campsNuls                   EXCEPTION;
   votacioInexistent           EXCEPTION;
   votacioTancada              EXCEPTION;
   votacioOberta               EXCEPTION;
  
BEGIN
   n_numopcio_ins    :=n_numopcio;
   n_idvotacio_ins   :=n_idvotacio;
   n_descripcio_ins  := n_descripcio;
       

  --verifiquem que el codi de la votacio i el numero de la opci i la descripci no siguin nuls
  IF (n_numopcio IS NULL) OR (n_idvotacio_ins IS NULL) OR (n_descripcio IS NULL)THEN 
    RAISE campsNuls;
  END IF; 
  
  --verifiquem que la votacio s'hagi donat d alta al sistema.
  BEGIN
       SELECT 'S'
         INTO EXISTE
         FROM VOTACIO
         WHERE idvotacio          = n_idvotacio
         AND ROWNUM               = 1;

        
     EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RAISE votacioInexistent;
     END;
     
   -- verifiquem que la votaci no estigui oberta
   SELECT estatvotacio
   INTO n_estat_ins
   FROM VOTACIO
   WHERE idvotacio=n_idvotacio_ins;
   
   IF (n_estat_ins=2) THEN  RAISE votacioOberta;
   END IF;
   
   -- verifiquem que la votaci no estigui tancada
   IF (n_estat_ins=3) THEN  RAISE votacioTancada; 
   END IF;
    
   -- Modifiquem la opci
   UPDATE  OPCIOVOT SET descripcio = n_descripcio_ins
   WHERE idvotacio = n_idvotacio_ins
   AND   numopcio  = n_numopcio_ins ;
   
   -- si ha finalitzat correctament retorna OK, en cas contrari gestionem els errors
   rst := 'OK';

    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'ModOpcio',
          'numopcio: ' || n_numopcio ||', idVotacio: ' || n_idvotacio || ', descripcio: '|| n_descripcio,
          'RSP: '||rst);   
   COMMIT;
EXCEPTION
   WHEN votacioInexistent
   THEN
      rst := 'ERROR: la votaci no existeix';
   
   INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'ModOpcio',
          'numopcio: ' || n_numopcio ||', idVotacio: ' || n_idvotacio || ', descripcio: '|| n_descripcio,
          'RSP: '||rst);   
   COMMIT;
   WHEN votacioOberta
   THEN
      rst := 'ERROR: la votaci ja est oberta';
   
   INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'ModOpcio',
          'numopcio: ' || n_numopcio ||', idVotacio: ' || n_idvotacio || ', descripcio: '|| n_descripcio,
          'RSP: '||rst);   
   COMMIT;
   WHEN votacioTancada
   THEN
      rst := 'ERROR: la votaci ja est tancada';
   
   INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'ModOpcio',
          'numopcio: ' || n_numopcio ||', idVotacio: ' || n_idvotacio || ', descripcio: '|| n_descripcio,
          'RSP: '||rst);   
   COMMIT;
   WHEN campsNuls
   THEN
      rst := 'ERROR: el codi de la votacio, el nmero de la opci i la descripci no poden ser nuls';
      
  INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'ModOpcio',
          'numopcio: ' || n_numopcio ||', idVotacio: ' || n_idvotacio || ', descripcio: '|| n_descripcio,
          'RSP: '||rst);   
   COMMIT;
   WHEN OTHERS
   THEN
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:'
             || err_msg;
   
   INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'ModOpcio',
          'numopcio: ' || n_numopcio ||', idVotacio: ' || n_idvotacio || ', descripcio: '|| n_descripcio,
          'RSP: '||rst);   
   COMMIT;
END modOpcioVot;
/
-- --------------------------------------------------------------------------------------------

create or replace PROCEDURE modVotacio (
   n_idvotacio      IN       NUMBER,
   n_titol          IN       VARCHAR2,
   n_descripcio     IN       VARCHAR2,
   n_datainici      IN       DATE,
   n_datafi         IN       DATE,
   n_datapublicacio IN       DATE,
   n_president      IN       NUMBER,
   n_web            IN       VARCHAR2,
   rst              OUT      VARCHAR2
)
IS   
/******************************************************************************
   modVotacio:  modifica les dades d'una votaci
                
   PARAMETRES:  n_idvotacio, n_titol, n_descripcio, n_datainici, n_datafi, n_datapublicacio, n_president, n_web, rst
   INPUT:       n_idvotacio, n_titol, n_descripcio, n_datainici, n_datafi, n_datapublicacio, n_president, n_web
   OUTPUT:      rst
   RETORNA:     'OK' si s'ha executat correctament, 'ERROR: Tipus d'Error' en
                cas contrari
   EXEMPLE D'US:    modVotacio(1,'votacio 2491','descripcio de la votacio',
                                  '21/01/2012','29/01/2012', '10/01/2012',
                                  125, 'http://www.planaweb.org', rst)
******************************************************************************/
   n_idvotacio_ins                NUMBER;
   n_titol_ins                    VARCHAR2(50);
   n_descripcio_ins               VARCHAR2(100);
   n_datainici_ins                DATE;
   n_datafi_ins                   DATE;
   n_datapublicacio_ins           DATE;
   n_president_ins                NUMBER;
   n_web_ins                      VARCHAR2(50);
   n_estat_ins                    NUMBER;
   err_msg                        VARCHAR2 (100);
   existe                         VARCHAR2(1);
   campsNuls                      EXCEPTION;
   votacioInexistent              EXCEPTION;  
   votacioOberta                  EXCEPTION;
   votacioTancada                 EXCEPTION;
   presidentInexistent            EXCEPTION;
   datapublicacioerronea          EXCEPTION;
   datafierronea                  EXCEPTION;
  
BEGIN
   n_idvotacio_ins        := n_idvotacio;
   n_titol_ins            := n_titol;
   n_descripcio_ins       := n_descripcio;
   n_datainici_ins        := n_datainici;
   n_datafi_ins           := n_datafi;
   n_datapublicacio_ins   := n_datapublicacio;
   n_president_ins        := n_president;
   n_web_ins              := n_web;
  
  -- verifiquem que el codi de la votaci,el ttol, la descripcio, la data d'inici, 
  -- la de fi i la de publicaci i el president no siguin nuls.
  
  IF (n_idvotacio_ins IS NULL) OR (n_titol_ins IS NULL) OR (n_descripcio_ins IS NULL) OR (n_datainici_ins IS NULL) 
      OR (n_datafi_ins IS NULL) OR (n_datapublicacio_ins IS NULL)OR (n_president_ins IS NULL) THEN 
    RAISE campsNuls;
  END IF; 
  
   --verifiquem que la votacio s'hagi donat d alta al sistema.
  BEGIN
       SELECT 'S'
         INTO EXISTE
         FROM VOTACIO
         WHERE idvotacio          = n_idvotacio_ins
         AND ROWNUM               = 1;

        
     EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RAISE votacioInexistent;
     END;
  
    -- verifiquem que la votaci no estigui oberta
   SELECT estatvotacio
   INTO n_estat_ins
   FROM VOTACIO
   WHERE idvotacio=n_idvotacio_ins;
   
   IF (n_estat_ins=2) THEN  RAISE votacioOberta;
   END IF;
   
   -- verifiquem que la votaci no estigui tancada
   IF (n_estat_ins=3) THEN  RAISE votacioTancada; 
   END IF;
   
    -- Verifiquem que existeix el ciutada que far el paper de president.
    
   BEGIN
       SELECT 'S'
         INTO EXISTE
         FROM CIUTADA
         WHERE idciutada          = n_president
         AND ROWNUM               = 1;                          
                        
     EXCEPTION
       WHEN NO_DATA_FOUND THEN
          RAISE presidentInexistent;
     END;
     
  -- Verifiquem que la data de puclicaci es com a mnim una setmana abans que la d'inici de la votaci
  
  IF (n_datainici<=n_datapublicacio+7) THEN
   RAISE datapublicacioerronea;
     END IF;
     
  -- Verifiquem que la data de inici sigui anterior a la data de fi de la votacio
  
  IF (n_datainici>=n_datafi) THEN
   RAISE datafierronea;
     END IF;
  
  -- Si ha passat les verificacions actualitzem la votaci.
  
  UPDATE  VOTACIO 
  SET  titol                = n_titol_ins,
       descripcio           = n_descripcio_ins,
       datainici            = n_datainici_ins,
       datafi               = n_datafi_ins,
       datapublicacio       = n_datapublicacio_ins,
       president            = n_president_ins,
       web                  = n_web_ins 
  WHERE idvotacio=n_idvotacio_ins;
  
      
  -- si ha finalitzat correctament retorna OK, en cas contrari gestionem els errors
   rst := 'OK';

    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'ModVotacio',
          'idvotacio: ' || n_idvotacio || 'ttol: ' || n_titol||', descripcio:  ' || n_descripcio ||
          ' ,datainici: ' || to_char(n_datainici)||' datafi : ' ||to_char(n_datafi)|| ' datapublicacio: '
          ||to_char(n_datapublicacio)|| ' president: '||n_president|| ' web: ' ||n_web,
          'RSP: '||rst);   
   COMMIT;
EXCEPTION        
     WHEN votacioOberta
   THEN
   ROLLBACK;
      rst := 'ERROR: La votaci ja est oberta';
   
   INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'ModVotacio',
          'idvotacio: ' || n_idvotacio || 'ttol: ' || n_titol||', descripcio:  ' || n_descripcio ||
          ' ,datainici: ' || to_char(n_datainici)||' datafi : ' ||to_char(n_datafi)|| ' datapublicacio: '
          ||to_char(n_datapublicacio)|| ' president: '||n_president|| ' web: ' ||n_web,
          'RSP: '||rst);   
   COMMIT;
    WHEN votacioTancada
   THEN
   ROLLBACK;
      rst := 'ERROR: la votacio est tancada';
   
   INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'ModVotacio',
          'idvotacio: ' || n_idvotacio || 'ttol: ' || n_titol||', descripcio:  ' || n_descripcio ||
          ' ,datainici: ' || to_char(n_datainici)||' datafi : ' ||to_char(n_datafi)|| ' datapublicacio: '
          ||to_char(n_datapublicacio)|| ' president: '||n_president|| ' web: ' ||n_web,
          'RSP: '||rst);   
   COMMIT;
   WHEN votacioInexistent
   THEN
   ROLLBACK;
      rst := 'ERROR: la votaci no est donada d alta al sistema';
   
  INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'ModVotacio',
          'idvotacio: ' || n_idvotacio || 'ttol: ' || n_titol||', descripcio:  ' || n_descripcio ||
          ' ,datainici: ' || to_char(n_datainici)||' datafi : ' ||to_char(n_datafi)|| ' datapublicacio: '
          ||to_char(n_datapublicacio)|| ' president: '||n_president|| ' web: ' ||n_web,
          'RSP: '||rst);   
   COMMIT;
   WHEN presidentInexistent
   THEN
   ROLLBACK;
      rst := 'ERROR: El ciutad que es vol introduir com a president no existeix';
   
    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'ModVotacio',
          'idvotacio: ' || n_idvotacio || 'ttol: ' || n_titol||', descripcio:  ' || n_descripcio ||
          ' ,datainici: ' || to_char(n_datainici)||' datafi : ' ||to_char(n_datafi)|| ' datapublicacio: '
          ||to_char(n_datapublicacio)|| ' president: '||n_president|| ' web: ' ||n_web,
          'RSP: '||rst);   
   COMMIT;
   WHEN datapublicacioerronea
   THEN
   ROLLBACK;
      rst := 'ERROR: la data de publicaci ha de ser com a mnim una setmana abans que la d inici de la votacio';
   
    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'ModVotacio',
          'idvotacio: ' || n_idvotacio || 'ttol: ' || n_titol||', descripcio:  ' || n_descripcio ||
          ' ,datainici: ' || to_char(n_datainici)||' datafi : ' ||to_char(n_datafi)|| ' datapublicacio: '
          ||to_char(n_datapublicacio)|| ' president: '||n_president|| ' web: ' ||n_web,
          'RSP: '||rst);   
   COMMIT;
   WHEN datafierronea
   THEN
   ROLLBACK;
      rst := 'ERROR: la data de finalitzaci no pot ser anterior ni igual que la d inici de la votacio';
   
  INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'ModVotacio',
          'idvotacio: ' || n_idvotacio || 'ttol: ' || n_titol||', descripcio:  ' || n_descripcio ||
          ' ,datainici: ' || to_char(n_datainici)||' datafi : ' ||to_char(n_datafi)|| ' datapublicacio: '
          ||to_char(n_datapublicacio)|| ' president: '||n_president|| ' web: ' ||n_web,
          'RSP: '||rst);   
   COMMIT;
   WHEN campsNuls
   THEN
   ROLLBACK;
      rst := 'ERROR: hi ha camps nuls que no poden ser nuls';
   INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'ModVotacio',
          'idvotacio: ' || n_idvotacio || 'ttol: ' || n_titol||', descripcio:  ' || n_descripcio ||
          ' ,datainici: ' || to_char(n_datainici)||' datafi : ' ||to_char(n_datafi)|| ' datapublicacio: '
          ||to_char(n_datapublicacio)|| ' president: '||n_president|| ' web: ' ||n_web,
          'RSP: '||rst);   
   COMMIT;
   WHEN OTHERS
   THEN
   ROLLBACK;
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:'
             || err_msg;
    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'ModVotacio',
          'idvotacio: ' || n_idvotacio || 'ttol: ' || n_titol||', descripcio:  ' || n_descripcio ||
          ' ,datainici: ' || to_char(n_datainici)||' datafi : ' ||to_char(n_datafi)|| ' datapublicacio: '
          ||to_char(n_datapublicacio)|| ' president: '||n_president|| ' web: ' ||n_web,
          'RSP: '||rst);   
   COMMIT;
      
END modVotacio;
/
-- ---------------------------------------------------------------------------------------------

/***********************************************************************************************
				PROCEDIMENTS DE FUNCIONAMENT
***********************************************************************************************/

create or replace PROCEDURE tancarvotacio (
   n_idvotacio         IN       NUMBER,
   rst                 OUT      VARCHAR2
)
IS   
/******************************************************************************
   votar:  tanca la votaci i fa el recompte dels vots
                
   PARAMETRES:  n_idvotacio,rst
   INPUT:       n_idvotacio
   OUTPUT:      rst
   RETORNA:     'OK' si s'ha executat correctament, 'ERROR: Tipus d'Error' en
                cas contrari
   EXEMPLE D'US:     tancarvotacio(1, rst)
******************************************************************************/
   n_idvotacio_ins             NUMBER;
   n_numvotants_ins            NUMBER;
   n_datafi_ins                DATE;
   n_votsEmesos_ins            NUMBER;
   n_opcio_ins                 NUMBER;
   n_percentatge_ins           NUMBER;
   n_opcioguanyadora_ins       NUMBER;
   n_peropcioguanyadora_ins    NUMBER;
   n_opciomenysvotada_ins      NUMBER;
   n_peropciomenysvotada_ins   NUMBER;
   n_idpais_ins                NUMBER;
   err_msg                     VARCHAR2 (100);
   existe                      VARCHAR2(1);
   campsNuls                   EXCEPTION;
   votacioInexistent           EXCEPTION;
   votacioForadeDates          EXCEPTION;
   votacioTancada              EXCEPTION;
   
  
BEGIN
   n_idvotacio_ins       :=n_idvotacio;
        
  --verifiquem que el codi de la votacio  , no siguin nul
  
  IF  (n_idvotacio_ins IS NULL) THEN 
    RAISE campsNuls;
  END IF; 
  
   -- Verifiquem que la votaci existeixi
  
  BEGIN
       SELECT 'S'
         INTO EXISTE
         FROM VOTACIO
         WHERE idvotacio        = n_idvotacio
         AND ROWNUM             = 1;

       EXCEPTION
        WHEN NO_DATA_FOUND THEN
             RAISE votacioInexistent;
      END;       
       
  -- Verifiquem que  la votacio estigui oberta    
  BEGIN   
       SELECT datafi
         INTO n_datafi_ins
         FROM votacio
         WHERE idvotacio = n_idvotacio;

     END;

  IF sysdate < n_datafi_ins THEN
     RAISE votacioForadeDates;
  END IF;
  
  --verifiquem que la votacio no s'hagi tancat previament.
  BEGIN
       SELECT 'S'
         INTO EXISTE
         FROM VOTACIOTANCADA
         WHERE idvotacio        = n_idvotacio
         AND ROWNUM               = 1;

        RAISE votacioTancada;
     EXCEPTION
        WHEN NO_DATA_FOUND THEN
           NULL;
     END;
    
   -- fem el recompte dels vots i els introduim a la taula d'opcions votacio tancada
   BEGIN
   
   INSERT INTO  OPCIONSVOTACIOTANCADA (idvotacio,numopcio, numvots, participacio)
    (SELECT idvotacio,numopcio, COUNT(numopcio)AS numvots,
    (SELECT COUNT (idvotacio) FROM VOTEMES WHERE idvotacio=n_idvotacio_ins
            GROUP BY idvotacio)AS participacio
     FROM VOTEMES
    WHERE idvotacio=  n_idvotacio_ins
    GROUP BY numopcio, idvotacio);
    COMMIT;
      
   END;
   -- cerquem el nombre de votants, els vots emesos, el percentatge de participaci
  
   BEGIN
   SELECT numtotalsciutadans
   INTO n_numvotants_ins
   FROM VOTACIO
   WHERE idvotacio= n_idvotacio_ins;
   END;
   
   BEGIN
   SELECT COUNT (idvotacio)
   INTO n_votsEmesos_ins
   FROM VOTEMES
   WHERE idvotacio=n_idvotacio_ins
   GROUP BY idvotacio;
   
   n_percentatge_ins:= TRUNC ((n_votsEmesos_ins/n_numvotants_ins)*100, 2);
      
   SELECT numopcio, numvots
   INTO n_opcioguanyadora_ins, n_peropcioguanyadora_ins
   FROM OPCIONSVOTACIOTANCADA
   WHERE idvotacio=n_idvotacio_ins
   AND  numvots= (SELECT MAX(numvots)FROM opcionsvotaciotancada);
     
   
   n_peropcioguanyadora_ins:= TRUNC ((n_peropcioguanyadora_ins/n_votsEmesos_ins)*100, 2);
   
   SELECT numopcio, numvots
   INTO n_opciomenysvotada_ins, n_peropciomenysvotada_ins
   FROM OPCIONSVOTACIOTANCADA
   WHERE idvotacio=n_idvotacio_ins
   AND  numvots= (SELECT MIN(numvots)FROM opcionsvotaciotancada);
   
   
   n_peropciomenysvotada_ins:= TRUNC((n_peropciomenysvotada_ins/n_votsEmesos_ins)*100,2);
   
   END;  
 
  
  INSERT INTO VOTACIOTANCADA(nombrevotants, idvotacio, votsemesos, pparticipacio, opcioguanyadora,
              popguanyadora, opmenysvotada, popmenysvotada) 
         VALUES ( n_numvotants_ins,n_idvotacio_ins, n_votsEmesos_ins, n_percentatge_ins, n_opcioguanyadora_ins,
                  n_peropcioguanyadora_ins, n_opciomenysvotada_ins, n_peropciomenysvotada_ins );
   
  -- Canvia l'estat de la votaci a tancada (3)
 
  UPDATE VOTACIO SET estatvotacio=3
        WHERE idvotacio=n_idvotacio_ins;
 
  -- Omple les dades Estadistiques de la Estadistica 1 : votacions tancades en un any concret  
  actualitzaE1(n_idvotacio_ins, rst);
  
  -- Omple les dades Estadistiques de la Estadistica 2 : mitjana de participaci de les votacions tancades en un any concret  
  actualitzaE2(n_idvotacio_ins, rst);
  
   -- si ha finalitzat correctament retorna OK, en cas contrari gestionem els errors
   
   rst := 'OK';

 INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
  VALUES (SYSDATE,'TancarVotacio',
          'idVotacio: ' || n_idvotacio,
          'RSP: '||rst);  
   COMMIT;
   
EXCEPTION
        
   WHEN votacioInexistent
     THEN
      rst := 'ERROR: La votacio no existeix';
   
  INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
  VALUES (SYSDATE,'TancarVotacio',
          'idVotacio: ' || n_idvotacio,
          'RSP: '||rst);   
      COMMIT;   
      
   WHEN campsNuls
    THEN
      rst := 'ERROR: el codi  de la votaci no pot ser nul';
      
  INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
  VALUES (SYSDATE,'TancarVotacio',
          'idVotacio: ' || n_idvotacio,
          'RSP: '||rst);   
      COMMIT;
 
   WHEN votacioForaDeDates
   THEN
      rst := 'ERROR: La votacio no ha finalitzat, o encara est oberta';
   
  INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
  VALUES (SYSDATE,'TancarVotacio',
          'idVotacio: ' || n_idvotacio,
          'RSP: '||rst);   
      COMMIT; 
      
      WHEN votacioTancada
   THEN
      rst := 'ERROR: La votacio ja est tancada';
   
  INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
  VALUES (SYSDATE,'TancarVotacio',
          'idVotacio: ' || n_idvotacio,
          'RSP: '||rst);   
      COMMIT; 
      
   WHEN OTHERS
   THEN
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:'
             || err_msg;
   
  INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
  VALUES (SYSDATE,'TancarVotacio',
          'idVotacio: ' || n_idvotacio,
          'RSP: '||rst);   
      COMMIT;
END tancarvotacio;
/
-- -------------------------------------------------------------------------------

create or replace PROCEDURE votar (
   n_votacio         IN       NUMBER,
   n_numopcio        IN       NUMBER,
   n_idciutada       IN       NUMBER,
   rst               OUT      VARCHAR2
)
IS   
/******************************************************************************
   votar:  introdueix un vot al sistema
                
   PARAMETRES:  n_votacio,n_opcio,  n_idciutada,rst
   INPUT:       n_votacio,n_opcio,  n_idciutada
   OUTPUT:      rst
   RETORNA:     'OK' si s'ha executat correctament, 'ERROR: Tipus d'Error' en
                cas contrari
   EXEMPLE D'US:     votar(1,1,1 rst)
******************************************************************************/
   n_idciutada_ins             NUMBER;
   n_votacio_ins               NUMBER;
   n_numopcio_ins              NUMBER;
   n_datainici_ins             DATE;
   n_datafi_ins                DATE;
   n_cens_ins                  NUMBER;
   n_tipus_cens_ins            NUMBER;
   n_idpais_ins                VARCHAR2(2);
   n_maxparticipacio_ins       NUMBER;
   n_actiu_ins                 NUMBER;
   err_msg                     VARCHAR2 (100);
   existe                      VARCHAR2(1);
   campsNuls                   EXCEPTION;
   ciutadaInexistent           EXCEPTION;
   ciutadaBaixa                EXCEPTION;
   votacioInexistent           EXCEPTION;
   opcioInexistent             EXCEPTION;
   noConvocat                  EXCEPTION;
   votJaEmes                   EXCEPTION;
   votacioForadeDates          EXCEPTION;
  
BEGIN
   n_idciutada_ins     :=n_idciutada;
   n_votacio_ins       :=n_votacio;
   n_numopcio_ins      :=n_numopcio;

     

  --verifiquem que el codi del ciutada, el codi de la votacio  i el de la opcio, no siguin nuls
  
  IF (n_idciutada_ins IS NULL) OR (n_votacio_ins IS NULL) OR (n_numopcio_ins IS NULL) THEN 
    RAISE campsNuls;
  END IF; 
  
  -- Verifiquem que el ciutad existeixi
   BEGIN
       SELECT 'S'
         INTO EXISTE
         FROM CIUTADA
         WHERE idciutada        = n_idciutada
         AND ROWNUM             = 1;
       
        SELECT actiu
         INTO n_actiu_ins
         FROM CIUTADA
         WHERE idciutada        = n_idciutada;
         
         IF (n_actiu_ins=0) THEN RAISE ciutadaBaixa;
         END IF;
         
       EXCEPTION
        WHEN NO_DATA_FOUND THEN
             RAISE ciutadaInexistent;
  END;
  -- Verifiquem que la votaci existeixi
  
  BEGIN
       SELECT 'S'
         INTO EXISTE
         FROM VOTACIO
         WHERE idvotacio        = n_votacio
         AND ROWNUM             = 1;

       EXCEPTION
        WHEN NO_DATA_FOUND THEN
             RAISE votacioInexistent;
      END;       
  -- Verifiquem que la opci existeixi
  BEGIN
       SELECT 'S'
         INTO EXISTE
         FROM OPCIOVOT
         WHERE idvotacio        = n_votacio
         AND numopcio           = n_numopcio
         AND ROWNUM             = 1;

       EXCEPTION
        WHEN NO_DATA_FOUND THEN
             RAISE opcioInexistent;
  
  END;
  -- Verifiquem que el ciutada estigui convocat a la votacio
   BEGIN
       SELECT 'S'
         INTO EXISTE
         FROM VOTACIONSCIUTADA
         WHERE idvotacio         = n_votacio
         AND idciutada           = n_idciutada
         AND ROWNUM              = 1;

       EXCEPTION
        WHEN NO_DATA_FOUND THEN
             RAISE noConvocat;
  
   END;
  --verifiquem que el ciutada no hagi votat en aquesta votaci previament.
  BEGIN
       SELECT 'S'
         INTO EXISTE
         FROM VOTEMES
         WHERE idciutada        = n_idciutada
         AND idvotacio          = n_votacio
         AND ROWNUM             = 1;

        RAISE votJaEmes;
     EXCEPTION
        WHEN NO_DATA_FOUND THEN
           NULL;
     END;
     
  -- Verifiquem que  la votacio estigui oberta    
  BEGIN   
       SELECT datainici,datafi
         INTO n_datainici_ins, n_datafi_ins
         FROM votacio
         WHERE idvotacio = n_votacio;

     END;

  IF SYSDATE NOT BETWEEN n_datainici_ins AND n_datafi_ins THEN
      RAISE votacioForadeDates;
  END IF;
    
   -- afegim el vot   
   INSERT INTO VOTEMES(idvotacio, numopcio, idciutada) VALUES ( n_votacio_ins, n_numopcio_ins, n_idciutada_ins);
   
   -- Acualitzem les dades de participacio
   UPDATE VOTACIONSCIUTADA set PARTICIPACIO=1 
   WHERE idvotacio=n_votacio_ins AND idciutada=n_idciutada_ins;
   
   UPDATE CIUTADA SET NUMVOTSEMESOS=NUMVOTSEMESOS+1
   WHERE idciutada=n_idciutada_ins;
   
   -- Actualitzem les dades estadstiques Estadstica 10
   BEGIN
     SELECT 'S'
         INTO EXISTE
         FROM EST10
         WHERE  idtipuse       = 10
         AND    ciutada        = n_idciutada_ins
         AND ROWNUM            = 1;
         
               
         UPDATE EST10 set nummaximvotacions=nummaximvotacions+1
         WHERE  idtipuse     = 10
         AND    ciutada      = n_idciutada_ins;               
         
     EXCEPTION
       WHEN NO_DATA_FOUND THEN
          
     SELECT *
        INTO n_idpais_ins
        FROM(SELECT idcodicens
        FROM CENSCIUTADANS
        WHERE idciutada     = n_idciutada_ins
        AND   idtipuscens   = 2
        ORDER BY idcodicens DESC)
      WHERE ROWNUM=1;
          
          INSERT INTO EST10 (idtipuse, nummaximvotacions, ciutada, pais) 
                 VALUES (10, 1, n_idciutada_ins, n_idpais_ins);
                 
                 END;
   
   -- si ha finalitzat correctament retorna OK, en cas contrari gestionem els errors
   rst := 'OK';

    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'Votar',
          'codiVotacio: ' || n_votacio||', numOpcio:  ' || n_numopcio|| ' idCiutada: ' || n_idciutada,
          'RSP: '||rst);   
   COMMIT;
   
EXCEPTION
   WHEN ciutadaInexistent
   THEN
      rst := 'ERROR: El ciutada no existeix';
   
    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'Votar',
          'codiVotacio: ' || n_votacio||', numOpcio:  ' || n_numopcio|| ' idCiutada: ' || n_idciutada,
          'RSP: '||rst);   
      COMMIT;       
      
   WHEN ciutadaBaixa
   THEN
      rst := 'ERROR: El ciutada ha estat donat de baixa';
   
    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'Votar',
          'codiVotacio: ' || n_votacio||', numOpcio:  ' || n_numopcio|| ' idCiutada: ' || n_idciutada,
          'RSP: '||rst);   
      COMMIT;       
      
     WHEN votacioInexistent
   THEN
      rst := 'ERROR: La votacio no existeix';
   
    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'Votar',
          'codiVotacio: ' || n_votacio||', numOpcio:  ' || n_numopcio|| ' idCiutada: ' || n_idciutada,
          'RSP: '||rst);   
      COMMIT;   
      
  WHEN opcioInexistent
   THEN
      rst := 'ERROR: La opci no existeix';
   
    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'Votar',
          'codiVotacio: ' || n_votacio||', numOpcio:  ' || n_numopcio|| ' idCiutada: ' || n_idciutada,
          'RSP: '||rst);   
      COMMIT;   
      
   WHEN campsNuls
   THEN
      rst := 'ERROR: el codi del ciutada, el de la votaci i el num de opci no poden ser nuls';
      
   INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'Votar',
          'codiVotacio: ' || n_votacio||', numOpcio:  ' || n_numopcio|| ' idCiutada: ' || n_idciutada,
          'RSP: '||rst);     
      COMMIT;
      
   WHEN noConvocat
   THEN
      rst := 'ERROR: el ciutadano ha estat convocat a aquesta votaci';
      
   INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'Votar',
          'codiVotacio: ' || n_votacio||', numOpcio:  ' || n_numopcio|| ' idCiutada: ' || n_idciutada,
          'RSP: '||rst);     
      COMMIT;
   
   WHEN votJaEmes
   THEN
      rst := 'ERROR: El ciutada ja ha votat en aquesta convocatria';
   
    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'Votar',
          'codiVotacio: ' || n_votacio||', numOpcio:  ' || n_numopcio|| ' idCiutada: ' || n_idciutada,
          'RSP: '||rst);   
      COMMIT; 
      
   WHEN votacioForaDeDates
   THEN
      rst := 'ERROR: La votacio ja ha finalitzat, o encara no est oberta';
   
    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'Votar',
          'codiVotacio: ' || n_votacio||', numOpcio:  ' || n_numopcio|| ' idCiutada: ' || n_idciutada,
          'RSP: '||rst);   
      COMMIT;   
   WHEN OTHERS
   THEN
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:'
             || err_msg;
   
    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'Votar',
          'codiVotacio: ' || n_votacio||', numOpcio:  ' || n_numopcio|| ' idCiutada: ' || n_idciutada,
          'RSP: '||rst);   
      COMMIT;
END votar;
/
-- ---------------------------------------------------------------------------------------

create or replace PROCEDURE activarvotacio (
   n_idvotacio     IN       NUMBER,
   rst             OUT      VARCHAR2
)
IS   
/******************************************************************************
   activarVotacio:  activa una votaci tot donant d'alta els ciutadans convocats.
                
   PARAMETRES:  n_idvotacio, rst
   INPUT:       n_idvotacio
   OUTPUT:      rst
   RETORNA:     'OK' si s'ha executat correctament, 'ERROR: Tipus d'Error' en
                cas contrari
   EXEMPLE D'US:     activarVotacio(1, rst)
******************************************************************************/
   n_idvotacio_ins            NUMBER;
   n_estat                    NUMBER;
   err_msg                    VARCHAR2 (100);
   existe                     VARCHAR2(1);
   campsNuls                  EXCEPTION;
   votacioInexistent          EXCEPTION;
   votacioOberta              EXCEPTION;
   votacioTancada             EXCEPTION;
   
  
BEGIN
   n_idvotacio_ins    := n_idvotacio;
  
  --verifiquem que el codi de la votacio no sigui nul
  
  IF (n_idvotacio_ins IS NULL)  THEN 
    RAISE campsNuls;
  END IF; 
   
  -- Verifiquem que la votacio existeixi
  
  BEGIN
       SELECT 'S'
         INTO EXISTE
         FROM VOTACIO
         WHERE idvotacio          = n_idvotacio
         AND ROWNUM               = 1;
                        
     EXCEPTION
       WHEN NO_DATA_FOUND THEN
          RAISE votacioInexistent;
     END;
  -- Verifiquem que la votaci no estigui oberta
  
  BEGIN
       SELECT estatvotacio
         INTO n_estat
         FROM VOTACIO
         WHERE idvotacio          = n_idvotacio
         AND ROWNUM               = 1;
                        
     END;
     
     IF (n_estat=2) THEN RAISE votacioOberta;
     END IF;
     
 -- verifiquem que la votaci no estigui tancada
 IF (n_estat=3) THEN RAISE votacioTancada;
     END IF;
 -- Afegim els ciutadans 
  
  
   INSERT INTO votacionsciutada (idvotacio, idciutada, idcodicens, idtipuscens,anno)
   (SELECT am.idvotacio, cc.idciutada,cc.idcodicens, cc.idtipuscens, EXTRACT(YEAR FROM v.datainici)
    FROM   VOTACIO v JOIN AMBIT am ON v.idvotacio= am.idvotacio JOIN CENSCIUTADANS cc ON am.codicens= cc.idcodicens AND am.idtipuscens=cc.idtipuscens 
    WHERE  am.idvotacio=n_idvotacio_ins);
 
 -- Actualitzem l'estat de la votaci
 
    UPDATE VOTACIO SET estatvotacio=2
    WHERE idvotacio=n_idvotacio_ins; 
 
   -- si ha finalitzat correctament retorna OK, en cas contrari gestionem els errors
   rst := 'OK';

    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'ActivarVotacio',
          'idVotacio: ' || n_idvotacio, 
          'RSP: '||rst);   
   COMMIT;
EXCEPTION   
   WHEN votacioInexistent
   THEN
   ROLLBACK;
      rst := 'ERROR: La votaci no existeix';
   
    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'ActivarVotacio',
          'idVotacio: ' || n_idvotacio, 
          'RSP: '||rst);  
    COMMIT;
          
   WHEN votacioOberta
   THEN
   ROLLBACK;
      rst := 'ERROR: La votaci ja est oberta';
   
    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'ActivarVotacio',
          'idVotacio: ' || n_idvotacio, 
          'RSP: '||rst);   
    COMMIT;      
    
  WHEN votacioTancada
   THEN
   ROLLBACK;
      rst := 'ERROR: La votaci est tancada';
   
    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'ActivarVotacio',
          'idVotacio: ' || n_idvotacio, 
          'RSP: '||rst);   
    COMMIT;      
      
   WHEN campsNuls
   THEN
   ROLLBACK;
      rst := 'ERROR: el codi de la votaci, el codi del cens i el tipus de cens no poden ser nuls';
    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'ActivarVotacio',
          'idVotacio: ' || n_idvotacio, 
          'RSP: '||rst);          
      COMMIT;
   WHEN OTHERS
   THEN
   ROLLBACK;
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:'
             || err_msg;
   
    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'ActivarVotacio',
          'idVotacio: ' || n_idvotacio, 
          'RSP: '||rst); 
    COMMIT;
END activarVotacio;
/
-- ------------------------------------------------------------------------------------

/**************************************************************************************
			PROCEDIMENTS DEL MDUL ESTADSTIC
**************************************************************************************/

CREATE OR REPLACE
PROCEDURE actualitzaE1 (
   n_idvotacio     IN       NUMBER,
   rst             OUT      VARCHAR2
)
IS   
/******************************************************************************
   ACTUALITZAE1:  actualitza les dades a la taula EST13, Amb el nombre de votacions 
                  tancades en un any concret per a un pas concret
                
   PARAMETRES:  n_idvotacio, rst
   INPUT:       n_idvotacio
   OUTPUT:      rst
   RETORNA:     'OK' si s'ha executat correctament, 'ERROR: Tipus d'Error' en
                cas contrari
   EXEMPLE D'US:     ActualitzaE1 (1, rst)
******************************************************************************/
   n_idvotacio_ins            NUMBER;
   n_tipuscens_ins            NUMBER;
   n_codicens_ins             NUMBER;
   n_any_ins                  NUMBER;
   err_msg                    VARCHAR2 (100);
   existe                     VARCHAR2(1);
   N                          NUMBER:=0;
   campsNuls                  EXCEPTION;
   votacioInexistent          EXCEPTION;
   
  -- cursor del cens global que llista tots els pasos 
   CURSOR CUR_GLOBAL IS
        SELECT idpais
         FROM CENSESTATAL;
  -- cursor del cens Estatal que llista els pasos convocats a una votaci     
   CURSOR CUR_ESTATAL IS
   SELECT idtipuscens, codicens
   FROM ambit
   WHERE idvotacio=n_idvotacio;
   
   -- cursor del cens Regional que llista els pasos convocats a una votaci
   CURSOR CUR_REGIONAL IS
   SELECT ce.idpais
   FROM ambit am join  censregional cr ON am.idtipuscens=cr.idtipuscens AND am.codicens=cr.idregio 
                 join censestatal ce on cr.idpais=ce.codipais
   WHERE idvotacio=n_idvotacio
   group by ce.idpais;
   
   -- cursor del cens Local que llista els pasos convocats a una votaci
   CURSOR CUR_LOCAL IS
   SELECT ce.idpais
   FROM ambit am join  censlocal cl ON am.idtipuscens=cl.idtipuscens AND am.codicens=cl.idlocal 
                 join censestatal ce on cl.idpais=ce.codipais
   WHERE idvotacio=n_idvotacio
   group by ce.idpais;
   
BEGIN
   n_idvotacio_ins    := n_idvotacio;
   
  
  --verifiquem que el codi de la votacio no sigui nul
  
  IF (n_idvotacio_ins IS NULL)  THEN 
    RAISE campsNuls;
  END IF; 
   
  -- Verifiquem que la votacio existeixi
  
  BEGIN
       SELECT 'S'
         INTO EXISTE
         FROM VOTACIO
         WHERE idvotacio          = n_idvotacio
         AND ROWNUM               = 1;
                        
     EXCEPTION
       WHEN NO_DATA_FOUND THEN
          RAISE votacioInexistent;
     END;
     
   SELECT idtipuscens
   INTO n_tipuscens_ins
   FROM ambit 
   WHERE idvotacio=n_idvotacio
   AND ROWNUM               = 1;
   
 -- extraiem l'any de la votaci
 SELECT  EXTRACT(YEAR FROM datainici) 
 INTO  n_any_ins
 FROM votacio  
 WHERE idvotacio= n_idvotacio_ins;
 
 -- si el cens es global, per tots els pasos donats d'alta afegim una lnia 
 -- a la taula EST13 amb la dada valor =1 , si la lnia existeix, actualitzem el valor
 IF (n_tipuscens_ins=1) THEN
  FOR R IN CUR_GLOBAL LOOP
    N := N + 1;
    BEGIN
       SELECT 'S'
         INTO EXISTE
         FROM EST13
         WHERE  tipuse             = 1
         AND    tipuscens          = 2
         AND    codicens           = R.idpais
         AND    anno               = n_any_ins
         AND ROWNUM                = 1;
        
         UPDATE EST13 set valor=valor + 1 
         WHERE  tipuse             = 1
         AND    tipuscens          = 2
         AND    codicens           = R.idpais
         AND    anno               = n_any_ins;
         
     EXCEPTION
       WHEN NO_DATA_FOUND THEN
          INSERT INTO EST13 (tipusE, tipuscens, codicens, anno, valor) 
                 VALUES (1, 2, R.idpais, n_any_ins, 1);
     END;         
  END LOOP;
  END IF;
 -- si el cens es Estatal, per tots els pasos convocats afegim una lnia 
 -- a la taula EST13 amb la dada valor =1 , si la lnia existeix, actualitzem el valor
  IF (n_tipuscens_ins=2) THEN
  FOR R IN CUR_ESTATAL LOOP
    N := N + 1;
     BEGIN
       SELECT 'S'
         INTO EXISTE
         FROM EST13
         WHERE  tipuse             = 1
         AND    tipuscens          = 2
         AND    codicens           = R.codicens
         AND    anno               = n_any_ins
         AND ROWNUM                = 1;
        
         UPDATE EST13 set valor=valor + 1 
         WHERE tipuse             = 1
         AND   tipuscens          = 2
         AND   codicens           = R.codicens
         AND   anno               = n_any_ins;
         
     EXCEPTION
       WHEN NO_DATA_FOUND THEN
          INSERT INTO EST13 (tipusE, tipuscens, codicens, anno, valor) 
                 VALUES (1, 2, R.codicens, n_any_ins, 1);
     END;         
  END LOOP;
  END IF;
 -- si el cens es Regional, per tots els pasos als quals pertnyen les regions convocades
 -- afegim una lnia a la taula EST13 amb la dada valor =1 , si la lnia existeix, actualitzem el valor 
  IF (n_tipuscens_ins=3) THEN
  FOR R IN CUR_REGIONAL LOOP
    N := N + 1;
 
    BEGIN
       SELECT 'S'
         INTO EXISTE
         FROM EST13
         WHERE tipuse             = 1
         AND   tipuscens          = 2
         AND   codicens           = R.idpais
         AND   anno               = n_any_ins
         AND ROWNUM               = 1;
        
         UPDATE EST13 set valor=valor + 1 
         WHERE tipuse             = 1
         AND   tipuscens          = 2
         AND   codicens           = R.idpais
         AND   anno               = n_any_ins;
         
     EXCEPTION
       WHEN NO_DATA_FOUND THEN
          INSERT INTO EST13 (tipusE, tipuscens, codicens, anno, valor) 
                 VALUES (1, 2, R.idpais, n_any_ins, 1);
     END;         
  END LOOP;
  END IF;
  
 -- si el cens es Local, per tots els pasos als quals pertnyen les localitats convocades
 -- afegim una lnia a la taula EST13 amb la dada valor =1 , si la lnia existeix, actualitzem el valor 
  IF (n_tipuscens_ins=4) THEN
  FOR R IN CUR_LOCAL LOOP
    N := N + 1;
 
    BEGIN
       SELECT 'S'
         INTO EXISTE
         FROM EST13
         WHERE tipuse             = 1
         AND   tipuscens          = 2
         AND   codicens           = R.idpais
         AND   anno               = n_any_ins
         AND ROWNUM               = 1;
        
         UPDATE EST13 set valor=valor + 1 
         WHERE tipuse             = 1
         AND   tipuscens          = 2
         AND   codicens           = R.idpais
         AND   anno               = n_any_ins;
         
     EXCEPTION
       WHEN NO_DATA_FOUND THEN
          INSERT INTO EST13 (tipusE, tipuscens, codicens, anno, valor) 
                 VALUES (1, 2, R.idpais, n_any_ins, 1);
     END;         
  END LOOP;
  END IF;
 
 -- si ha finalitzat correctament retorna OK, en cas contrari gestionem els errors
   rst := 'OK';

    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'ActualitzaE1',
          'idVotacio: ' || n_idvotacio, 
          'RSP: '||rst);   
   COMMIT;
EXCEPTION
    
   WHEN votacioInexistent
   THEN
   ROLLBACK;
      rst := 'ERROR: La votaci no existeix';
   
    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
   VALUES (SYSDATE,'ActualitzaE1',
          'idVotacio: ' || n_idvotacio, 
          'RSP: '||rst);   
      COMMIT;       
      
   WHEN campsNuls
   THEN
   ROLLBACK;
      rst := 'ERROR: el codi de la votaci no pot ser nul';
    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'ActualitzaE1',
          'idVotacio: ' || n_idvotacio, 
          'RSP: '||rst);   
   
      COMMIT;
   WHEN OTHERS
   THEN
   ROLLBACK;
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:'
             || err_msg;
   
    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'ActualitzaE1',
          'idVotacio: ' || n_idvotacio, 
          'RSP: '||rst);   
END actualitzaE1;
/
-- ---------------------------------------------------------------------------------------

create or replace PROCEDURE actualitzaE2 (
   n_idvotacio     IN       NUMBER,
   rst             OUT      VARCHAR2
)
IS   
/******************************************************************************
   ACTUALITZAE2:  actualitza les dades a la taula EST13, Amb la mitjana de participaci 
                  de les votacions tancades en un any concret per a un cens concret
                
   PARAMETRES:  n_idvotacio, rst
   INPUT:       n_idvotacio
   OUTPUT:      rst
   RETORNA:     'OK' si s'ha executat correctament, 'ERROR: Tipus d'Error' en
                cas contrari
   EXEMPLE D'US:     ActualitzaE2 (1, rst)
******************************************************************************/
   n_idvotacio_ins            NUMBER;
   n_tipuscens_ins            NUMBER;
   n_codicens_ins             NUMBER;
   n_any_ins                  NUMBER;
   n_percpart_ins             NUMBER;
   err_msg                    VARCHAR2 (500);
   existe                     VARCHAR2(1);
   N                          NUMBER:=0;
   campsNuls                  EXCEPTION;
   votacioInexistent          EXCEPTION;
   
  -- cursors del cens global que llisten tots els censos 
   CURSOR CUR_GLOBAL_PAIS IS
        SELECT idpais
        FROM CENSESTATAL;
   
    CURSOR CUR_GLOBAL_REGIO IS
        SELECT idregio
        FROM CENSREGIONAL;
         
    CURSOR CUR_GLOBAL_LOCAL IS
        SELECT idlocal
        FROM CENSLOCAL;
  -- cursor del cens Estatal que llista els pasos convocats a una votaci     
    CURSOR CUR_ESTATAL IS
        SELECT idtipuscens, codicens
        FROM ambit
        WHERE idvotacio=n_idvotacio;
   
   -- cursor del cens Regional que llista les regions i els pasos associats a les regions
   -- convocades a una votaci
   CURSOR CUR_REGIONAL_PAIS IS
        SELECT ce.idpais
        FROM ambit am JOIN  censregional cr ON am.idtipuscens=cr.idtipuscens AND am.codicens=cr.idregio 
                 JOIN censestatal ce ON cr.idpais=ce.codipais
        WHERE idvotacio=n_idvotacio
        GROUP BY ce.idpais;
   
   CURSOR CUR_REGIONAL_REGIO IS
      SELECT cr.idregio, cr.idtipuscens
      FROM ambit am JOIN  censregional cr ON am.idtipuscens=cr.idtipuscens AND am.codicens=cr.idregio 
      WHERE idvotacio=n_idvotacio;
   
   -- cursor del cens Local que llista les localitats i els pasos associats a les localitats convocades
   -- a una votaci
   CURSOR CUR_LOCAL_PAIS IS
      SELECT ce.idpais
      FROM ambit am JOIN  censlocal cl ON am.idtipuscens=cl.idtipuscens AND am.codicens=cl.idlocal 
                 JOIN censestatal ce ON cl.idpais=ce.codipais
      WHERE idvotacio=n_idvotacio
      GROUP BY ce.idpais;
   
   CURSOR CUR_LOCAL_REGIO IS
      SELECT cr.idregio
      FROM ambit am JOIN censlocal cl ON am.idtipuscens=cl.idtipuscens AND am.codicens=cl.idlocal 
                 JOIN censregional cr ON cl.idregio=cr.idregio
      WHERE idvotacio=n_idvotacio
      GROUP BY cr.idregio;
   
   CURSOR CUR_LOCAL_LOCAL IS
      SELECT cl.idlocal, cl.idtipuscens
      FROM ambit am JOIN  censlocal cl ON am.idtipuscens=cl.idtipuscens AND am.codicens=cl.idlocal 
      WHERE idvotacio=n_idvotacio;
   
BEGIN
   n_idvotacio_ins    := n_idvotacio;
    
  --verifiquem que el codi de la votacio no sigui nul
  IF (n_idvotacio_ins IS NULL)  THEN 
    RAISE campsNuls;
  END IF; 
   
  -- Verifiquem que la votacio existeixi
  BEGIN
       SELECT 'S'
         INTO EXISTE
         FROM VOTACIO
         WHERE idvotacio          = n_idvotacio
         AND ROWNUM               = 1;
                        
     EXCEPTION
       WHEN NO_DATA_FOUND THEN
          RAISE votacioInexistent;
     END;
     
 SELECT idtipuscens
   INTO n_tipuscens_ins
   FROM ambit 
   WHERE idvotacio=n_idvotacio
   AND ROWNUM = 1;

-- extraiem l'any de la votaci     
SELECT  EXTRACT(YEAR FROM datainici) 
  INTO  n_any_ins
  FROM votacio 
  WHERE idvotacio= n_idvotacio_ins; 
 
 -- si el cens es global, per tots els pasos donats d'alta afegim una lnia 
 -- a la taula EST13 amb la dada valor = a la mitjana de participaci , si 
 -- la lnia existeix, actualitzem el valor
 
 -- ----------------------------------------------------------------------------
 IF (n_tipuscens_ins=1) THEN
 -- calculem el valor mig de la participaci tenint en compte les votacions en 
 -- les que ha participat el cens global (els pasos, les regions i les localitats)
SELECT AVG(pparticipacio)
INTO n_percpart_ins
FROM votaciotancada
WHERE idvotacio IN (SELECT idvotacio FROM ambit WHERE idtipuscens=1);

  FOR R IN CUR_GLOBAL_PAIS LOOP
    N := N + 1;
    BEGIN
       SELECT 'S'
         INTO EXISTE
         FROM EST13
         WHERE tipuse             = 2
         AND tipuscens            = 2
         AND   codicens           = R.idpais
         AND   anno               = n_any_ins
         AND ROWNUM               = 1;
        
         UPDATE EST13 set valor=n_percpart_ins 
         WHERE tipuse     =2
         AND   tipuscens  =2
         AND   codicens   =R.idpais
         AND   anno       =n_any_ins;
         
     EXCEPTION
       WHEN NO_DATA_FOUND THEN
          INSERT INTO EST13 (tipusE, tipuscens, codicens, anno, valor) 
                 VALUES (2, 2, R.idpais, n_any_ins, n_percpart_ins);
     END;         
  END LOOP;
  
  N:=0;  
  FOR R IN CUR_GLOBAL_REGIO LOOP
    N := N + 1;
    SELECT AVG(pparticipacio)
    INTO n_percpart_ins
    FROM votaciotancada
    WHERE idvotacio IN (SELECT idvotacio FROM ambit WHERE (idtipuscens=3 AND codicens=R.idregio) OR (idtipuscens=1 AND codicens=1));
    
    BEGIN
       SELECT 'S'
         INTO EXISTE
         FROM EST13
         where tipuse             = 2
         AND   tipuscens          = 3
         AND   codicens           = R.idregio
         AND anno                 = n_any_ins
         AND ROWNUM               = 1;
        
         UPDATE EST13 set valor=n_percpart_ins 
         where  tipuse      = 2
         AND    tipuscens   = 3
         AND    codicens    = R.idregio
         AND    anno        = n_any_ins;
         
     EXCEPTION
       WHEN NO_DATA_FOUND THEN
          INSERT INTO EST13 (tipusE, tipuscens, codicens, anno, valor) 
                 VALUES (2, 3, R.idregio, n_any_ins, n_percpart_ins);
     END;         
  END LOOP;
   N:=0;  
  FOR R IN CUR_GLOBAL_LOCAL LOOP
    N := N + 1;
    SELECT AVG(pparticipacio)
    INTO n_percpart_ins
    FROM votaciotancada
    WHERE idvotacio IN (SELECT idvotacio FROM ambit WHERE (idtipuscens=4 AND codicens=R.idlocal) OR (idtipuscens=1 AND codicens=1));
    
    BEGIN
       SELECT 'S'
         INTO EXISTE
         FROM EST13
         where tipuse             = 2
         AND   tipuscens          = 4
         AND   codicens           = R.idlocal
         AND anno                 = n_any_ins
         AND ROWNUM               = 1;
        
         UPDATE EST13 set valor=n_percpart_ins 
         where tipuse       = 2 
         AND   tipuscens    = 4
         AND   codicens     = R.idlocal
         AND   anno         = n_any_ins;
         
     EXCEPTION
       WHEN NO_DATA_FOUND THEN
          INSERT INTO EST13 (tipusE, tipuscens, codicens, anno, valor) 
                 VALUES (2, 4, R.idlocal, n_any_ins, n_percpart_ins);
     END;         
  END LOOP;
  END IF;
  
 -- ----------------------------------------------------------------------------
 -- si el cens es Estatal, per tots els pasos convocats afegim una lnia 
 -- a la taula EST13 amb la dada valor = percentatge de participaci , si la lnia 
 -- existeix, actualitzem el valor
  IF (n_tipuscens_ins=2) THEN
  N:=0;
  FOR R IN CUR_ESTATAL LOOP
    N := N + 1;
    
    SELECT AVG(pparticipacio)
    INTO n_percpart_ins
    FROM votaciotancada
    WHERE idvotacio IN (SELECT idvotacio FROM ambit WHERE (idtipuscens=R.idtipuscens AND codicens=R.codicens)OR (idtipuscens=1 AND codicens=1));
    
     BEGIN
       SELECT 'S'
         INTO EXISTE
         FROM EST13
         WHERE tipuse             = 2
         AND   tipuscens          = 2
         AND   codicens           = R.codicens
         AND   anno               = n_any_ins
         AND ROWNUM               = 1;
        
         UPDATE EST13 set valor=n_percpart_ins 
         where tipuse       = 2
         AND   tipuscens    = 2
         AND   codicens     = R.codicens
         AND   anno         = n_any_ins;
         
     EXCEPTION
       WHEN NO_DATA_FOUND THEN
          INSERT INTO EST13 (tipusE, tipuscens, codicens, anno, valor) 
                 VALUES (2, 2, R.codicens, n_any_ins, n_percpart_ins);
     END;         
  END LOOP;
  END IF;
 -- ----------------------------------------------------------------------------
 -- si el cens es REGIONAL, per totes les regions i els paisos associats a les regions 
 -- convocades afegim una lnia a la taula EST13 amb la dada valor = percentathe de 
 -- participaci , si la lnia existeix actualitzem el valor
  IF (n_tipuscens_ins=3) THEN
  N:=0;
  FOR R IN CUR_REGIONAL_PAIS LOOP
    N := N + 1;
    
    SELECT AVG(pparticipacio)
    INTO n_percpart_ins
    FROM votaciotancada
    WHERE idvotacio IN (SELECT idvotacio FROM ambit WHERE (idtipuscens= 2 AND codicens=R.idpais) OR (idtipuscens=1 AND codicens=1));
    
     BEGIN
       SELECT 'S'
         INTO EXISTE
         FROM EST13
         WHERE tipuse             = 2
         AND tipuscens            = 2
         AND   codicens           = R.idpais
         AND anno                 = n_any_ins
         AND ROWNUM               = 1;
        
         UPDATE EST13 set valor=n_percpart_ins 
         where tipuse       = 2
         AND   tipuscens    = 2
         AND   codicens     = R.idpais
         AND   anno         = n_any_ins;
         
     EXCEPTION
       WHEN NO_DATA_FOUND THEN
          INSERT INTO EST13 (tipusE, tipuscens, codicens, anno, valor) 
                 VALUES (2, 2, R.idpais, n_any_ins, n_percpart_ins);
     END;         
  END LOOP;
  
  N:=0;
  FOR R IN CUR_REGIONAL_REGIO LOOP
    N := N + 1;
    
    SELECT AVG(pparticipacio)
    INTO n_percpart_ins
    FROM votaciotancada
    WHERE idvotacio IN (SELECT idvotacio FROM ambit WHERE (idtipuscens= R.idtipuscens AND codicens=R.idregio) OR (idtipuscens=1 AND codicens=1));
    
     BEGIN
       SELECT 'S'
         INTO EXISTE
         FROM EST13
         WHERE tipuse             = 2
         AND tipuscens            = R.idtipuscens
         AND   codicens           = R.idregio
         AND anno                 = n_any_ins
         AND ROWNUM               = 1;
        
         UPDATE EST13 set valor=n_percpart_ins 
         where tipuse       = 2
         AND   tipuscens    = R.idtipuscens
         AND   codicens     = R.idregio
         AND   anno         = n_any_ins;
         
     EXCEPTION
       WHEN NO_DATA_FOUND THEN
          INSERT INTO EST13 (tipusE, tipuscens, codicens, anno, valor) 
                 VALUES (2, 2, R.idregio, n_any_ins, n_percpart_ins);
     END;         
  END LOOP;
  
  
  END IF;
  
-- ----------------------------------------------------------------------------
 -- si el cens es LOCAL,per totes les localitats i per totes les regions i els paisos 
 -- associats a les localitats convocades afegim una lnia a la taula EST13 amb la 
 -- dada valor = percentathe de participaci , si la lnia existeix actualitzem el valor
  IF (n_tipuscens_ins=4) THEN
  N:=0;
  FOR R IN CUR_LOCAL_PAIS LOOP
    N := N + 1;
    
    SELECT AVG(pparticipacio)
    INTO n_percpart_ins
    FROM votaciotancada
    WHERE idvotacio IN (SELECT idvotacio FROM ambit WHERE (idtipuscens= 2 AND codicens=R.idpais) OR (idtipuscens=1 AND codicens=1));
    
     BEGIN
       SELECT 'S'
         INTO EXISTE
         FROM EST13
         WHERE tipuse             = 2
         AND tipuscens            = 2
         AND   codicens           = R.idpais
         AND anno                 = n_any_ins
         AND ROWNUM               = 1;
        
         UPDATE EST13 set valor=n_percpart_ins 
         where tipuse       = 2
         AND   tipuscens    = 2
         AND   codicens     = R.idpais
         AND   anno         = n_any_ins;
         
     EXCEPTION
       WHEN NO_DATA_FOUND THEN
          INSERT INTO EST13 (tipusE, tipuscens, codicens, anno, valor) 
                 VALUES (2, 2, R.idpais, n_any_ins, n_percpart_ins);
     END;         
  END LOOP;
  N:=0;
  FOR R IN CUR_LOCAL_REGIO LOOP
    N := N + 1;
    
    SELECT AVG(pparticipacio)
    INTO n_percpart_ins
    FROM votaciotancada
    WHERE idvotacio IN (SELECT idvotacio FROM ambit WHERE (idtipuscens= 3 AND codicens=R.idregio) OR (idtipuscens=1 AND codicens=1));
    
     BEGIN
       SELECT 'S'
         INTO EXISTE
         FROM EST13
         WHERE tipuse             = 2
         AND tipuscens            = 3
         AND   codicens           = R.idregio
         AND anno                 = n_any_ins
         AND ROWNUM               = 1;
        
         UPDATE EST13 set valor=n_percpart_ins 
         where tipuse       = 2
         AND   tipuscens    = 3
         AND   codicens     = R.idregio
         AND   anno         = n_any_ins;
         
     EXCEPTION
       WHEN NO_DATA_FOUND THEN
          INSERT INTO EST13 (tipusE, tipuscens, codicens, anno, valor) 
                 VALUES (2, 3, R.idregio, n_any_ins, n_percpart_ins);
     END;         
  END LOOP;
  
  n:=0;
  FOR R IN CUR_LOCAL_LOCAL LOOP
    N := N + 1;
    
    SELECT AVG(pparticipacio)
    INTO n_percpart_ins
    FROM votaciotancada
    WHERE idvotacio IN (SELECT idvotacio FROM ambit WHERE (idtipuscens= R.idtipuscens AND codicens=R.idlocal) OR (idtipuscens=1 AND codicens=1));
    
     BEGIN
       SELECT 'S'
         INTO EXISTE
         FROM EST13
         WHERE tipuse             = 2
         AND tipuscens            = R.idtipuscens
         AND   codicens           = R.idlocal
         AND anno                 = n_any_ins
         AND ROWNUM               = 1;
        
         UPDATE EST13 set valor=n_percpart_ins 
         where tipuse       = 2
         AND   tipuscens    = R.idtipuscens
         AND   codicens     = R.idlocal
         AND   anno         = n_any_ins;
         
     EXCEPTION
       WHEN NO_DATA_FOUND THEN
          INSERT INTO EST13 (tipusE, tipuscens, codicens, anno, valor) 
                 VALUES (2, R.idtipuscens, R.idlocal, n_any_ins, n_percpart_ins);
     END;         
  END LOOP;
  
  
  END IF;
  
 -- si ha finalitzat correctament retorna OK, en cas contrari gestionem els errors
   rst := 'OK';

    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'ActualitzaE2',
          'idVotacio: ' || n_idvotacio, 
          'RSP: '||rst);   
   COMMIT;
EXCEPTION
    
   WHEN votacioInexistent
   THEN
   ROLLBACK;
      rst := 'ERROR: La votaci no existeix';
   
    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
   VALUES (SYSDATE,'ActualitzaE2',
          'idVotacio: ' || n_idvotacio, 
          'RSP: '||rst);   
      COMMIT;       
      
   WHEN campsNuls
   THEN
   ROLLBACK;
      rst := 'ERROR: el codi de la votaci no pot ser nul';
    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'ActualitzaE2',
          'idVotacio: ' || n_idvotacio, 
          'RSP: '||rst);   
   
      COMMIT;
   WHEN OTHERS
   THEN
   ROLLBACK;
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:'
             || err_msg;
   
    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'ActualitzaE2',
          'idVotacio: ' || n_idvotacio, 
          'RSP: '||rst);   
END actualitzaE2;
/
-- ------------------------------------------------------------------------------------

create or replace PROCEDURE actualitzaE3 (
   n_idvotacio     IN       NUMBER,
   n_idtipuscens   IN       NUMBER,
   n_idcodicens    IN       NUMBER,
   rst             OUT      VARCHAR2
)
IS   
/******************************************************************************
   ACTUALITZAE3:  actualitza les dades a la taula EST13, Amb el nombre de votacions 
                  en un any concret per a un cens concret
                
   PARAMETRES:  n_idvotacio, n_idtipuscens, n_idcodicens, rst
   INPUT:       n_idvotacio, n_idtipuscens, n_idcodicens
   OUTPUT:      rst
   RETORNA:     'OK' si s'ha executat correctament, 'ERROR: Tipus d'Error' en
                cas contrari
   EXEMPLE D'US:     ActualitzaE3 (1,1,1, rst)
******************************************************************************/
   n_idvotacio_ins            NUMBER;
   n_idtipuscens_ins          NUMBER;
   n_idcodicens_ins           NUMBER;
   n_idpais_ins               VARCHAR2(2);
   n_idregio_ins              NUMBER;
   n_any_ins                  NUMBER;
   err_msg                    VARCHAR2 (100);
   existe                     VARCHAR2(1);
   N                          NUMBER:=0;
   campsNuls                  EXCEPTION;
   votacioInexistent          EXCEPTION;
   tipuscensInexistent        EXCEPTION;
   codicensInexistent         EXCEPTION;
   
  -- cursor del cens global que llista tots els pasos, les regions i les localitats 
   CURSOR CUR_GLOBAL_ESTATAL IS
        SELECT idpais
         FROM CENSESTATAL;
         
   CURSOR CUR_GLOBAL_REGIONAL IS
        SELECT idregio
        FROM CENSREGIONAL;
        
   CURSOR CUR_GLOBAL_LOCAL IS
        SELECT idlocal
         FROM CENSLOCAL;
         
  -- cursor del cens estatal que llista totes les regions i les localitats
  
  CURSOR CUR_ESTATAL_REGIONAL IS
        SELECT idregio
        FROM CENSREGIONAL
        WHERE IDPAIS= n_idpais_ins;
        
   CURSOR CUR_ESTATAL_LOCAL IS
        SELECT idlocal
         FROM CENSLOCAL
         WHERE IDPAIS= n_idpais_ins;
         
  -- cursor del cens regional que llista totes les localitats
  
  CURSOR CUR_REGIONAL_LOCAL IS
        SELECT idlocal
         FROM CENSLOCAL
         WHERE IDREGIO= n_idcodicens_ins;
  
   
BEGIN
   n_idvotacio_ins    := n_idvotacio;
   n_idtipuscens_ins  := n_idtipuscens;
   n_idcodicens_ins   := n_idcodicens;
   
   
  --verifiquem que el codi de la votacio, el codi del cens i el tipus de cens no sigui nuls
  
  IF (n_idvotacio_ins IS NULL)OR (n_idtipuscens_ins IS NULL) OR (n_idcodicens_ins IS NULL)  THEN 
    RAISE campsNuls;
  END IF; 
   
  -- Verifiquem que la votacio existeixi
  
  BEGIN
       SELECT 'S'
         INTO EXISTE
         FROM VOTACIO
         WHERE idvotacio          = n_idvotacio
         AND ROWNUM               = 1;
                        
     EXCEPTION
       WHEN NO_DATA_FOUND THEN
          RAISE votacioInexistent;
     END;
     
         
 
 -- extraiem l'any de la votaci
 SELECT  EXTRACT(YEAR FROM datainici) 
 INTO  n_any_ins
 FROM votacio 
 WHERE idvotacio= n_idvotacio_ins;
 
 -- si el cens es global, per tots els censos donats d'alta afegim una lnia 
 -- a la taula EST13 amb la dada valor =1 , si la lnia existeix, actualitzem el valor
 IF (n_idtipuscens_ins=1) THEN
  FOR R IN CUR_GLOBAL_ESTATAL LOOP
    N := N + 1;
    BEGIN
       SELECT 'S'
         INTO EXISTE
         FROM EST13
         WHERE  tipuse         = 3
         AND    tipuscens      = 2
         AND   codicens        = R.idpais
         AND anno              = n_any_ins
         AND ROWNUM            = 1;
        
         UPDATE EST13 set valor=valor + 1 
         where tipuse     = 3
         AND   tipuscens  = 2
         AND   codicens   = R.idpais
         AND   anno       = n_any_ins;
         
     EXCEPTION
       WHEN NO_DATA_FOUND THEN
          INSERT INTO EST13 (tipusE, tipuscens, codicens, anno, valor) 
                 VALUES (3, 2, R.idpais, n_any_ins, 1);
     END;         
  END LOOP;
  
  N:=0;
  FOR R IN CUR_GLOBAL_REGIONAL LOOP
    N := N + 1;
    BEGIN
       SELECT 'S'
         INTO EXISTE
         FROM EST13
         WHERE  tipuse         = 3
         AND    tipuscens      = 3
         AND   codicens        = R.idregio
         AND anno              = n_any_ins
         AND ROWNUM            = 1;
        
         UPDATE EST13 set valor=valor + 1 
         where tipuse     = 3
         AND   tipuscens  = 3
         AND   codicens   = R.idregio
         AND   anno       = n_any_ins;
         
     EXCEPTION
       WHEN NO_DATA_FOUND THEN
          INSERT INTO EST13 (tipusE, tipuscens, codicens, anno, valor) 
                 VALUES (3, 3, R.idregio, n_any_ins, 1);
     END;         
  END LOOP;
 
 N:=0; 
 FOR R IN CUR_GLOBAL_LOCAL LOOP
    N := N + 1;
    BEGIN
       SELECT 'S'
         INTO EXISTE
         FROM EST13
         WHERE  tipuse         = 3
         AND    tipuscens      = 4
         AND   codicens        = R.idlocal
         AND anno              = n_any_ins
         AND ROWNUM            = 1;
        
         UPDATE EST13 set valor=valor + 1 
         where tipuse     = 3
         AND   tipuscens  = 4
         AND   codicens   = R.idlocal
         AND   anno       = n_any_ins;
         
     EXCEPTION
       WHEN NO_DATA_FOUND THEN
          INSERT INTO EST13 (tipusE, tipuscens, codicens, anno, valor) 
                 VALUES (3, 4, R.idlocal, n_any_ins, 1);
     END;         
  END LOOP;
  END IF;
 -- si el cens es Estatal, per a la regi donada d'alta afegim una lnia 
 -- a la taula EST13 amb la dada valor =1 , si la lnia existeix, actualitzem el valor
  IF (n_idtipuscens_ins=2) THEN
      
     BEGIN
       SELECT 'S'
         INTO EXISTE
         FROM EST13
         WHERE tipuse             = 3
         AND   tipuscens          = n_idtipuscens_ins
         AND   codicens           = n_idcodicens_ins
         AND anno                 = n_any_ins
         AND ROWNUM               = 1;
        
         UPDATE EST13 set valor=valor + 1 
         where tipuse         = 3
         AND   tipuscens    = n_idtipuscens_ins
         AND   codicens       = n_idcodicens_ins
         AND   anno           = n_any_ins;
         
         EXCEPTION
       WHEN NO_DATA_FOUND THEN
          INSERT INTO EST13 (tipusE, tipuscens, codicens, anno, valor) 
                 VALUES (3, n_idtipuscens, n_idcodicens, n_any_ins, 1);
     END;  
    
    SELECT codipais 
    INTO   n_idpais_ins
    FROM CENSESTATAL
    where idpais=n_idcodicens_ins;
         
    FOR R IN CUR_ESTATAL_REGIONAL LOOP
    N := N + 1;
    BEGIN
       SELECT 'S'
         INTO EXISTE
         FROM EST13
         WHERE  tipuse         = 3
         AND    tipuscens      = 3
         AND   codicens        = R.idregio
         AND anno              = n_any_ins
         AND ROWNUM            = 1;
        
         UPDATE EST13 set valor=valor + 1 
         where tipuse     = 3
         AND   tipuscens  = 3
         AND   codicens   = R.idregio
         AND   anno       = n_any_ins;
         
     EXCEPTION
       WHEN NO_DATA_FOUND THEN
          INSERT INTO EST13 (tipusE, tipuscens, codicens, anno, valor) 
                 VALUES (3, 3, R.idregio, n_any_ins, 1);
     END;         
  END LOOP;
  
  N:=0;
  FOR R IN CUR_ESTATAL_LOCAL LOOP
    N := N + 1;
    BEGIN
       SELECT 'S'
         INTO EXISTE
         FROM EST13
         WHERE  tipuse         = 3
         AND    tipuscens      = 4
         AND   codicens        = R.idlocal
         AND anno              = n_any_ins
         AND ROWNUM            = 1;
        
         UPDATE EST13 set valor=valor + 1 
         where tipuse     = 3
         AND   tipuscens  = 4
         AND   codicens   = R.idlocal
         AND   anno       = n_any_ins;
         
     EXCEPTION
       WHEN NO_DATA_FOUND THEN
          INSERT INTO EST13 (tipusE, tipuscens, codicens, anno, valor) 
                 VALUES (3, 4, R.idlocal, n_any_ins, 1);
     END;         
  END LOOP;
  END IF;
  
 -- si el cens es Regional, per a la regi donada d'alta afegim una lnia
 -- a la taula EST13 amb la dada valor =1 , si la lnia existeix, actualitzem el valor
  IF (n_idtipuscens_ins=3) THEN
   BEGIN
       SELECT 'S'
         INTO EXISTE
         FROM EST13
         WHERE tipuse             = 3
         AND   tipuscens          = n_idtipuscens_ins
         AND   codicens           = n_idcodicens_ins
         AND anno                 = n_any_ins
         AND ROWNUM               = 1;
        
         UPDATE EST13 set valor=valor + 1 
         where tipuse         = 3
         AND   tipuscens      = n_idtipuscens_ins
         AND   codicens       = n_idcodicens_ins
         AND   anno           = n_any_ins;
         
     EXCEPTION
       WHEN NO_DATA_FOUND THEN
          INSERT INTO EST13 (tipusE, tipuscens, codicens, anno, valor) 
                 VALUES (3, n_idtipuscens, n_idcodicens, n_any_ins, 1);
     END; 
         
    FOR R IN CUR_REGIONAL_LOCAL LOOP
    N := N + 1;
    BEGIN
       SELECT 'S'
         INTO EXISTE
         FROM EST13
         WHERE  tipuse         = 3
         AND    tipuscens      = 4
         AND   codicens        = R.idlocal
         AND anno              = n_any_ins
         AND ROWNUM            = 1;
        
         UPDATE EST13 set valor=valor + 1 
         where tipuse     = 3
         AND   tipuscens  = 4
         AND   codicens   = R.idlocal
         AND   anno       = n_any_ins;
         
     EXCEPTION
       WHEN NO_DATA_FOUND THEN
          INSERT INTO EST13 (tipusE, tipuscens, codicens, anno, valor) 
                 VALUES (3, 4, R.idlocal, n_any_ins, 1);
     END;         
  END LOOP;
  END IF;
  
 -- si el cens es Local, per a la localitat donada d'alta afegim una lnia
 -- a la taula EST13 amb la dada valor =1 , si la lnia existeix, actualitzem el valor 
  IF (n_idtipuscens_ins=4) THEN
 BEGIN
       SELECT 'S'
         INTO EXISTE
         FROM EST13
         WHERE tipuse             = 3
         AND   tipuscens          = n_idtipuscens_ins
         AND   codicens           = n_idcodicens_ins
         AND anno                 = n_any_ins
         AND ROWNUM               = 1;
        
         UPDATE EST13 set valor=valor + 1 
         where tipuse         = 3
         AND   tipuscens      = n_idtipuscens_ins
         AND   codicens       = n_idcodicens_ins
         AND   anno           = n_any_ins;
         
     EXCEPTION
       WHEN NO_DATA_FOUND THEN
          INSERT INTO EST13 (tipusE, tipuscens, codicens, anno, valor) 
                 VALUES (3, n_idtipuscens, n_idcodicens, n_any_ins, 1);
     END;         

  END IF;
 
 -- si ha finalitzat correctament retorna OK, en cas contrari gestionem els errors
   rst := 'OK';

    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'ActualitzaE3',
          'idVotacio: ' || n_idvotacio || 'idcodicens' || n_idcodicens || 'idtipuscens' || n_idtipuscens, 
          'RSP: '||rst);   
   COMMIT;
EXCEPTION
    
   WHEN votacioInexistent
   THEN
   ROLLBACK;
      rst := 'ERROR: La votaci no existeix';
   
    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
   VALUES (SYSDATE,'ActualitzaE3',
          'idVotacio: ' || n_idvotacio || 'idcodicens' || n_idcodicens || 'idtipuscens' || n_idtipuscens, 
          'RSP: '||rst);   
      COMMIT;       
      
   WHEN campsNuls
   THEN
   ROLLBACK;
      rst := 'ERROR: el codi de la votaci, el codi del cens i el tipus de cens no poden ser nuls';
    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'ActualitzaE3',
          'idVotacio: ' || n_idvotacio || 'idcodicens' || n_idcodicens || 'idtipuscens' || n_idtipuscens, 
          'RSP: '||rst);   
   
      COMMIT;
   WHEN OTHERS
   THEN
   ROLLBACK;
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:'
             || err_msg;
   
    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'ActualitzaE3',
          'idVotacio: ' || n_idvotacio || 'idcodicens' || n_idcodicens || 'idtipuscens' || n_idtipuscens, 
          'RSP: '||rst);   
END actualitzaE3;
/
-- ------------------------------------------------------------------------------------
CREATE OR REPLACE PROCEDURE ACTUALITZAE5 (
  rst             OUT      VARCHAR2
)
IS   
/******************************************************************************
   ACTUALITZAE5:  actualitza les dades a la taula EST46, Amb la localitat que ms 
		  vots han ems els seus ciutadans
                
   PARAMETRES:  rst
   INPUT:       no hi ha
   OUTPUT:      rst
   RETORNA:     'OK' si s'ha executat correctament, 'ERROR: Tipus d'Error' en
                cas contrari
   EXEMPLE D'US:     ActualitzaE5 (rst)
******************************************************************************/
EXISTE                 VARCHAR2(1);
n_idlocalitat          NUMBER;
n_participacio         NUMBER;
n_ANNO                 NUMBER;

  
  
 BEGIN
 select *
 INTO n_idlocalitat, n_participacio, n_ANNO
 FROM(SELECT v.idlocalitat,SUM(participacio)as part, anno
      FROM votacionsciutada v join censlocal l on v.idlocalitat=l.idlocal
      WHERE l.actiu=1
      GROUP BY v.idlocalitat, participacio, anno, l.actiu
      ORDER BY part DESC)
 WHERE ROWNUM=1;
       
               
  
       SELECT 'S'
         INTO EXISTE
         FROM EST46
         WHERE  idtipuse         = 5
         AND    anno           = n_anno
         AND ROWNUM            = 1;
         
               
         UPDATE EST46 set valor=n_participacio, idv= n_idlocalitat
         where idtipuse     = 5
         AND   anno  = n_anno;           
         
     EXCEPTION
       WHEN NO_DATA_FOUND THEN
          INSERT INTO EST46 (idtipuse, anno, valor, idv) 
                 VALUES (5, n_anno, n_participacio, n_idlocalitat);
                 
     rst := 'OK';

    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'ActualitzaE5',
          'sense parametres d entrada ' ,
          'RSP: '||rst);
   
   COMMIT;
         
  END ActualitzaE5;         
/
-- ------------------------------------------------------------------------------------

/***************************************************************************************
			TIPUS NECESSARIS PER A LES FUNCIONS DE CONSULTA
***************************************************************************************/

create or replace TYPE T_LLI_CENSCIUTADA_OBJ as object (
  codiciutada                VARCHAR2 (11), 
  tipuscens                   NUMBER,
  codicens                    NUMBER
  
);
/
-- --------------------------------------------------------------------------------------

create or replace TYPE  T_LLI_CENSCIUTADA_TAB is table of T_LLI_CENSCIUTADA_OBJ;
/
-- --------------------------------------------------------------------------------------

create or replace TYPE T_LLI_MENYSDIFVOTS_OBJ as object (
  titolvotacio                VARCHAR2 (50), 
  datainici                   DATE,
  datafi                      DATE,
  opcioguanyadora             NUMBER,
  percopcioguanyadora         NUMBER,
  opcioperdedora              NUMBER,
  percopcioperdedora          NUMBER,
  diferencia                  NUMBER
  
);
/
-- --------------------------------------------------------------------------------------

create or replace TYPE  T_LLI_MENYSDIFVOTS_TAB is table of T_LLI_MENYSDIFVOTS_OBJ;
/
-- --------------------------------------------------------------------------------------

create or replace TYPE T_LLI_MESDIFVOTS_OBJ as object (
  titolvotacio                VARCHAR2 (50), 
  datainici                   DATE,
  datafi                      DATE,
  opcioguanyadora             NUMBER,
  percopcioguanyadora         NUMBER,
  opcioperdedora              NUMBER,
  percopcioperdedora          NUMBER,
  diferencia                  NUMBER
  
);
/
-- --------------------------------------------------------------------------------------

create or replace TYPE  T_LLI_MESDIFVOTS_TAB is table of T_LLI_MESDIFVOTS_OBJ;
/
-- --------------------------------------------------------------------------------------

create or replace TYPE T_LLI_VOT_PAIS_OBJ as object (
  titolvotacio                VARCHAR2 (50), 
  datainici                   DATE,
  datafi                      DATE,
  numtotalciutadans           NUMBER,
  estat                       NUMBER,
  percparticipaciototal       VARCHAR2 (10),
  opcioguanyadora             VARCHAR2 (10),
  percopcioguanyadora         VARCHAR2 (10)
  
);
/
-- --------------------------------------------------------------------------------------

create or replace TYPE  T_LLI_VOT_PAIS_TAB is table of T_LLI_VOT_PAIS_OBJ;  
/
-- --------------------------------------------------------------------------------------

create or replace TYPE T_LLI_VOTCIUTADA_OBJ as object (
  codiciutada                VARCHAR2 (11), 
  idvotacio                  NUMBER,
  titol                      VARCHAR (50),
  datainici                  DATE,
  datafi                     DATE,
  participacio               NUMBER
  
);
/
-- ---------------------------------------------------------------------------------------

create or replace TYPE  T_LLI_VOTCIUTADA_TAB is table of T_LLI_VOTCIUTADA_OBJ;
/
-- ---------------------------------------------------------------------------------------

/*****************************************************************************************
				FUNCIONS DE CONSULTA
*****************************************************************************************/

create or replace FUNCTION LLI_CENSCIUTADA (n_codiciutada  IN  VARCHAR2 )
RETURN T_LLI_CENSCIUTADA_TAB
IS

  /******************************************************************************
   NOM:         LLI_CENSCIUTADA 
   PROPSIT:    Funci que que retorna un llistat dels censos en els quals est 
                donat d'alta un ciutad, indicant
                la segent informaci:
                
                - codi aleatori del ciutad
                - tipus de cens
                - codi del cens
                
                 PARAMETRES:  n_codiciutada,rst
                 INPUT:       n_codiciutada
                 OUTPUT:      rst
   RETORNA:     'OK' si s'ha executat correctament, 'ERROR: Tipus d'Error' en
                cas contrari
   EXEMPLE D'US: SELECT * FROM TABLE(LLI_CENSCIUTADA('AAA000000AA'));
   ******************************************************************************/

   L_LLI_CENSCIUTADA_TAB T_LLI_CENSCIUTADA_TAB:= T_LLI_CENSCIUTADA_TAB();
   n_codiciutada_ins    VARCHAR2 (11);
   n_idciutada_ins      NUMBER;
   campsNuls            EXCEPTION;   
   N                    NUMBER := 0;
   rst                  VARCHAR2(2000);
   err_num              NUMBER;
   err_msg              VARCHAR2(100);
   
  
   CURSOR CUR_LLI_CENSCIUTADA(n_codiciutada VARCHAR2 ) IS
        SELECT ci.codiciutada, ce.idtipuscens, ce.idcodicens 
        FROM ciutada ci join censciutadans ce on ce.idciutada = ci.idciutada
        WHERE ci.codiciutada= n_codiciutada;
        
  PRAGMA AUTONOMOUS_TRANSACTION;
    
BEGIN
    
  --verifica que els camps obligatoris no siguin nuls
  
  IF (n_codiciutada IS NULL)
  THEN
      RAISE campsNuls;
  END IF;
               
  FOR R IN CUR_LLI_CENSCIUTADA(n_codiciutada) LOOP
    L_LLI_CENSCIUTADA_TAB.EXTEND;
    N := N + 1;
    L_LLI_CENSCIUTADA_TAB(N) := T_LLI_CENSCIUTADA_OBJ(R.codiciutada, R.idtipuscens, R.idcodicens);            
  END LOOP;
  
  rst := 'OK';  

   INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'LLI_CENSCIUTADA',
          'codiciutada: ' || n_codiciutada, 
          'RSP: '||rst);   
  
  COMMIT;
  
  RETURN L_LLI_CENSCIUTADA_TAB;

  
EXCEPTION
   WHEN campsNuls
   THEN
     rst := 'ERROR: El codi del ciutada no pot ser nul';

    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'LLI_CENSCIUTADA',
          'codiciutada: ' || n_codiciutada, 
          'RSP: '||rst);   

  WHEN OTHERS THEN
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:'
             || err_msg;

     rst:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'LLI_CENSCIUTADA',
          'codiciutada: ' || n_codiciutada, 
          'RSP: '||rst);   

     COMMIT;   
  
END;     
/
-- ---------------------------------------------------------------------------------------------

create or replace FUNCTION LLI_MENYSDIFVOTS 
RETURN T_LLI_MENYSDIFVOTS_TAB
IS

  /******************************************************************************
   NOMBRE:      LLI_MENYSDIFVOTS 
   PROPSIT:    Funci que que retorna un llistat de les 10 votacions on hi ha 
                hagut menys diferncia de vots entre la opci ms votada i la menys 
                votada, indicant la segent informaci:
                
                - ttol de la votaci.
                - data d'inici de la votaci.
                - data de finalitzaci de la votaci.
                - opci guanyadora.
                - percentatge de vots de la opci guanyadora.
                - opci menys votada.
                - percentatge de la opci menys votada.
                - diferncia de percentatge
                
                 PARAMETRES:  rst
                 INPUT:       cap
                 OUTPUT:      rst
   RETORNA:     'OK' si s'ha executat correctament, 'ERROR: Tipus d'Error' en
                cas contrari
   EXEMPLE D'US: SELECT * FROM TABLE(LLI_MENYSDIFVOTS());
   ******************************************************************************/

   L_LLI_MENYSDIFVOTS_TAB T_LLI_MENYSDIFVOTS_TAB:= T_LLI_MENYSDIFVOTS_TAB();
   N                 NUMBER := 0;
   rst               VARCHAR2(2000);
   err_msg           VARCHAR2(100);
   
  
   CURSOR CUR_LLI_MENYSDIFVOTS IS
        
         SELECT * 
        FROM (SELECT v.titol, v.datainici, v.datafi, vt.opcioguanyadora, vt.popguanyadora, 
                     vt.opmenysvotada, vt.popmenysvotada,
                     (vt.popguanyadora-vt.popmenysvotada) AS diferencia
              FROM  votacio v  JOIN votaciotancada vt ON v.idvotacio = vt.idvotacio 
              where v.estatvotacio=3
              ORDER BY diferencia ASC)
        WHERE ROWNUM <10;
         
         
        
  PRAGMA AUTONOMOUS_TRANSACTION;
    
BEGIN
    
               
  FOR R IN CUR_LLI_MENYSDIFVOTS LOOP
    L_LLI_MENYSDIFVOTS_TAB.EXTEND;
    N := N + 1;
    L_LLI_MENYSDIFVOTS_TAB(N) := T_LLI_MENYSDIFVOTS_OBJ(R.titol, R.datainici, R.datafi, R.opcioguanyadora, 
                                                    R.popguanyadora, R.opmenysvotada,
                                                    R.popmenysvotada, R.diferencia);            
  END LOOP;
  
  rst := 'OK';  

   INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'LLI_MENYSDIFVOTS',
          'No necessita parmetres d entrada ' , 
          'RSP: '||rst);   
  
  COMMIT;
  
  RETURN L_LLI_MENYSDIFVOTS_TAB;

  
EXCEPTION
 
  WHEN OTHERS THEN
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:'
             || err_msg;

     rst:= 'ERROR : '||err_msg;

     INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'LLI_MENYSDIFVOTS',
          'No necessita parmetres d entrada ' , 
          'RSP: '||rst);   
     COMMIT;   
  
END;     
/
-- -------------------------------------------------------------------------------------

create or replace FUNCTION LLI_MESDIFVOTS 
RETURN T_LLI_MESDIFVOTS_TAB
IS

  /******************************************************************************
   NOMBRE:      LLI_MESDIFVOTS 
   PROPSIT:    Funci que que retorna un llistat de les 10 votacions on hi ha 
                hagut ms diferncia de vots entre la opci ms votada i la menys 
                votada, indicant la segent informaci:
                
                - ttol de la votaci.
                - data d'inici de la votaci.
                - data de finalitzaci de la votaci.
                - opci guanyadora.
                - percentatge de vots de la opci guanyadora.
                - opci menys votada.
                - percentatge de la opci menys votada.
                - diferncia de percentatge
                
                 PARAMETRES:  rst
                 INPUT:       cap
                 OUTPUT:      rst
   RETORNA:     'OK' si s'ha executat correctament, 'ERROR: Tipus d'Error' en
                cas contrari
   EXEMPLE D'US: SELECT * FROM TABLE(LLI_MESDIFVOTS());
   ******************************************************************************/

   L_LLI_MESDIFVOTS_TAB T_LLI_MESDIFVOTS_TAB:= T_LLI_MESDIFVOTS_TAB();
   N                 NUMBER := 0;
   rst               VARCHAR2(2000);
   err_msg           VARCHAR2(100);
   
  
   CURSOR CUR_LLI_MESDIFVOTS IS
        
         SELECT * 
        FROM (SELECT v.titol, v.datainici, v.datafi, vt.opcioguanyadora, vt.popguanyadora, 
                     vt.opmenysvotada, vt.popmenysvotada,
                     (vt.popguanyadora-vt.popmenysvotada) AS diferencia
              FROM  votacio v  JOIN votaciotancada vt ON v.idvotacio = vt.idvotacio 
              where v.estatvotacio=3
              ORDER BY diferencia DESC)
        WHERE ROWNUM <10;
         
         
        
  PRAGMA AUTONOMOUS_TRANSACTION;
    
BEGIN
    
               
  FOR R IN CUR_LLI_MESDIFVOTS LOOP
    L_LLI_MESDIFVOTS_TAB.EXTEND;
    N := N + 1;
    L_LLI_MESDIFVOTS_TAB(N) := T_LLI_MESDIFVOTS_OBJ(R.titol, R.datainici, R.datafi, R.opcioguanyadora, 
                                                    R.popguanyadora, R.opmenysvotada,
                                                    R.popmenysvotada, R.diferencia);            
  END LOOP;
  
  rst := 'OK';  

   INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'LLI_MESDIFVOTS',
          'No necessita parmetres d entrada ' , 
          'RSP: '||rst);   
  
  COMMIT;
  
  RETURN L_LLI_MESDIFVOTS_TAB;

  
EXCEPTION
 
  WHEN OTHERS THEN
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:'
             || err_msg;

     rst:= 'ERROR : '||err_msg;

     INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'LLI_MESDIFVOTS',
          'No necessita parmetres d entrada ' , 
          'RSP: '||rst);   
     COMMIT;   
  
END;     
/     
-- --------------------------------------------------------------------------------------------

create or replace FUNCTION LLI_VOT_PAIS (n_idpais    IN  NUMBER)
RETURN T_LLI_VOT_PAIS_TAB
IS

  /******************************************************************************
   NOMBRE:      LLI_VOT_PAIS 
   PROPSIT:    Funci que que retorna un llistat de les votacions que s'han 
                realitzat en un pas i en tots els seus censos associats, indicant
                la segent informaci:
                
                - ttol de la votaci
                - data d'inici de la votaci
                - data de finalitzaci de la votaci
                - nmero total de ciutadans convocats a la votaci.
                - estat de la votaci.
                - percentatge de participaci a la votaci.
                - opci guanyadora.
                - percentatge de vots de la opci guanyadora.
                
                 PARAMETRES:  n_idpais,rst
                 INPUT:       n_idpais
                 OUTPUT:      rst
   RETORNA:     'OK' si s'ha executat correctament, 'ERROR: Tipus d'Error' en
                cas contrari
   EXEMPLE D'US: SELECT * FROM TABLE(LLI_VOT_PAIS(10));
   ******************************************************************************/

   L_LLI_VOT_PAIS_TAB T_LLI_VOT_PAIS_TAB:= T_LLI_VOT_PAIS_TAB();
   n_codipais_ins    VARCHAR2 (2);
   campsNuls         EXCEPTION;   
   N                 NUMBER := 0;
   rst               VARCHAR2(2000);
   err_num           NUMBER;
   err_msg           VARCHAR2(100);
   
  
   CURSOR CUR_LLI_VOT_PAIS(n_idpais NUMBER) IS
        SELECT v.titol, v.datainici, v.datafi,  v.numtotalsciutadans, v.estatvotacio, to_char(vt.pparticipacio)as percentatgeparticipacio, 
        to_char(vt.opcioguanyadora)as opcioguanyadora, to_char(vt.popguanyadora) as percentatgeopcioguanyadora
         FROM votacio v left join votaciotancada vt on v.idvotacio = vt.idvotacio
         where v.idvotacio IN
              (select idvotacio
                from ambit where idtipuscens=1
                OR (idtipuscens=2 AND codicens=n_idpais)
                OR (idtipuscens=3 AND codicens IN (select idregio from censregional where idpais= n_codipais_ins))
                OR (idtipuscens=4 AND codicens IN (select idlocal from censlocal where idpais= n_codipais_ins))
                group by idvotacio)
        
        ORDER BY v.datainici;
        
  PRAGMA AUTONOMOUS_TRANSACTION;
    
BEGIN
    
  --verifica que els camps obligatoris no siguin nuls
  
  IF (n_idpais IS NULL)
  THEN
      RAISE campsNuls;
  END IF;
  
   -- busquem el codi del pais
   BEGIN   
      SELECT codipais
         INTO n_codipais_ins
         FROM CENSESTATAL
         WHERE idpais = n_idpais;

    END;

                
                
  FOR R IN CUR_LLI_VOT_PAIS(n_idpais) LOOP
    L_LLI_VOT_PAIS_TAB.EXTEND;
    N := N + 1;
    L_LLI_VOT_PAIS_TAB(N) := T_LLI_VOT_PAIS_OBJ(R.titol, R.datainici, R.datafi, R.numtotalsciutadans,
                                          R.estatvotacio, NVL (R.percentatgeparticipacio,'-'), NVL (R.opcioguanyadora, '-'), 
                                          NVL( R.percentatgeopcioguanyadora, '-'));            
  END LOOP;
  
  rst := 'OK';  

   INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'LLI_VOT_PAIS',
          'idPais: ' || n_idpais, 
          'RSP: '||rst);   
  
  COMMIT;
  
  RETURN L_LLI_VOT_PAIS_TAB;

  
EXCEPTION
   WHEN campsNuls
   THEN
     rst := 'ERROR: LA id del pas no pot ser nulla)';

    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'LLI_VOT_PAIS',
          'idPais: ' || n_idpais, 
          'RSP: '||rst);   

  WHEN OTHERS THEN
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:'
             || err_msg;

     rst:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'LLI_VOT_PAIS',
          'idPais: ' || n_idpais, 
          'RSP: '||rst);   

     COMMIT;   
  
END;     
/     
-- --------------------------------------------------------------------------------------------

create or replace FUNCTION LLI_VOTCIUTADA (n_codiciutada  IN  VARCHAR2 )
RETURN T_LLI_VOTCIUTADA_TAB
IS

  /******************************************************************************
   NOM:         LLI_VOTCIUTADA 
   PROPSIT:    Funci que que retorna un llistat deles votacions a les cuals ha
                estat convocat un ciutad, indicant
                la segent informaci:
                
                - codi aleatori del ciutad
                - id de la votaci
                - titol de la votaci
                - data d'inici de la votaci
                - data de fi de la votaci
                - participaci (0 si no ha participat i 1 si ha pasticipat)
                
                
                 PARAMETRES:  n_codiciutada,rst
                 INPUT:       n_codiciutada
                 OUTPUT:      rst
   RETORNA:     'OK' si s'ha executat correctament, 'ERROR: Tipus d'Error' en
                cas contrari
   EXEMPLE D'US: SELECT * FROM TABLE(LLI_VOTCIUTADA('AAA000000AA'));
   ******************************************************************************/

   L_LLI_VOTCIUTADA_TAB T_LLI_VOTCIUTADA_TAB:= T_LLI_VOTCIUTADA_TAB();
   campsNuls            EXCEPTION;   
   N                    NUMBER := 0;
   rst                  VARCHAR2(2000);
   err_num              NUMBER;
   err_msg              VARCHAR2(100);
   
  
   CURSOR CUR_LLI_VOTCIUTADA(n_codiciutada VARCHAR2 ) IS
        SELECT ci.codiciutada, vc.idvotacio, v.titol,v.datainici, v.datafi, vc.participacio
        FROM ciutada ci JOIN votacionsciutada vc ON ci.idciutada= vc.idciutada 
                        JOIN votacio v ON vc.idvotacio= v.idvotacio
        WHERE ci.codiciutada=n_codiciutada;
        
  PRAGMA AUTONOMOUS_TRANSACTION;
    
BEGIN
    
  --verifica que els camps obligatoris no siguin nuls
  
  IF (n_codiciutada IS NULL)
  THEN
      RAISE campsNuls;
  END IF;
               
  FOR R IN CUR_LLI_VOTCIUTADA(n_codiciutada) LOOP
    L_LLI_VOTCIUTADA_TAB.EXTEND;
    N := N + 1;
    L_LLI_VOTCIUTADA_TAB(N) := T_LLI_VOTCIUTADA_OBJ(R.codiciutada, R.idvotacio, R.titol,R.datainici, 
                                                    R.datafi, R.participacio);            
  END LOOP;
  
  rst := 'OK';  

   INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'LLI_VOTCIUTADA',
          'codiciutada: ' || n_codiciutada, 
          'RSP: '||rst);   
  
  COMMIT;
  
  RETURN L_LLI_VOTCIUTADA_TAB;

  
EXCEPTION
   WHEN campsNuls
   THEN
     rst := 'ERROR: El codi del ciutada no pot ser nul';

    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'LLI_VOTCIUTADA',
          'codiciutada: ' || n_codiciutada, 
          'RSP: '||rst);   

  WHEN OTHERS THEN
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:'
             || err_msg;

     rst:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'LLI_VOTCIUTADA',
          'codiciutada: ' || n_codiciutada, 
          'RSP: '||rst);   

     COMMIT;   
  
END;     
/
-- -----------------------------------------------------------------------------------

/*************************************************************************************
				ALTRES FUNCIONS     
*************************************************************************************/

create or replace FUNCTION CODICIUTADA_IDCIUTADA (n_codiciutada  IN  VARCHAR2 )
RETURN NUMBER
IS

  /******************************************************************************
   NOM:         CODICIUTADA_IDCIUTADA 
   PROPSIT:    Funci que que retorna la id d'un ciutada a partir del seu codi alfanumric
                
                 PARAMETRES:  n_codiciutada,rst
                 INPUT:       n_codiciutada
                 OUTPUT:      n_idciutada
   RETORNA:     'OK' si s'ha executat correctament, 'ERROR: Tipus d'Error' en
                cas contrari
   EXEMPLE D'US: CODIPAIS_IDPAIS('AAA000000AA');
   ******************************************************************************/
 
   n_idciutada          NUMBER;
   campsNuls            EXCEPTION;   
   rst                  VARCHAR2(2000);
   err_msg              VARCHAR2(100);
   
    
BEGIN
    
  --verifica que els camps obligatoris no siguin nuls
  
  IF (n_codiciutada IS NULL)
  THEN
      RAISE campsNuls;
  END IF;
               
 SELECT idciutada
 INTO n_idciutada
 FROM ciutada
 WHERE codiciutada=n_codiciutada;
  
  rst := 'OK';  

   INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'CODICIUTADA_IDCIUTADA',
          'codiciutada: ' || n_codiciutada, 
          'RSP: '||rst); 
  COMMIT;
  
  RETURN n_idciutada;

  
EXCEPTION
   WHEN campsNuls
   THEN
     rst := 'ERROR: El codi del ciutada no pot ser nul';

    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'CODICIUTADA_IDCIUTADA',
          'codiciutada: ' || n_codiciutada, 
          'RSP: '||rst);
    COMMIT;

  WHEN OTHERS THEN
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:'
             || err_msg;

     rst:= 'ERROR '||err_msg;

    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'CODICIUTADA_IDCIUTADA',
          'codiciutada: ' || n_codiciutada, 
          'RSP: '||rst); 
    COMMIT;   
  
END;    
/
-- ---------------------------------------------------------------------------------

create or replace FUNCTION CODIPAIS_IDPAIS (n_codipais  IN  VARCHAR2 )
RETURN NUMBER
IS

  /******************************************************************************
   NOM:         CODIPAIS_IDPAIS 
   PROPSIT:    Funci que que retorna la id d'un pas a partir del seu codi alfanumric
                
                 PARAMETRES:  n_codipais,rst
                 INPUT:       n_codipais
                 OUTPUT:      n_idpais
   RETORNA:     'OK' si s'ha executat correctament, 'ERROR: Tipus d'Error' en
                cas contrari
   EXEMPLE D'US: CODIPAIS_IDPAIS('ES');
   ******************************************************************************/
 
   n_idpais             NUMBER;
   campsNuls            EXCEPTION;   
   rst                  VARCHAR2(2000);
   err_msg              VARCHAR2(100);
   err_num              NUMBER;
    
BEGIN
    
  --verifica que els camps obligatoris no siguin nuls
  
  IF (n_codipais IS NULL)
  THEN
      RAISE campsNuls;
  END IF;
               
 Select idpais
 into n_idpais
 from censestatal
 where codipais=n_codipais;
  
  rst := 'OK';  

   INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'CODIPAIS_IDPAIS',
          'codiciutada: ' || n_codipais, 
          'RSP: '||rst);   
  
  COMMIT;
  
  RETURN n_idpais;

  
EXCEPTION
   WHEN campsNuls
   THEN
     rst := 'ERROR: El codi del pais no pot ser nul';

    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'CODIPAIS_IDPAIS',
          'codipais: ' || n_codipais, 
          'RSP: '||rst);   

  WHEN OTHERS THEN
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:'
             || err_msg;

     rst:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'CODIPAIS_IDPAIS',
          'codipais: ' || n_codipais, 
          'RSP: '||rst);   

     COMMIT;   
  
END;     
/     
-- ----------------------------------------------------------------------------------

create or replace FUNCTION IDCIUTADA_CODICIUTADA (n_idciutada  IN  NUMBER )
RETURN VARCHAR2
IS

  /******************************************************************************
   NOM:         IDCIUTADA_CODICIUTADA 
   PROPSIT:    Funci  que retorna el codi alfanumric d'un ciutada a partir de la seva id
                
                 PARAMETRES:  n_idciutada,rst
                 INPUT:       n_idciutada
                 OUTPUT:      n_codiciutada
   RETORNA:     'OK' si s'ha executat correctament, 'ERROR: Tipus d'Error' en
                cas contrari
   EXEMPLE D'US: CODIPAIS_IDPAIS('AAA000000AA');
   ******************************************************************************/
 
   n_codiciutada        VARCHAR2(11);
   campsNuls            EXCEPTION;   
   rst                  VARCHAR2(2000);
   err_msg              VARCHAR2(100);
   
    
BEGIN
    
  --verifica que els camps obligatoris no siguin nuls
  
  IF (n_idciutada IS NULL)
  THEN
      RAISE campsNuls;
  END IF;
               
 SELECT codiciutada
 INTO n_codiciutada
 FROM ciutada
 WHERE idciutada=n_idciutada;
  
  rst := 'OK';  

   INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'IDCIUTADA_CODICIUTADA',
          'idciutada: ' || n_idciutada, 
          'RSP: '||rst);   
   COMMIT;
  
  RETURN n_codiciutada;

  
EXCEPTION
   WHEN campsNuls
   THEN
     rst := 'ERROR: El id del ciutad no pot ser nul';

    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'IDCIUTADA_CODICIUTADA',
          'idciutada: ' || n_idciutada, 
          'RSP: '||rst);   
   COMMIT;

  WHEN OTHERS THEN
      err_msg := SUBSTR (SQLERRM, 1, 100);
      rst :=    'ERROR:'
             || err_msg;

     rst:= 'ERROR '||err_msg;

    INSERT INTO Logs (datalog,nomprocediment, paramE, paramS)
    VALUES (SYSDATE,'IDCIUTADA_CODICIUTADA',
          'idciutada: ' || n_idciutada, 
          'RSP: '||rst);   
   COMMIT;
  
END;    
/
-- -----------------------------------------------------------------------------------
     
     









